@@ -289,9 +289,9 @@ static void chest_trap(int y, int x, s16b o_idx)
 	if (trap & (CHEST_POISON))
 	{
 		msg_print("A puff of green gas surrounds you!");
-		if (!(p_ptr->resist_pois || p_ptr->oppose_pois))
+		if (!(p_ptr->resist_pois || p_ptr->timed[TMD_OPP_POIS]))
 		{
-			(void)set_poisoned(p_ptr->poisoned + 10 + randint(20));
+			(void)inc_timed(TMD_POISONED, 10 + randint(20));
 		}
 	}
 
@@ -301,7 +301,7 @@ static void chest_trap(int y, int x, s16b o_idx)
 		msg_print("A puff of yellow gas surrounds you!");
 		if (!p_ptr->free_act)
 		{
-			(void)set_paralyzed(p_ptr->paralyzed + 10 + randint(20));
+			(void)inc_timed(TMD_PARALYZED, 10 + randint(20));
 		}
 	}
 
@@ -353,11 +353,11 @@ static bool do_cmd_open_chest(int y, int x, s16b o_idx)
 		flag = FALSE;
 
 		/* Get the "disarm" factor */
-		i = p_ptr->skill_dis;
+		i = p_ptr->skills[SKILL_DIS];
 
 		/* Penalize some conditions */
-		if (p_ptr->blind || no_lite()) i = i / 10;
-		if (p_ptr->confused || p_ptr->image) i = i / 10;
+		if (p_ptr->timed[TMD_BLIND] || no_lite()) i = i / 10;
+		if (p_ptr->timed[TMD_CONFUSED] || p_ptr->timed[TMD_IMAGE]) i = i / 10;
 
 		/* Extract the difficulty */
 		j = i - o_ptr->pval;
@@ -391,6 +391,9 @@ static bool do_cmd_open_chest(int y, int x, s16b o_idx)
 
 		/* Let the Chest drop items */
 		chest_death(y, x, o_idx);
+
+		/* Squelch chest if autosquelch calls for it */
+		p_ptr->notice |= PN_SQUELCH;
 	}
 
 	/* Result */
@@ -415,11 +418,11 @@ static bool do_cmd_disarm_chest(int y, int x, s16b o_idx)
 
 
 	/* Get the "disarm" factor */
-	i = p_ptr->skill_dis;
+	i = p_ptr->skills[SKILL_DIS];
 
 	/* Penalize some conditions */
-	if (p_ptr->blind || no_lite()) i = i / 10;
-	if (p_ptr->confused || p_ptr->image) i = i / 10;
+	if (p_ptr->timed[TMD_BLIND] || no_lite()) i = i / 10;
+	if (p_ptr->timed[TMD_CONFUSED] || p_ptr->timed[TMD_IMAGE]) i = i / 10;
 
 	/* Extract the difficulty */
 	j = i - o_ptr->pval;
@@ -665,11 +668,11 @@ static bool do_cmd_open_aux(int y, int x)
 	else if (cave_feat[y][x] >= FEAT_DOOR_HEAD + 0x01)
 	{
 		/* Disarm factor */
-		i = p_ptr->skill_dis;
+		i = p_ptr->skills[SKILL_DIS];
 
 		/* Penalize some conditions */
-		if (p_ptr->blind || no_lite()) i = i / 10;
-		if (p_ptr->confused || p_ptr->image) i = i / 10;
+		if (p_ptr->timed[TMD_BLIND] || no_lite()) i = i / 10;
+		if (p_ptr->timed[TMD_CONFUSED] || p_ptr->timed[TMD_IMAGE]) i = i / 10;
 
 		/* Extract the lock power */
 		j = cave_feat[y][x] - FEAT_DOOR_HEAD;
@@ -1081,7 +1084,7 @@ static bool do_cmd_tunnel_aux(int y, int x)
 	else if (cave_feat[y][x] >= FEAT_WALL_EXTRA)
 	{
 		/* Tunnel */
-		if ((p_ptr->skill_dig > 40 + rand_int(1600)) && twall(y, x))
+		if ((p_ptr->skills[SKILL_DIG] > 40 + rand_int(1600)) && twall(y, x))
 		{
 			msg_print("You have finished the tunnel.");
 		}
@@ -1117,13 +1120,13 @@ static bool do_cmd_tunnel_aux(int y, int x)
 		/* Quartz */
 		if (hard)
 		{
-			okay = (p_ptr->skill_dig > 20 + rand_int(800));
+			okay = (p_ptr->skills[SKILL_DIG] > 20 + rand_int(800));
 		}
 
 		/* Magma */
 		else
 		{
-			okay = (p_ptr->skill_dig > 10 + rand_int(400));
+			okay = (p_ptr->skills[SKILL_DIG] > 10 + rand_int(400));
 		}
 
 		/* Success */
@@ -1168,7 +1171,7 @@ static bool do_cmd_tunnel_aux(int y, int x)
 	else if (cave_feat[y][x] == FEAT_RUBBLE)
 	{
 		/* Remove the rubble */
-		if ((p_ptr->skill_dig > rand_int(200)) && twall(y, x))
+		if ((p_ptr->skills[SKILL_DIG] > rand_int(200)) && twall(y, x))
 		{
 			/* Message */
 			msg_print("You have removed the rubble.");
@@ -1179,8 +1182,9 @@ static bool do_cmd_tunnel_aux(int y, int x)
 				/* Create a simple object */
 				place_object(y, x, FALSE, FALSE);
 
-				/* Observe new object */
-				if (player_can_see_bold(y, x))
+				/* Observe the new object */
+				if (!squelch_hide_item(&o_list[cave_o_idx[y][x]]) &&
+				    player_can_see_bold(y, x))
 				{
 					msg_print("You have found something!");
 				}
@@ -1199,7 +1203,7 @@ static bool do_cmd_tunnel_aux(int y, int x)
 	else if (cave_feat[y][x] >= FEAT_SECRET)
 	{
 		/* Tunnel */
-		if ((p_ptr->skill_dig > 30 + rand_int(1200)) && twall(y, x))
+		if ((p_ptr->skills[SKILL_DIG] > 30 + rand_int(1200)) && twall(y, x))
 		{
 			msg_print("You have finished the tunnel.");
 		}
@@ -1220,7 +1224,7 @@ static bool do_cmd_tunnel_aux(int y, int x)
 	else
 	{
 		/* Tunnel */
-		if ((p_ptr->skill_dig > 30 + rand_int(1200)) && twall(y, x))
+		if ((p_ptr->skills[SKILL_DIG] > 30 + rand_int(1200)) && twall(y, x))
 		{
 			msg_print("You have finished the tunnel.");
 		}
@@ -1366,11 +1370,11 @@ static bool do_cmd_disarm_aux(int y, int x)
 	name = (f_name + f_info[cave_feat[y][x]].name);
 
 	/* Get the "disarm" factor */
-	i = p_ptr->skill_dis;
+	i = p_ptr->skills[SKILL_DIS];
 
 	/* Penalize some conditions */
-	if (p_ptr->blind || no_lite()) i = i / 10;
-	if (p_ptr->confused || p_ptr->image) i = i / 10;
+	if (p_ptr->timed[TMD_BLIND] || no_lite()) i = i / 10;
+	if (p_ptr->timed[TMD_CONFUSED] || p_ptr->timed[TMD_IMAGE]) i = i / 10;
 
 	/* XXX XXX XXX Variable power? */
 
@@ -1635,7 +1639,7 @@ static bool do_cmd_bash_aux(int y, int x)
 		msg_print("You are off-balance.");
 
 		/* Hack -- Lose balance ala paralysis */
-		(void)set_paralyzed(p_ptr->paralyzed + 2 + rand_int(2));
+		(void)inc_timed(TMD_PARALYZED, 2 + rand_int(2));
 	}
 
 	/* Result */
@@ -1912,7 +1916,7 @@ static bool do_cmd_spike_test(int y, int x)
  */
 void do_cmd_spike(void)
 {
-	int y, x, dir, item;
+	int y, x, dir, item = 0;
 
 
 	/* Get a spike */
@@ -1993,7 +1997,7 @@ void do_cmd_spike(void)
 /*
  * Determine if a given grid may be "walked"
  */
-static bool do_cmd_walk_test(int y, int x)
+bool do_cmd_walk_test(int y, int x)
 {
 	/* Hack -- walking obtains knowledge XXX XXX */
 	if (!(cave_info[y][x] & (CAVE_MARK))) return (TRUE);
@@ -2041,13 +2045,12 @@ static bool do_cmd_walk_test(int y, int x)
 
 
 /*
- * Helper function for the "walk" and "jump" commands.
+ * Helper function for the "walk" command.
  */
-static void do_cmd_walk_or_jump(int jumping)
+void do_cmd_walk(void)
 {
 	int y, x, dir;
 
-
 	/* Get a direction (or abort) */
 	if (!get_rep_dir(&dir)) return;
 
@@ -2090,30 +2093,34 @@ static void do_cmd_walk_or_jump(int jumping)
 	}
 
 	/* Move the player */
-	move_player(dir, jumping);
+	move_player(dir);
 }
 
-
 /*
- * Walk into a grid.
- */
-void do_cmd_walk(void)
-{
-	/* Move (normal) */
-	do_cmd_walk_or_jump(FALSE);
-}
-
-
-/*
- * Jump into a grid.
+ * Jump into a trap, turn off pickup.
+ *
+ * What a horrible function.
  */
 void do_cmd_jump(void)
 {
-	/* Move (jump) */
-	do_cmd_walk_or_jump(TRUE);
+	bool old_easy_alter;
+
+	/* Picking up NOT okay, so whatever you heard is obviously wrong */
+	p_ptr->auto_pickup_okay = FALSE;
+
+	/* easy_alter can be turned off (don't disarm traps) */
+	old_easy_alter = easy_alter;
+	easy_alter = FALSE;
+
+	/* Now actually do this silly walk */
+	do_cmd_walk();
+
+	/* Restore easy_alter */
+	easy_alter = old_easy_alter;
 }
 
 
+
 /*
  * Start running.
  *
@@ -2125,7 +2132,7 @@ void do_cmd_run(void)
 
 
 	/* Hack XXX XXX XXX */
-	if (p_ptr->confused)
+	if (p_ptr->timed[TMD_CONFUSED])
 	{
 		msg_print("You are too confused!");
 		return;
@@ -2148,13 +2155,37 @@ void do_cmd_run(void)
 	run_step(dir);
 }
 
+/*
+ * Start running with pathfinder.
+ *
+ * Note that running while confused is not allowed.
+ */
+void do_cmd_pathfind(int y, int x)
+{
+	/* Hack XXX XXX XXX */
+	if (p_ptr->timed[TMD_CONFUSED])
+	{
+		msg_print("You are too confused!");
+		return;
+	}
+
+	if (findpath(y, x))
+	{
+		p_ptr->running = 1000;
+		/* Calculate torch radius */
+		p_ptr->update |= (PU_TORCH);
+		p_ptr->running_withpathfind = TRUE;
+		run_step(0);
+	}
+}
+
 
 
 /*
  * Stay still.  Search.  Enter stores.
  * Pick up treasure if "pickup" is true.
  */
-static void do_cmd_hold_or_stay(int pickup)
+void do_cmd_hold(void)
 {
 	/* Allow repeated command */
 	if (p_ptr->command_arg)
@@ -2173,7 +2204,7 @@ static void do_cmd_hold_or_stay(int pickup)
 	p_ptr->energy_use = 100;
 
 	/* Spontaneous Searching */
-	if ((p_ptr->skill_fos >= 50) || (0 == rand_int(50 - p_ptr->skill_fos)))
+	if ((p_ptr->skills[SKILL_FOS] >= 50) || (0 == rand_int(50 - p_ptr->skills[SKILL_FOS])))
 	{
 		search();
 	}
@@ -2184,8 +2215,8 @@ static void do_cmd_hold_or_stay(int pickup)
 		search();
 	}
 
-	/* Handle "objects" */
-	py_pickup(pickup);
+	/* Handle objects now.  XXX XXX XXX */
+	p_ptr->energy_use += py_pickup(0) * 10;
 
 	/* Hack -- enter a store if we are on one */
 	if ((cave_feat[p_ptr->py][p_ptr->px] >= FEAT_SHOP_HEAD) &&
@@ -2203,26 +2234,26 @@ static void do_cmd_hold_or_stay(int pickup)
 }
 
 
-/*
- * Hold still (usually pickup)
- */
-void do_cmd_hold(void)
-{
-	/* Hold still (usually pickup) */
-	do_cmd_hold_or_stay(always_pickup);
-}
-
 
 /*
- * Stay still (usually do not pickup)
+ * Pick up objects on the floor beneath you.  -LM-
  */
-void do_cmd_stay(void)
+void do_cmd_pickup(void)
 {
-	/* Stay still (usually do not pickup) */
-	do_cmd_hold_or_stay(!always_pickup);
+	int energy_cost;
+
+	/* Pick up floor objects, forcing a menu for multiple objects. */
+	energy_cost = py_pickup(1) * 10;
+
+	/* Maximum time expenditure is a full turn. */
+	if (energy_cost > 100) energy_cost = 100;
+
+	/* Charge this amount of energy. */
+	p_ptr->energy_use = energy_cost;
 }
 
 
+
 /*
  * Rest (restores hit points and mana and such)
  */
@@ -2233,10 +2264,7 @@ void do_cmd_rest(void)
 	{
 		cptr p = "Rest (0-9999, '*' for HP/SP, '&' as needed): ";
 
-		char out_val[5];
-
-		/* Default */
-		strcpy(out_val, "&");
+		char out_val[5] = "& ";
 
 		/* Ask for duration */
 		if (!get_string(p, out_val, sizeof(out_val))) return;
@@ -2288,7 +2316,7 @@ void do_cmd_rest(void)
 	handle_stuff();
 
 	/* Refresh XXX XXX XXX */
-	if (fresh_before) Term_fresh();
+	Term_fresh();
 }
 
 
@@ -2480,7 +2508,7 @@ void do_cmd_fire(void)
 
 	/* Actually "fire" the object */
 	bonus = (p_ptr->to_h + i_ptr->to_h + j_ptr->to_h);
-	chance = (p_ptr->skill_thb + (bonus * BTH_PLUS_ADJ));
+	chance = (p_ptr->skills[SKILL_THB] + (bonus * BTH_PLUS_ADJ));
 
 	/* Assume a base multiplier */
 	tmul = p_ptr->ammo_mult;
@@ -2537,19 +2565,15 @@ void do_cmd_fire(void)
 			/* Visual effects */
 			print_rel(missile_char, missile_attr, y, x);
 			move_cursor_relative(y, x);
-			if (fresh_before)
-			{
-				Term_fresh();
-				if (p_ptr->window) window_stuff();
-			}
+
+			Term_fresh();
+			if (p_ptr->window) window_stuff();
 
 			Term_xtra(TERM_XTRA_DELAY, msec);
 			lite_spot(y, x);
-			if (fresh_before)
-			{
-				Term_fresh();
-				if (p_ptr->window) window_stuff();
-			}
+
+			Term_fresh();
+			if (p_ptr->window) window_stuff();
 		}
 
 		/* Delay anyway for consistency */
@@ -2779,7 +2803,7 @@ void do_cmd_throw(void)
 	tdam = damroll(i_ptr->dd, i_ptr->ds) + i_ptr->to_d;
 
 	/* Chance of hitting */
-	chance = (p_ptr->skill_tht + (p_ptr->to_h * BTH_PLUS_ADJ));
+	chance = (p_ptr->skills[SKILL_THT] + (p_ptr->to_h * BTH_PLUS_ADJ));
 
 
 	/* Take a turn */
@@ -2827,18 +2851,15 @@ void do_cmd_throw(void)
 			/* Visual effects */
 			print_rel(missile_char, missile_attr, y, x);
 			move_cursor_relative(y, x);
-			if (fresh_before)
-			{
-				Term_fresh();
-				if (p_ptr->window) window_stuff();
-			}
+
+			Term_fresh();
+			if (p_ptr->window) window_stuff();
+
 			Term_xtra(TERM_XTRA_DELAY, msec);
 			lite_spot(y, x);
-			if (fresh_before)
-			{
-				Term_fresh();
-				if (p_ptr->window) window_stuff();
-			}
+
+			Term_fresh();
+			if (p_ptr->window) window_stuff();
 		}
 
 		/* Delay anyway for consistency */
@@ -2957,3 +2978,52 @@ void do_cmd_throw(void)
 	/* Drop (or break) near that location */
 	drop_near(i_ptr, j, y, x);
 }
+
+
+/*
+ * See if one can squelch a given kind of item.
+ */
+static bool squelchable_hook(const object_type *o_ptr)
+{
+	object_kind *k_ptr = &k_info[o_ptr->k_idx];
+
+	/* No point in double-squelching things */
+	if (k_ptr->squelch) return FALSE;
+
+	/* Don't squelch bad tvals */
+	if (!squelch_tval(o_ptr->tval)) return FALSE;
+
+	/* Only allow if aware */
+	return object_aware_p(o_ptr);
+}
+
+
+
+/*
+ * Mark item as "squelch".
+ */
+void do_cmd_mark_squelch()
+{
+	const char *q = "Squelch which item kind? ";
+	const char *s = "You have nothing you can squelch.";
+
+	object_type *o_ptr;
+	object_kind *k_ptr;
+	int item;
+
+	/* Get an item */
+	item_tester_hook = squelchable_hook;
+	if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
+
+	/* Get the object */
+	if (item >= 0)
+		o_ptr = &inventory[item];
+	else
+		o_ptr = &o_list[0 - item];
+
+	/* Get object kind */
+	k_ptr = &k_info[o_ptr->k_idx];
+
+	/* Set squelch flag */
+	k_ptr->squelch = TRUE;
+}
