@@ -132,11 +132,14 @@ void do_cmd_wield(void)
 	object_type *i_ptr;
 	object_type object_type_body;
 
+	object_type *equip_o_ptr;
+
 	cptr act;
 
 	cptr q, s;
 
 	char o_name[80];
+	char out_val[160];
 
 
 	/* Restrict the choices */
@@ -163,11 +166,14 @@ void do_cmd_wield(void)
 	/* Check the slot */
 	slot = wield_slot(o_ptr);
 
+	/* Get a pointer to the slot to be removed */
+	equip_o_ptr = &inventory[slot];
+
 	/* Prevent wielding into a cursed slot */
-	if (cursed_p(&inventory[slot]))
+	if (cursed_p(equip_o_ptr))
 	{
 		/* Describe it */
-		object_desc(o_name, sizeof(o_name), &inventory[slot], FALSE, 0);
+		object_desc(o_name, sizeof(o_name), equip_o_ptr, FALSE, 0);
 
 		/* Message */
 		msg_format("The %s you are %s appears to be cursed.",
@@ -177,6 +183,33 @@ void do_cmd_wield(void)
 		return;
 	}
 
+	/* Double-check taking off an item with "!t" */
+	if (equip_o_ptr->note)
+	{
+		/* Find a '!' */
+		s = strchr(quark_str(equip_o_ptr->note), '!');
+
+		/* Process preventions */
+		/* XXX Perhaps this should be factored out to a separate function? */
+		while (s)
+		{
+			/* Check the "restriction" */
+			if (s[1] == 't')
+			{
+				/* Describe it */
+				object_desc(o_name, sizeof(o_name), equip_o_ptr, TRUE, 3);
+
+				/* Prompt */
+				strnfmt(out_val, sizeof(out_val), "Really take off %s? ", o_name);
+
+				/* Forget it */
+				if (!get_check(out_val)) return;
+			}
+
+			/* Find another '!' */
+			s = strchr(s + 1, '!');
+		}
+	}
 
 	/* Take a turn */
 	p_ptr->energy_use = 100;
@@ -255,14 +288,14 @@ void do_cmd_wield(void)
 		sound(MSG_CURSED);
 		msg_print("Oops! It feels deathly cold!");
 
-		/* Remove special inscription, if any */
-		if (o_ptr->discount >= INSCRIP_NULL) o_ptr->discount = 0;
-
-		/* Sense the object if allowed */
-		if (o_ptr->discount == 0) o_ptr->discount = INSCRIP_CURSED;
+		/* Sense the object */
+		o_ptr->pseudo = INSCRIP_CURSED;
 
 		/* The object has been "sensed" */
 		o_ptr->ident |= (IDENT_SENSE);
+
+		/* Set squelched status */
+		p_ptr->notice |= PN_SQUELCH;
 	}
 
 	/* Recalculate bonuses */
@@ -398,16 +431,21 @@ void do_cmd_destroy(void)
 	object_type object_type_body;
 
 	char o_name[120];
-
 	char out_val[160];
 
 	cptr q, s;
 
-
 	/* Get an item */
 	q = "Destroy which item? ";
 	s = "You have nothing to destroy.";
-	if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
+	if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR | CAN_SQUELCH))) return;
+
+	/* Deal with squelched items */
+	if (item == ALL_SQUELCHED)
+	{
+		squelch_items();
+		return;
+	}
 
 	/* Get the item (in the pack) */
 	if (item >= 0)
@@ -448,14 +486,8 @@ void do_cmd_destroy(void)
 	object_desc(o_name, sizeof(o_name), i_ptr, TRUE, 3);
 
 	/* Verify destruction */
-	if (verify_destroy)
-	{
-		strnfmt(out_val, sizeof(out_val), "Really destroy %s? ", o_name);
-		if (!get_check(out_val)) return;
-	}
-
-	/* Take a turn */
-	p_ptr->energy_use = 100;
+	strnfmt(out_val, sizeof(out_val), "Really destroy %s? ", o_name);
+	if (!get_check(out_val)) return; 
 
 	/* Artifacts cannot be destroyed */
 	if (artifact_p(o_ptr))
@@ -471,14 +503,14 @@ void do_cmd_destroy(void)
 		{
 			/* Already sensed objects always get improved feelings */
 			if (cursed_p(o_ptr) || broken_p(o_ptr))
-				o_ptr->discount = INSCRIP_TERRIBLE;
+				o_ptr->pseudo = INSCRIP_TERRIBLE;
 			else
-				o_ptr->discount = INSCRIP_SPECIAL;
+				o_ptr->pseudo = INSCRIP_SPECIAL;
 		}
 		else
 		{
 			/* Mark the object as indestructible */
-			o_ptr->discount = INSCRIP_INDESTRUCTIBLE;
+			o_ptr->pseudo = INSCRIP_INDESTRUCTIBLE;
 		}
 
 		/* Combine the pack */
@@ -514,6 +546,21 @@ void do_cmd_destroy(void)
 		floor_item_describe(0 - item);
 		floor_item_optimize(0 - item);
 	}
+
+#if 0
+	/*
+	 * We can only re-enable this when it can be made to interact well with
+	 * the repeat code.
+	 */
+
+	/* We have destroyed a floor item, and the floor is not empty */
+	if ((item < 0) && (cave_o_idx[p_ptr->py][p_ptr->px]))
+	{
+		/* Automatically repeat this command (unless disturbed) */
+		p_ptr->command_cmd = 'k';
+		p_ptr->command_rep = 2;
+	}
+#endif
 }
 
 
@@ -595,8 +642,8 @@ void do_cmd_uninscribe(void)
 	/* Remove the inscription */
 	o_ptr->note = 0;
 
-	/* Combine the pack */
-	p_ptr->notice |= (PN_COMBINE);
+	/* Combine the pack, check for squelchables */
+	p_ptr->notice |= (PN_COMBINE | PN_SQUELCH);
 
 	/* Window stuff */
 	p_ptr->window |= (PW_INVEN | PW_EQUIP);
@@ -644,7 +691,7 @@ void do_cmd_inscribe(void)
 	message_flush();
 
 	/* Start with nothing */
-	strcpy(tmp, "");
+	tmp[0] = '\0';
 
 	/* Use old inscription */
 	if (o_ptr->note)
@@ -659,8 +706,8 @@ void do_cmd_inscribe(void)
 		/* Save the inscription */
 		o_ptr->note = quark_add(tmp);
 
-		/* Combine the pack */
-		p_ptr->notice |= (PN_COMBINE);
+		/* Combine the pack, check for squelchables */
+		p_ptr->notice |= (PN_COMBINE | PN_SQUELCH);
 
 		/* Window stuff */
 		p_ptr->window |= (PW_INVEN | PW_EQUIP);
@@ -674,13 +721,19 @@ void do_cmd_inscribe(void)
  */
 static bool item_tester_refill_lantern(const object_type *o_ptr)
 {
+	u32b f1, f2, f3;
+
+	/* Get flags */
+	object_flags(o_ptr, &f1, &f2, &f3);
+
 	/* Flasks of oil are okay */
 	if (o_ptr->tval == TV_FLASK) return (TRUE);
 
-	/* Non-empty lanterns are okay */
+	/* Non-empty, non-everburning lanterns are okay */
 	if ((o_ptr->tval == TV_LITE) &&
 	    (o_ptr->sval == SV_LITE_LANTERN) &&
-	    (o_ptr->pval > 0))
+	    (o_ptr->timeout > 0) &&
+		!(f3 & TR3_NO_FUEL))
 	{
 		return (TRUE);
 	}
@@ -731,19 +784,19 @@ static void do_cmd_refill_lamp(void)
 	j_ptr = &inventory[INVEN_LITE];
 
 	/* Refuel */
-	j_ptr->pval += o_ptr->pval;
+	j_ptr->timeout += o_ptr->timeout ? o_ptr->timeout : o_ptr->pval;
 
 	/* Message */
 	msg_print("You fuel your lamp.");
 
 	/* Comment */
-	if (j_ptr->pval >= FUEL_LAMP)
+	if (j_ptr->timeout >= FUEL_LAMP)
 	{
-		j_ptr->pval = FUEL_LAMP;
+		j_ptr->timeout = FUEL_LAMP;
 		msg_print("Your lamp is full.");
 	}
 
-	/* Refilled from a latern */
+	/* Refilled from a lantern */
 	if (o_ptr->sval == SV_LITE_LANTERN)
 	{
 		/* Unstack if necessary */
@@ -762,7 +815,7 @@ static void do_cmd_refill_lamp(void)
 			i_ptr->number = 1;
 
 			/* Remove fuel */
-			i_ptr->pval = 0;
+			i_ptr->timeout = 0;
 
 			/* Unstack the used item */
 			o_ptr->number--;
@@ -775,11 +828,11 @@ static void do_cmd_refill_lamp(void)
 				drop_near(i_ptr, 0, p_ptr->py, p_ptr->px);
 		}
 
-		/* Empty a single latern */
+		/* Empty a single lantern */
 		else
 		{
 			/* No more fuel */
-			o_ptr->pval = 0;
+			o_ptr->timeout = 0;
 		}
 
 		/* Combine / Reorder the pack (later) */
@@ -823,9 +876,18 @@ static void do_cmd_refill_lamp(void)
  */
 static bool item_tester_refill_torch(const object_type *o_ptr)
 {
+	u32b f1, f2, f3;
+
+	/* Get flags */
+	object_flags(o_ptr, &f1, &f2, &f3);
+
 	/* Torches are okay */
 	if ((o_ptr->tval == TV_LITE) &&
-	    (o_ptr->sval == SV_LITE_TORCH)) return (TRUE);
+	    (o_ptr->sval == SV_LITE_TORCH) &&
+		!(f3 & TR3_NO_FUEL))
+	{
+		return (TRUE);
+	}
 
 	/* Assume not okay */
 	return (FALSE);
@@ -873,15 +935,15 @@ static void do_cmd_refill_torch(void)
 	j_ptr = &inventory[INVEN_LITE];
 
 	/* Refuel */
-	j_ptr->pval += o_ptr->pval + 5;
+	j_ptr->timeout += o_ptr->timeout + 5;
 
 	/* Message */
 	msg_print("You combine the torches.");
 
 	/* Over-fuel message */
-	if (j_ptr->pval >= FUEL_TORCH)
+	if (j_ptr->timeout >= FUEL_TORCH)
 	{
-		j_ptr->pval = FUEL_TORCH;
+		j_ptr->timeout = FUEL_TORCH;
 		msg_print("Your torch is fully fueled.");
 	}
 
@@ -923,26 +985,34 @@ static void do_cmd_refill_torch(void)
 void do_cmd_refill(void)
 {
 	object_type *o_ptr;
+	u32b f1, f2, f3;
 
 	/* Get the light */
 	o_ptr = &inventory[INVEN_LITE];
 
+	/* Get flags */
+	object_flags(o_ptr, &f1, &f2, &f3);
+
+
 	/* It is nothing */
 	if (o_ptr->tval != TV_LITE)
 	{
 		msg_print("You are not wielding a light.");
 	}
 
-	/* It's a lamp */
-	else if (o_ptr->sval == SV_LITE_LANTERN)
+	else if (!(f3 & TR3_NO_FUEL))
 	{
-		do_cmd_refill_lamp();
-	}
+		/* It's a lamp */
+		if (o_ptr->sval == SV_LITE_LANTERN)
+		{
+			do_cmd_refill_lamp();
+		}
 
-	/* It's a torch */
-	else if (o_ptr->sval == SV_LITE_TORCH)
-	{
-		do_cmd_refill_torch();
+		/* It's a torch */
+		else if (o_ptr->sval == SV_LITE_TORCH)
+		{
+			do_cmd_refill_torch();
+		}
 	}
 
 	/* No torch to refill */
@@ -1321,17 +1391,17 @@ void do_cmd_query_symbol(void)
 	if (sym == KTRL('A'))
 	{
 		all = TRUE;
-		strcpy(buf, "Full monster list.");
+		my_strcpy(buf, "Full monster list.", sizeof(buf));
 	}
 	else if (sym == KTRL('U'))
 	{
 		all = uniq = TRUE;
-		strcpy(buf, "Unique monster list.");
+		my_strcpy(buf, "Unique monster list.", sizeof(buf));
 	}
 	else if (sym == KTRL('N'))
 	{
 		all = norm = TRUE;
-		strcpy(buf, "Non-unique monster list.");
+		my_strcpy(buf, "Non-unique monster list.", sizeof(buf));
 	}
 	else if (ident_info[i])
 	{
@@ -1484,20 +1554,14 @@ void do_cmd_query_symbol(void)
 		if (query == '-')
 		{
 			if (++i == n)
-			{
 				i = 0;
-				if (!expand_list) break;
-			}
 		}
 
 		/* Move to "next" monster */
 		else
 		{
 			if (i-- == 0)
-			{
 				i = n - 1;
-				if (!expand_list) break;
-			}
 		}
 	}
 
