@@ -2875,6 +2875,7 @@ static void cave_gen(void)
 		by = rand_int(dun->row_rooms);
 		bx = rand_int(dun->col_rooms);
 
+#if 0
 		/* Align dungeon rooms */
 		if (dungeon_align)
 		{
@@ -2884,6 +2885,7 @@ static void cave_gen(void)
 			/* Slide some rooms left */
 			if ((bx % 3) == 2) bx--;
 		}
+#endif
 
 		/* Destroyed levels are boring */
 		if (destroyed)
@@ -3408,8 +3410,8 @@ void generate_cave(void)
 
 
 		/* Hack -- illegal panel */
-		p_ptr->wy = DUNGEON_HGT;
-		p_ptr->wx = DUNGEON_WID;
+		Term->offset_y = DUNGEON_HGT;
+		Term->offset_x = DUNGEON_WID;
 
 
 		/* Reset the monster generation level */
@@ -3425,22 +3427,18 @@ void generate_cave(void)
 		rating = 0;
 
 
-		/* Event -- generate level */
-		if (!generate_level_hook(p_ptr->depth))
+		/* Build the town */
+		if (!p_ptr->depth)
 		{
-			/* Build the town */
-			if (!p_ptr->depth)
-			{
-				/* Make a town */
-				town_gen();
-			}
+			/* Make a town */
+			town_gen();
+		}
 
-			/* Build a real level */
-			else
-			{
-				/* Make a dungeon */
-				cave_gen();
-			}
+		/* Build a real level */
+		else
+		{
+			/* Make a dungeon */
+			cave_gen();
 		}
 
 		/* Extract the feeling */
@@ -3454,8 +3452,10 @@ void generate_cave(void)
 		else if (rating > 0) feeling = 9;
 		else feeling = 10;
 
+#if 0
 		/* Hack -- Have a special feeling sometimes */
-		if (good_item_flag && !adult_preserve) feeling = 1;
+		if (good_item_flag && adult_no_preserve) feeling = 1;
+#endif
 
 		/* It takes 1000 game turns for "feelings" to recharge */
 		if (((turn - old_turn) < 1000) && (old_turn > 1)) feeling = 0;
@@ -3485,7 +3485,7 @@ void generate_cave(void)
 		}
 
 		/* Mega-Hack -- "auto-scum" */
-		if (auto_scum && (num < 100))
+		if (adult_autoscum && (num < 100))
 		{
 			/* Require "goodness" */
 			if ((feeling > 9) ||
