@@ -39,6 +39,9 @@
 typedef struct header header;
 
 typedef errr (*parse_info_txt_func)(char *buf, header *head);
+typedef errr (*eval_info_power_func)(header *head);
+typedef errr (*emit_info_txt_index_func)(FILE *fp, header *head, int i);
+typedef errr (*emit_info_txt_always_func)(FILE *fp, header *head);
 
 /*
  * Template file header information (see "init.c").  16 bytes.
@@ -93,11 +96,20 @@ struct header
 	char *text_ptr;
 
 	parse_info_txt_func parse_info_txt;
+	eval_info_power_func eval_info_power;	/* Evaluate after parsing */
+	emit_info_txt_index_func emit_info_txt_index;
+	emit_info_txt_always_func emit_info_txt_always;
+	
 };
 
 extern errr init_info_txt(FILE *fp, char *buf, header *head,
                           parse_info_txt_func parse_info_txt_line);
 
+extern errr eval_info(eval_info_power_func eval_info_process, header *head);
+
+extern errr emit_info_txt(FILE *fp, FILE *template, char *buf, header *head,
+   emit_info_txt_index_func emit_info_txt_index, emit_info_txt_always_func emit_info_txt_always);
+
 #ifdef ALLOW_TEMPLATES
 extern errr parse_z_info(char *buf, header *head);
 extern errr parse_v_info(char *buf, header *head);
@@ -112,6 +124,16 @@ extern errr parse_h_info(char *buf, header *head);
 extern errr parse_b_info(char *buf, header *head);
 extern errr parse_g_info(char *buf, header *head);
 extern errr parse_flavor_info(char *buf, header *head);
+extern errr parse_s_info(char *buf, header *head);
+
+#ifdef ALLOW_TEMPLATES_PROCESS
+extern errr eval_r_power(header *head);
+#endif
+
+#ifdef ALLOW_TEMPLATES_OUTPUT
+extern errr emit_r_info_index(FILE *fp, header *head, int i);
+#endif
+
 
 /*
  * Error tracking
