@@ -123,7 +123,7 @@ static term_data data[MAX_XXX_TERM];
  * Often, the "Term_xtra(TERM_XTRA_REACT, 0)" hook is used to initialize
  * "color_data" from "angband_color_table".  XXX XXX XXX
  */
-static local_color_data_type color_data[256];
+static local_color_data_type color_data[MAX_COLORS];
 
 #endif
 
@@ -326,21 +326,6 @@ static errr Term_xtra_xxx(int n, int v)
 			return (0);
 		}
 
-		case TERM_XTRA_SOUND:
-		{
-			/*
-			 * Make a sound XXX XXX XXX
-			 *
-			 * This action should produce sound number "v", where the
-			 * "name" of that sound is "sound_names[v]".  This method
-			 * is still under construction.
-			 *
-			 * This action is optional, and not very important.
-			 */
-
-			return (0);
-		}
-
 		case TERM_XTRA_BORED:
 		{
 			/*
@@ -358,7 +343,7 @@ static errr Term_xtra_xxx(int n, int v)
 			 * React to global changes XXX XXX XXX
 			 *
 			 * For example, this action can be used to react to
-			 * changes in the global "angband_color_table[256][4]" array.
+			 * changes in the global "angband_color_table[MAX_COLORS][4]" array.
 			 *
 			 * This action is optional, but can be very useful for
 			 * handling "color changes" and the "arg_sound" and/or
@@ -490,7 +475,7 @@ static errr Term_wipe_xxx(int x, int y, int n)
  * you must first call "Term_wipe_xxx()" to clear the area.
  *
  * In color environments, you should activate the color contained
- * in "color_data[a & 0x0F]", if needed, before drawing anything.
+ * in "color_data[a & BASIC_COLORS]", if needed, before drawing anything.
  *
  * You may ignore the "attribute" if you are only supporting a
  * monochrome environment, since this routine is normally never
@@ -645,7 +630,7 @@ static void term_data_link(int i)
  *   1st line = max 68 chars.
  *   Start next lines with 11 spaces, as in main-xaw.c.
  */
-cptr help_xxx = "Describe XXX, subopts -describe suboptions here";
+const char help_xxx[] = "Describe XXX, subopts -describe suboptions here";
 
 
 /*
@@ -708,6 +693,21 @@ static bool CheckEvents(bool wait)
 }
 
 
+
+/*
+ * Make a sound.
+ *
+ * This action should produce sound number "v", where the
+ * "name" of that sound is "sound_names[v]".
+ *
+ * This action is optional, and not very important.
+ */
+static void xxx_sound(int v)
+{
+	return;
+}
+
+
 /*
  * Init some stuff
  *
@@ -721,10 +721,18 @@ static void init_stuff(void)
 	/* This must in some way prepare the "path" variable */
 	/* so that it points at the "lib" directory.  Every */
 	/* machine handles this in a different way... */
-	strcpy(path, "XXX XXX XXX");
+	my_strcpy(path, "XXX XXX XXX", sizeof(path));
 
 	/* Prepare the filepaths */
 	init_file_paths(path);
+
+
+#ifdef USE_SOUND
+
+	/* Set up sound hook */
+	sound_hook = xxx_sound;
+
+#endif /* USE_SOUND */
 }
 
 
