@@ -13,6 +13,9 @@
 
 #include "angband.h"
 
+extern errr init_sound_sdl(int argc, char **argv);
+
+
 extern errr init_lfb(int argc, char **argv);
 extern errr init_gtk(int argc, char **argv);
 extern errr init_xaw(int argc, char **argv);
@@ -28,6 +31,7 @@ extern errr init_lsl(int argc, char **argv);
 extern errr init_ami(int argc, char **argv);
 extern errr init_vme(int argc, char **argv);
 extern errr init_vcs(int argc, char **argv);
+extern errr init_sdl(int argc, char **argv);
 
 
 extern const char help_lfb[];
@@ -45,6 +49,7 @@ extern const char help_sla[];
 extern const char help_emx[];
 extern const char help_ibm[];
 extern const char help_dos[];
+extern const char help_sdl[];
 
 
 struct module
