@@ -82,11 +82,11 @@ static void remove_bad_spells(int m_idx, u32b *f4p, u32b *f5p, u32b *f6p)
 
 
 	/* Must be cheating or learning */
-	if (!smart_cheat && !smart_learn) return;
+	if (!adult_ai_cheat && !adult_ai_learn) return;
 
 
 	/* Update acquired knowledge */
-	if (smart_learn)
+	if (adult_ai_learn)
 	{
 		/* Hack -- Occasionally forget player status */
 		if (m_ptr->smart && (rand_int(100) < 1)) m_ptr->smart = 0L;
@@ -97,7 +97,7 @@ static void remove_bad_spells(int m_idx, u32b *f4p, u32b *f5p, u32b *f6p)
 
 
 	/* Cheat if requested */
-	if (smart_cheat)
+	if (adult_ai_cheat)
 	{
 		/* Know weirdness */
 		if (p_ptr->free_act) smart |= (SM_IMM_FREE);
@@ -110,11 +110,11 @@ static void remove_bad_spells(int m_idx, u32b *f4p, u32b *f5p, u32b *f6p)
 		if (p_ptr->immune_cold) smart |= (SM_IMM_COLD);
 
 		/* Know oppositions */
-		if (p_ptr->oppose_acid) smart |= (SM_OPP_ACID);
-		if (p_ptr->oppose_elec) smart |= (SM_OPP_ELEC);
-		if (p_ptr->oppose_fire) smart |= (SM_OPP_FIRE);
-		if (p_ptr->oppose_cold) smart |= (SM_OPP_COLD);
-		if (p_ptr->oppose_pois) smart |= (SM_OPP_POIS);
+		if (p_ptr->timed[TMD_OPP_ACID]) smart |= (SM_OPP_ACID);
+		if (p_ptr->timed[TMD_OPP_ELEC]) smart |= (SM_OPP_ELEC);
+		if (p_ptr->timed[TMD_OPP_FIRE]) smart |= (SM_OPP_FIRE);
+		if (p_ptr->timed[TMD_OPP_COLD]) smart |= (SM_OPP_COLD);
+		if (p_ptr->timed[TMD_OPP_POIS]) smart |= (SM_OPP_POIS);
 
 		/* Know resistances */
 		if (p_ptr->resist_acid) smart |= (SM_RES_ACID);
@@ -479,7 +479,7 @@ static int choose_attack_spell(int m_idx, u32b f4, u32b f5, u32b f6)
 
 
 	/* Smart monsters restrict their spell choices. */
-	if (smart_monsters && !(r_ptr->flags2 & (RF2_STUPID)))
+	if (adult_ai_smart && !(r_ptr->flags2 & (RF2_STUPID)))
 	{
 		/* What have we got? */
 		has_escape = ((f4 & (RF4_ESCAPE_MASK)) ||
@@ -695,14 +695,12 @@ bool make_attack_spell(int m_idx)
 
 	char ddesc[80];
 
-	bool no_innate = FALSE;
-
 	/* Summon count */
 	int count = 0;
 
 
 	/* Extract the blind-ness */
-	bool blind = (p_ptr->blind ? TRUE : FALSE);
+	bool blind = (p_ptr->timed[TMD_BLIND] ? TRUE : FALSE);
 
 	/* Extract the "see-able-ness" */
 	bool seen = (!blind && m_ptr->ml);
@@ -727,30 +725,10 @@ bool make_attack_spell(int m_idx)
 	/* Not allowed to cast spells */
 	if (!chance) return (FALSE);
 
-#ifdef MONSTER_AI
-
-	if (!smart_monsters)
-	{
-		/* Only do spells occasionally */
-		if (rand_int(100) >= chance) return (FALSE);
-	}
-	else
-	{
-		/* Do spells more often, because they can fail */
-		if (rand_int(100) >= 2 * chance) return (FALSE);
-
-		/* Sometimes forbid innate attacks (breaths) */
-		if (rand_int(100) >= chance) no_innate = TRUE;
-	}
-
-#else /* MONSTER_AI */
 
 	/* Only do spells occasionally */
 	if (rand_int(100) >= chance) return (FALSE);
 
-#endif /* MONSTER_AI */
-
-
 
 	/* Hack -- require projectable player */
 	if (normal)
@@ -773,17 +751,6 @@ bool make_attack_spell(int m_idx)
 	f6 = r_ptr->flags6;
 
 
-#ifdef MONSTER_AI
-
-	/* Forbid innate attacks sometimes */
-	if (no_innate)
-	{
-		f4 &= ~(RF4_INNATE_MASK);
-		f5 &= ~(RF5_INNATE_MASK);
-		f6 &= ~(RF6_INNATE_MASK);
-	}
-
-#endif /* MONSTER_AI */
 
 	/* Hack -- allow "desperate" spells */
 	if ((r_ptr->flags2 & (RF2_SMART)) &&
@@ -813,7 +780,7 @@ bool make_attack_spell(int m_idx)
 #ifdef MONSTER_AI
 
 	/* Check whether summons and bolts are worth it. */
-	if (smart_monsters && !(r_ptr->flags2 & (RF2_STUPID)))
+	if (adult_ai_smart && !(r_ptr->flags2 & (RF2_STUPID)))
 	{
 		/* Check for a clean bolt shot */
 		if ((f4 & (RF4_BOLT_MASK) ||
@@ -866,7 +833,7 @@ bool make_attack_spell(int m_idx)
 	failrate = 25 - (rlev + 3) / 4;
 
 	/* Hack -- Stupid monsters will never fail (for jellies and such) */
-	if (!smart_monsters || r_ptr->flags2 & (RF2_STUPID)) failrate = 0;
+	if (!adult_ai_smart || r_ptr->flags2 & (RF2_STUPID)) failrate = 0;
 
 	/* Check for spell failure (innate attacks never fail) */
 	if ((thrown_spell >= RF5_OFFSET) && (rand_int(100) < failrate))
@@ -1407,7 +1374,7 @@ bool make_attack_spell(int m_idx)
 				msg_format("%^s gazes deep into your eyes.", m_name);
 			}
 
-			if (rand_int(100) < p_ptr->skill_sav)
+			if (rand_int(100) < p_ptr->skills[SKILL_SAV])
 			{
 				msg_print("You resist the effects!");
 			}
@@ -1416,7 +1383,7 @@ bool make_attack_spell(int m_idx)
 				msg_print("Your mind is blasted by psionic energy.");
 				if (!p_ptr->resist_confu)
 				{
-					(void)set_confused(p_ptr->confused + rand_int(4) + 4);
+					(void)inc_timed(TMD_CONFUSED, rand_int(4) + 4);
 				}
 				take_hit(damroll(8, 8), ddesc);
 			}
@@ -1436,7 +1403,7 @@ bool make_attack_spell(int m_idx)
 			{
 				msg_format("%^s looks deep into your eyes.", m_name);
 			}
-			if (rand_int(100) < p_ptr->skill_sav)
+			if (rand_int(100) < p_ptr->skills[SKILL_SAV])
 			{
 				msg_print("You resist the effects!");
 			}
@@ -1446,17 +1413,17 @@ bool make_attack_spell(int m_idx)
 				take_hit(damroll(12, 15), ddesc);
 				if (!p_ptr->resist_blind)
 				{
-					(void)set_blind(p_ptr->blind + 8 + rand_int(8));
+					(void)inc_timed(TMD_BLIND, 8 + rand_int(8));
 				}
 				if (!p_ptr->resist_confu)
 				{
-					(void)set_confused(p_ptr->confused + rand_int(4) + 4);
+					(void)inc_timed(TMD_CONFUSED, rand_int(4) + 4);
 				}
 				if (!p_ptr->free_act)
 				{
-					(void)set_paralyzed(p_ptr->paralyzed + rand_int(4) + 4);
+					(void)inc_timed(TMD_PARALYZED, rand_int(4) + 4);
 				}
-				(void)set_slow(p_ptr->slow + rand_int(4) + 4);
+				(void)inc_timed(TMD_SLOW, rand_int(4) + 4);
 			}
 			break;
 		}
@@ -1468,7 +1435,7 @@ bool make_attack_spell(int m_idx)
 			disturb(1, 0);
 			if (blind) msg_format("%^s mumbles.", m_name);
 			else msg_format("%^s points at you and curses.", m_name);
-			if (rand_int(100) < p_ptr->skill_sav)
+			if (rand_int(100) < p_ptr->skills[SKILL_SAV])
 			{
 				msg_print("You resist the effects!");
 			}
@@ -1486,7 +1453,7 @@ bool make_attack_spell(int m_idx)
 			disturb(1, 0);
 			if (blind) msg_format("%^s mumbles.", m_name);
 			else msg_format("%^s points at you and curses horribly.", m_name);
-			if (rand_int(100) < p_ptr->skill_sav)
+			if (rand_int(100) < p_ptr->skills[SKILL_SAV])
 			{
 				msg_print("You resist the effects!");
 			}
@@ -1504,7 +1471,7 @@ bool make_attack_spell(int m_idx)
 			disturb(1, 0);
 			if (blind) msg_format("%^s mumbles loudly.", m_name);
 			else msg_format("%^s points at you, incanting terribly!", m_name);
-			if (rand_int(100) < p_ptr->skill_sav)
+			if (rand_int(100) < p_ptr->skills[SKILL_SAV])
 			{
 				msg_print("You resist the effects!");
 			}
@@ -1522,14 +1489,14 @@ bool make_attack_spell(int m_idx)
 			disturb(1, 0);
 			if (blind) msg_format("%^s screams the word 'DIE!'", m_name);
 			else msg_format("%^s points at you, screaming the word DIE!", m_name);
-			if (rand_int(100) < p_ptr->skill_sav)
+			if (rand_int(100) < p_ptr->skills[SKILL_SAV])
 			{
 				msg_print("You resist the effects!");
 			}
 			else
 			{
 				take_hit(damroll(15, 15), ddesc);
-				(void)set_cut(p_ptr->cut + damroll(10, 10));
+				(void)inc_timed(TMD_CUT, damroll(10, 10));
 			}
 			break;
 		}
@@ -1669,13 +1636,13 @@ bool make_attack_spell(int m_idx)
 			{
 				msg_print("You refuse to be frightened.");
 			}
-			else if (rand_int(100) < p_ptr->skill_sav)
+			else if (rand_int(100) < p_ptr->skills[SKILL_SAV])
 			{
 				msg_print("You refuse to be frightened.");
 			}
 			else
 			{
-				(void)set_afraid(p_ptr->afraid + rand_int(4) + 4);
+				(void)inc_timed(TMD_AFRAID, rand_int(4) + 4);
 			}
 			update_smart_learn(m_idx, DRS_RES_FEAR);
 			break;
@@ -1692,13 +1659,13 @@ bool make_attack_spell(int m_idx)
 			{
 				msg_print("You are unaffected!");
 			}
-			else if (rand_int(100) < p_ptr->skill_sav)
+			else if (rand_int(100) < p_ptr->skills[SKILL_SAV])
 			{
 				msg_print("You resist the effects!");
 			}
 			else
 			{
-				(void)set_blind(12 + rand_int(4));
+				(void)set_timed(TMD_BLIND, 12 + rand_int(4));
 			}
 			update_smart_learn(m_idx, DRS_RES_BLIND);
 			break;
@@ -1715,13 +1682,13 @@ bool make_attack_spell(int m_idx)
 			{
 				msg_print("You disbelieve the feeble spell.");
 			}
-			else if (rand_int(100) < p_ptr->skill_sav)
+			else if (rand_int(100) < p_ptr->skills[SKILL_SAV])
 			{
 				msg_print("You disbelieve the feeble spell.");
 			}
 			else
 			{
-				(void)set_confused(p_ptr->confused + rand_int(4) + 4);
+				(void)inc_timed(TMD_CONFUSED, rand_int(4) + 4);
 			}
 			update_smart_learn(m_idx, DRS_RES_CONFU);
 			break;
@@ -1737,13 +1704,13 @@ bool make_attack_spell(int m_idx)
 			{
 				msg_print("You are unaffected!");
 			}
-			else if (rand_int(100) < p_ptr->skill_sav)
+			else if (rand_int(100) < p_ptr->skills[SKILL_SAV])
 			{
 				msg_print("You resist the effects!");
 			}
 			else
 			{
-				(void)set_slow(p_ptr->slow + rand_int(4) + 4);
+				(void)inc_timed(TMD_SLOW, rand_int(4) + 4);
 			}
 			update_smart_learn(m_idx, DRS_FREE);
 			break;
@@ -1760,13 +1727,13 @@ bool make_attack_spell(int m_idx)
 			{
 				msg_print("You are unaffected!");
 			}
-			else if (rand_int(100) < p_ptr->skill_sav)
+			else if (rand_int(100) < p_ptr->skills[SKILL_SAV])
 			{
 				msg_format("You resist the effects!");
 			}
 			else
 			{
-				(void)set_paralyzed(p_ptr->paralyzed + rand_int(4) + 4);
+				(void)inc_timed(TMD_PARALYZED, rand_int(4) + 4);
 			}
 			update_smart_learn(m_idx, DRS_FREE);
 			break;
@@ -1942,7 +1909,7 @@ bool make_attack_spell(int m_idx)
 			{
 				msg_print("You are unaffected!");
 			}
-			else if (rand_int(100) < p_ptr->skill_sav)
+			else if (rand_int(100) < p_ptr->skills[SKILL_SAV])
 			{
 				msg_print("You resist the effects!");
 			}
@@ -1990,14 +1957,11 @@ bool make_attack_spell(int m_idx)
 			disturb(1, 0);
 			msg_format("%^s tries to blank your mind.", m_name);
 
-			if (rand_int(100) < p_ptr->skill_sav)
-			{
+			if (rand_int(100) < p_ptr->skills[SKILL_SAV])
 				msg_print("You resist the effects!");
-			}
-			else if (lose_all_info())
-			{
-				msg_print("Your memories fade away.");
-			}
+			else
+				inc_timed(TMD_AMNESIA, 3);
+
 			break;
 		}
 
@@ -2366,8 +2330,6 @@ static int mon_will_run(int m_idx)
 {
 	monster_type *m_ptr = &mon_list[m_idx];
 
-#ifdef ALLOW_TERROR
-
 	monster_race *r_ptr = &r_info[m_ptr->r_idx];
 
 	u16b p_lev, m_lev;
@@ -2375,16 +2337,12 @@ static int mon_will_run(int m_idx)
 	u16b m_chp, m_mhp;
 	u32b p_val, m_val;
 
-#endif /* ALLOW_TERROR */
-
 	/* Keep monsters from running too far away */
 	if (m_ptr->cdis > MAX_SIGHT + 5) return (FALSE);
 
 	/* All "afraid" monsters will run away */
 	if (m_ptr->monfear) return (TRUE);
 
-#ifdef ALLOW_TERROR
-
 	/* Nearby monsters will not become terrified */
 	if (m_ptr->cdis <= 5) return (FALSE);
 
@@ -2413,8 +2371,6 @@ static int mon_will_run(int m_idx)
 	/* Strong players scare strong monsters */
 	if (p_val * m_mhp > m_val * p_mhp) return (TRUE);
 
-#endif /* ALLOW_TERROR */
-
 	/* Assume no terror */
 	return (FALSE);
 }
@@ -2460,7 +2416,7 @@ static bool get_moves_aux(int m_idx, int *yp, int *xp)
 	monster_race *r_ptr = &r_info[m_ptr->r_idx];
 
 	/* Monster flowing disabled */
-	if (!flow_by_sound) return (FALSE);
+	if (!adult_ai_sound) return (FALSE);
 
 	/* Monster can go through rocks */
 	if (r_ptr->flags2 & (RF2_PASS_WALL)) return (FALSE);
@@ -2477,7 +2433,7 @@ static bool get_moves_aux(int m_idx, int *yp, int *xp)
 		if (cave_when[y1][x1] == 0) return (FALSE);
 
 		/* The monster is not allowed to track the player */
-		if (!flow_by_smell) return (FALSE);
+		if (!adult_ai_smell) return (FALSE);
 	}
 
 	/* Monster is too far away to notice the player */
@@ -2538,7 +2494,7 @@ static bool get_fear_moves_aux(int m_idx, int *yp, int *xp)
 	monster_race *r_ptr = &r_info[m_ptr->r_idx];
 
 	/* Monster flowing disabled */
-	if (!flow_by_sound) return (FALSE);
+	if (!adult_ai_sound) return (FALSE);
 
 	/* Player location */
 	py = p_ptr->py;
@@ -2809,7 +2765,7 @@ static bool find_safety(int m_idx, int *yp, int *xp)
 			if (!cave_floor_bold(y, x)) continue;
 
 			/* Check for "availability" (if monsters can flow) */
-			if (flow_by_sound)
+			if (adult_ai_sound)
 			{
 				/* Ignore grids very far from the player */
 				if (cave_when[y][x] < cave_when[py][px]) continue;
@@ -2963,7 +2919,7 @@ static bool get_moves(int m_idx, int mm[5])
 #ifdef MONSTER_FLOW
 
 	/* Flow towards the player */
-	if (flow_by_sound)
+	if (adult_ai_sound)
 	{
 		/* Flow towards the player */
 		(void)get_moves_aux(m_idx, &y2, &x2);
@@ -2979,7 +2935,7 @@ static bool get_moves(int m_idx, int mm[5])
 #ifdef MONSTER_AI
 
 	/* Normal animal packs try to get the player out of corridors. */
-	if (smart_packs &&
+	if (adult_ai_packs &&
 	    (r_ptr->flags1 & RF1_FRIENDS) && (r_ptr->flags3 & RF3_ANIMAL) &&
 	    !((r_ptr->flags2 & (RF2_PASS_WALL)) || (r_ptr->flags2 & (RF2_KILL_WALL))))
 	{
@@ -3010,7 +2966,7 @@ static bool get_moves(int m_idx, int mm[5])
 	if (!done && mon_will_run(m_idx))
 	{
 		/* Try to find safe place */
-		if (!(smart_monsters && find_safety(m_idx, &y, &x)))
+		if (!(adult_ai_smart && find_safety(m_idx, &y, &x)))
 		{
 			/* This is not a very "smart" method XXX XXX */
 			y = (-y);
@@ -3023,7 +2979,7 @@ static bool get_moves(int m_idx, int mm[5])
 		else
 		{
 			/* Attempt to avoid the player */
-			if (flow_by_sound)
+			if (adult_ai_sound)
 			{
 				/* Adjust movement */
 				get_fear_moves_aux(m_idx, &y, &x);
@@ -3040,7 +2996,7 @@ static bool get_moves(int m_idx, int mm[5])
 #ifdef MONSTER_AI
 
 	/* Monster groups try to surround the player */
-	if (!done && smart_packs && (r_ptr->flags1 & RF1_FRIENDS))
+	if (!done && adult_ai_packs && (r_ptr->flags1 & RF1_FRIENDS))
 	{
 		int i;
 
@@ -4024,7 +3980,7 @@ static void process_monster(int m_idx)
 							did_take_item = TRUE;
 
 							/* Describe observable situations */
-							if (m_ptr->ml && player_has_los_bold(ny, nx))
+							if (m_ptr->ml && player_has_los_bold(ny, nx) && !squelch_hide_item(o_ptr))
 							{
 								/* Dump a message */
 								msg_format("%^s tries to pick up %s, but fails.",
@@ -4043,7 +3999,7 @@ static void process_monster(int m_idx)
 						did_take_item = TRUE;
 
 						/* Describe observable situations */
-						if (player_has_los_bold(ny, nx))
+						if (player_has_los_bold(ny, nx) && !squelch_hide_item(o_ptr))
 						{
 							/* Dump a message */
 							msg_format("%^s picks up %s.", m_name, o_name);
@@ -4069,7 +4025,7 @@ static void process_monster(int m_idx)
 						did_kill_item = TRUE;
 
 						/* Describe observable situations */
-						if (player_has_los_bold(ny, nx))
+						if (player_has_los_bold(ny, nx) && !squelch_hide_item(o_ptr))
 						{
 							/* Dump a message */
 							message_format(MSG_DESTROY, 0, "%^s crushes %s.", m_name, o_name);
@@ -4088,7 +4044,7 @@ static void process_monster(int m_idx)
 
 
 	/* If we haven't done anything, try casting a spell again */
-	if (smart_monsters && !do_turn && !do_move)
+	if (adult_ai_smart && !do_turn && !do_move)
 	{
 		/* Cast spell */
 		if (make_attack_spell(m_idx)) return;
@@ -4162,7 +4118,7 @@ static void process_monster(int m_idx)
 static bool monster_can_flow(int m_idx)
 {
 	/* Hack -- Monsters can "smell" the player from far away */
-	if (flow_by_sound)
+	if (adult_ai_sound)
 	{
 		monster_type *m_ptr = &mon_list[m_idx];
 		monster_race *r_ptr = &r_info[m_ptr->r_idx];
