@@ -286,43 +286,82 @@ static void compact_objects_aux(int i1, int i2)
 
 
 /*
- * Compact and Reorder the object list
+ * Compact and reorder the object list
  *
  * This function can be very dangerous, use with caution!
  *
- * When actually "compacting" objects, we base the saving throw on a
- * combination of object level, distance from player, and current
- * "desperation".
+ * When compacting objects, we first destroy gold, on the basis that by the
+ * time item compaction becomes an issue, the player really won't care.
+ * We also nuke items marked as squelch.
  *
- * After "compacting" (if needed), we "reorder" the objects into a more
- * compact order, and we reset the allocation info, and the "live" array.
+ * When compacting other objects, we base the saving throw on a combination of
+ * object level, distance from player, and current "desperation".
+ *
+ * After compacting, we "reorder" the objects into a more compact order, and we
+ * reset the allocation info, and the "live" array.
  */
 void compact_objects(int size)
 {
 	int py = p_ptr->py;
 	int px = p_ptr->px;
 
-	int i, y, x, num, cnt;
+	int i, y, x, cnt;
 
 	int cur_lev, cur_dis, chance;
 
 
-	/* Compact */
-	if (size)
+	/* Reorder objects when not passed a size */
+	if (!size)
 	{
-		/* Message */
-		msg_print("Compacting objects...");
+		/* Excise dead objects (backwards!) */
+		for (i = o_max - 1; i >= 1; i--)
+		{
+			object_type *o_ptr = &o_list[i];
 
-		/* Redraw map */
-		p_ptr->redraw |= (PR_MAP);
+			/* Skip real objects */
+			if (o_ptr->k_idx) continue;
 
-		/* Window stuff */
-		p_ptr->window |= (PW_OVERHEAD | PW_MAP);
+			/* Move last object into open hole */
+			compact_objects_aux(o_max - 1, i);
+
+			/* Compress "o_max" */
+			o_max--;
+		}
+
+		return;
+	}
+
+
+	/* Message */
+	msg_print("Compacting objects...");
+
+	/* Redraw map */
+	p_ptr->redraw |= (PR_MAP);
+
+	/* Window stuff */
+	p_ptr->window |= (PW_OVERHEAD | PW_MAP);
+
+
+
+
+	/*** Try destroying objects ***/
+
+	/* First do gold */
+	for (i = 1; (i < o_max) && (size); i++)
+	{
+		object_type *o_ptr = &o_list[i];
+
+		/* Nuke gold or squelched items */
+		if (o_ptr->tval == TV_GOLD || squelch_item_ok(o_ptr))
+		{
+			delete_object_idx(i);
+			size--;
+		}
 	}
 
 
 	/* Compact at least 'size' objects */
-	for (num = 0, cnt = 1; num < size; cnt++)
+	for (cnt = 1; size; cnt++)
 	{
 		/* Get more vicious each iteration */
 		cur_lev = 5 * cnt;
@@ -331,17 +370,17 @@ void compact_objects(int size)
 		cur_dis = 5 * (20 - cnt);
 
 		/* Examine the objects */
-		for (i = 1; i < o_max; i++)
+		for (i = 1; (i < o_max) && (size); i++)
 		{
 			object_type *o_ptr = &o_list[i];
-
 			object_kind *k_ptr = &k_info[o_ptr->k_idx];
 
 			/* Skip dead objects */
 			if (!o_ptr->k_idx) continue;
 
 			/* Hack -- High level objects start out "immune" */
-			if (k_ptr->level > cur_lev) continue;
+			if (k_ptr->level > cur_lev && !k_ptr->squelch)
+				continue;
 
 			/* Monster */
 			if (o_ptr->held_m_idx)
@@ -356,7 +395,8 @@ void compact_objects(int size)
 				x = m_ptr->fx;
 
 				/* Monsters protect their objects */
-				if (rand_int(100) < 90) continue;
+				if ((rand_int(100) < 90) && !k_ptr->squelch)
+					continue;
 			}
 
 			/* Dungeon */
@@ -368,11 +408,13 @@ void compact_objects(int size)
 			}
 
 			/* Nearby objects start out "immune" */
-			if ((cur_dis > 0) && (distance(py, px, y, x) < cur_dis)) continue;
+			if ((cur_dis > 0) && (distance(py, px, y, x) < cur_dis) && !k_ptr->squelch)
+				continue;
 
 			/* Saving throw */
 			chance = 90;
 
+
 			/* Hack -- only compact artifacts in emergencies */
 			if (artifact_p(o_ptr) && (cnt < 1000)) chance = 100;
 
@@ -381,27 +423,13 @@ void compact_objects(int size)
 
 			/* Delete the object */
 			delete_object_idx(i);
-
-			/* Count it */
-			num++;
+			size--;
 		}
 	}
 
 
-	/* Excise dead objects (backwards!) */
-	for (i = o_max - 1; i >= 1; i--)
-	{
-		object_type *o_ptr = &o_list[i];
-
-		/* Skip real objects */
-		if (o_ptr->k_idx) continue;
-
-		/* Move last object into open hole */
-		compact_objects_aux(o_max - 1, i);
-
-		/* Compress "o_max" */
-		o_max--;
-	}
+	/* Reorder objects */
+	compact_objects(0);
 }
 
 
@@ -430,15 +458,12 @@ void wipe_o_list(void)
 		/* Skip dead objects */
 		if (!o_ptr->k_idx) continue;
 
-		/* Mega-Hack -- preserve artifacts */
-		if (!character_dungeon || adult_preserve)
+		/* Preserve artifacts */
+		if (!character_dungeon || !adult_no_preserve)
 		{
-			/* Hack -- Preserve unknown artifacts */
+			/* Preserve only unknown artifacts */
 			if (artifact_p(o_ptr) && !object_known_p(o_ptr))
-			{
-				/* Mega-Hack -- Preserve the artifact */
 				a_info[o_ptr->name1].cur_num = 0;
-			}
 		}
 
 		/* Monster */
@@ -759,7 +784,7 @@ s16b get_obj_num(int level)
 void object_known(object_type *o_ptr)
 {
 	/* Remove special inscription, if any */
-	if (o_ptr->discount >= INSCRIP_NULL) o_ptr->discount = 0;
+	if (o_ptr->pseudo) o_ptr->pseudo = 0;
 
 	/* The object is not "sensed" */
 	o_ptr->ident &= ~(IDENT_SENSE);
@@ -783,7 +808,7 @@ void object_aware(object_type *o_ptr)
 	/* Fully aware of the effects */
 	k_info[o_ptr->k_idx].aware = TRUE;
 
-	/* MEGA-HACK - scrolls can change the graphics when becoming aware */
+	/* Scrolls can change the graphics when becoming aware */
 	if (o_ptr->tval == TV_SCROLL)
 	{
 		/* Redraw map */
@@ -1002,7 +1027,7 @@ static s32b object_value_real(const object_type *o_ptr)
 			break;
 		}
 
-		/* Rings/Amulets */
+		/* Rings/Amulets, Lites */
 		case TV_RING:
 		case TV_AMULET:
 		{
@@ -1137,13 +1162,6 @@ s32b object_value(const object_type *o_ptr)
 	}
 
 
-	/* Apply discount (if any) */
-	if (o_ptr->discount > 0 && o_ptr->discount < INSCRIP_NULL)
-	{
-		value -= (value * o_ptr->discount / 100L);
-	}
-
-
 	/* Return the final value */
 	return (value);
 }
@@ -1271,8 +1289,13 @@ bool object_similar(const object_type *o_ptr, const object_type *j_ptr)
 			/* Hack -- Never stack "powerful" items */
 			if (o_ptr->xtra1 || j_ptr->xtra1) return (FALSE);
 
-			/* Hack -- Never stack recharging items */
-			if (o_ptr->timeout || j_ptr->timeout) return (FALSE);
+			/* Hack - Never stack recharging items */
+			if ((o_ptr->timeout || j_ptr->timeout) && o_ptr->tval != TV_LITE) 
+				return FALSE;
+
+			/* Lites must have same amount of fuel */
+			else if(o_ptr->timeout != j_ptr->timeout && o_ptr->tval == TV_LITE)
+				return FALSE;
 
 			/* Require identical "values" */
 			if (o_ptr->ac != j_ptr->ac) return (FALSE);
@@ -1306,43 +1329,14 @@ bool object_similar(const object_type *o_ptr, const object_type *j_ptr)
 	/* Hack -- Require compatible inscriptions */
 	if (o_ptr->note != j_ptr->note)
 	{
-		/* Normally require matching inscriptions */
-		if (!stack_force_notes) return (0);
-
 		/* Never combine different inscriptions */
 		if (o_ptr->note && j_ptr->note) return (0);
 	}
 
 
-	/* Hack -- Require compatible "discount" fields */
-	if (o_ptr->discount != j_ptr->discount)
-	{
-		/* Both are (different) special inscriptions */
-		if ((o_ptr->discount >= INSCRIP_NULL) &&
-		    (j_ptr->discount >= INSCRIP_NULL))
-		{
-			/* Normally require matching inscriptions */
-			return (0);
-		}
-
-		/* One is a special inscription, one is a discount or nothing */
-		else if ((o_ptr->discount >= INSCRIP_NULL) ||
-		         (j_ptr->discount >= INSCRIP_NULL))
-		{
-			/* Normally require matching inscriptions */
-			if (!stack_force_notes) return (0);
-
-			/* Hack -- Never merge a special inscription with a discount */
-			if ((o_ptr->discount > 0) && (j_ptr->discount > 0)) return (0);
-		}
-
-		/* One is a discount, one is a (different) discount or nothing */
-		else
-		{
-			/* Normally require matching discounts */
-			if (!stack_force_costs) return (0);
-		}
-	}
+	/* Different pseudo-ID statuses preclude combination */
+	if (o_ptr->pseudo != j_ptr->pseudo)
+		return (0);
 
 
 	/* Maximal "stacking" limit */
@@ -1392,7 +1386,7 @@ void object_absorb(object_type *o_ptr, const object_type *j_ptr)
 	if (j_ptr->note != 0) o_ptr->note = j_ptr->note;
 
 	/* Mega-Hack -- Blend "discounts" */
-	if (o_ptr->discount < j_ptr->discount) o_ptr->discount = j_ptr->discount;
+	o_ptr->pseudo = j_ptr->pseudo;
 
 	/*
 	 * Hack -- if rods are stacking, re-calculate the
@@ -1864,87 +1858,6 @@ static bool make_artifact(object_type *o_ptr)
 }
 
 
-/*
- * Charge a new wand.
- */
-static void charge_wand(object_type *o_ptr)
-{
-	switch (o_ptr->sval)
-	{
-		case SV_WAND_HEAL_MONSTER:		o_ptr->pval = randint(20) + 8; break;
-		case SV_WAND_HASTE_MONSTER:		o_ptr->pval = randint(20) + 8; break;
-		case SV_WAND_CLONE_MONSTER:		o_ptr->pval = randint(5)  + 3; break;
-		case SV_WAND_TELEPORT_AWAY:		o_ptr->pval = randint(5)  + 6; break;
-		case SV_WAND_DISARMING:			o_ptr->pval = randint(5)  + 4; break;
-		case SV_WAND_TRAP_DOOR_DEST:	o_ptr->pval = randint(8)  + 6; break;
-		case SV_WAND_STONE_TO_MUD:		o_ptr->pval = randint(4)  + 3; break;
-		case SV_WAND_LITE:				o_ptr->pval = randint(10) + 6; break;
-		case SV_WAND_SLEEP_MONSTER:		o_ptr->pval = randint(15) + 8; break;
-		case SV_WAND_SLOW_MONSTER:		o_ptr->pval = randint(10) + 6; break;
-		case SV_WAND_CONFUSE_MONSTER:	o_ptr->pval = randint(12) + 6; break;
-		case SV_WAND_FEAR_MONSTER:		o_ptr->pval = randint(5)  + 3; break;
-		case SV_WAND_DRAIN_LIFE:		o_ptr->pval = randint(3)  + 3; break;
-		case SV_WAND_POLYMORPH:			o_ptr->pval = randint(8)  + 6; break;
-		case SV_WAND_STINKING_CLOUD:	o_ptr->pval = randint(8)  + 6; break;
-		case SV_WAND_MAGIC_MISSILE:		o_ptr->pval = randint(10) + 6; break;
-		case SV_WAND_ACID_BOLT:			o_ptr->pval = randint(8)  + 6; break;
-		case SV_WAND_ELEC_BOLT:			o_ptr->pval = randint(8)  + 6; break;
-		case SV_WAND_FIRE_BOLT:			o_ptr->pval = randint(8)  + 6; break;
-		case SV_WAND_COLD_BOLT:			o_ptr->pval = randint(5)  + 6; break;
-		case SV_WAND_ACID_BALL:			o_ptr->pval = randint(5)  + 2; break;
-		case SV_WAND_ELEC_BALL:			o_ptr->pval = randint(8)  + 4; break;
-		case SV_WAND_FIRE_BALL:			o_ptr->pval = randint(4)  + 2; break;
-		case SV_WAND_COLD_BALL:			o_ptr->pval = randint(6)  + 2; break;
-		case SV_WAND_WONDER:			o_ptr->pval = randint(15) + 8; break;
-		case SV_WAND_ANNIHILATION:		o_ptr->pval = randint(2)  + 1; break;
-		case SV_WAND_DRAGON_FIRE:		o_ptr->pval = randint(3)  + 1; break;
-		case SV_WAND_DRAGON_COLD:		o_ptr->pval = randint(3)  + 1; break;
-		case SV_WAND_DRAGON_BREATH:		o_ptr->pval = randint(3)  + 1; break;
-	}
-}
-
-
-
-/*
- * Charge a new staff.
- */
-static void charge_staff(object_type *o_ptr)
-{
-	switch (o_ptr->sval)
-	{
-		case SV_STAFF_DARKNESS:			o_ptr->pval = randint(8)  + 8; break;
-		case SV_STAFF_SLOWNESS:			o_ptr->pval = randint(8)  + 8; break;
-		case SV_STAFF_HASTE_MONSTERS:	o_ptr->pval = randint(8)  + 8; break;
-		case SV_STAFF_SUMMONING:		o_ptr->pval = randint(3)  + 1; break;
-		case SV_STAFF_TELEPORTATION:	o_ptr->pval = randint(4)  + 5; break;
-		case SV_STAFF_IDENTIFY:			o_ptr->pval = randint(15) + 5; break;
-		case SV_STAFF_REMOVE_CURSE:		o_ptr->pval = randint(3)  + 4; break;
-		case SV_STAFF_STARLITE:			o_ptr->pval = randint(5)  + 6; break;
-		case SV_STAFF_LITE:				o_ptr->pval = randint(20) + 8; break;
-		case SV_STAFF_MAPPING:			o_ptr->pval = randint(5)  + 5; break;
-		case SV_STAFF_DETECT_GOLD:		o_ptr->pval = randint(20) + 8; break;
-		case SV_STAFF_DETECT_ITEM:		o_ptr->pval = randint(15) + 6; break;
-		case SV_STAFF_DETECT_TRAP:		o_ptr->pval = randint(5)  + 6; break;
-		case SV_STAFF_DETECT_DOOR:		o_ptr->pval = randint(8)  + 6; break;
-		case SV_STAFF_DETECT_INVIS:		o_ptr->pval = randint(15) + 8; break;
-		case SV_STAFF_DETECT_EVIL:		o_ptr->pval = randint(15) + 8; break;
-		case SV_STAFF_CURE_LIGHT:		o_ptr->pval = randint(5)  + 6; break;
-		case SV_STAFF_CURING:			o_ptr->pval = randint(3)  + 4; break;
-		case SV_STAFF_HEALING:			o_ptr->pval = randint(2)  + 1; break;
-		case SV_STAFF_THE_MAGI:			o_ptr->pval = randint(2)  + 2; break;
-		case SV_STAFF_SLEEP_MONSTERS:	o_ptr->pval = randint(5)  + 6; break;
-		case SV_STAFF_SLOW_MONSTERS:	o_ptr->pval = randint(5)  + 6; break;
-		case SV_STAFF_SPEED:			o_ptr->pval = randint(3)  + 4; break;
-		case SV_STAFF_PROBING:			o_ptr->pval = randint(6)  + 2; break;
-		case SV_STAFF_DISPEL_EVIL:		o_ptr->pval = randint(3)  + 4; break;
-		case SV_STAFF_POWER:			o_ptr->pval = randint(3)  + 1; break;
-		case SV_STAFF_HOLINESS:			o_ptr->pval = randint(2)  + 2; break;
-		case SV_STAFF_BANISHMENT:		o_ptr->pval = randint(2)  + 1; break;
-		case SV_STAFF_EARTHQUAKES:		o_ptr->pval = randint(5)  + 3; break;
-		case SV_STAFF_DESTRUCTION:		o_ptr->pval = randint(3)  + 1; break;
-	}
-}
-
 
 
 /*
@@ -2514,6 +2427,8 @@ static void a_m_aux_3(object_type *o_ptr, int level, int power)
  */
 static void a_m_aux_4(object_type *o_ptr, int level, int power)
 {
+	object_kind *k_ptr = &k_info[o_ptr->k_idx];
+
 	/* Unused parameters */
 	(void)level;
 	(void)power;
@@ -2523,33 +2438,28 @@ static void a_m_aux_4(object_type *o_ptr, int level, int power)
 	{
 		case TV_LITE:
 		{
-			/* Hack -- Torches -- random fuel */
+			/* Hack -- Torches & lanterns -- random fuel at 1/2 max */
 			if (o_ptr->sval == SV_LITE_TORCH)
 			{
-				if (o_ptr->pval > 0) o_ptr->pval = randint(o_ptr->pval);
+				o_ptr->timeout = randint(FUEL_TORCH/2);
 			}
 
-			/* Hack -- Lanterns -- random fuel */
 			else if (o_ptr->sval == SV_LITE_LANTERN)
 			{
-				if (o_ptr->pval > 0) o_ptr->pval = randint(o_ptr->pval);
+				o_ptr->timeout = randint(FUEL_LAMP/2);
 			}
 
 			break;
 		}
 
 		case TV_WAND:
-		{
-			/* Hack -- charge wands */
-			charge_wand(o_ptr);
-
-			break;
-		}
-
 		case TV_STAFF:
 		{
-			/* Hack -- charge staffs */
-			charge_staff(o_ptr);
+			/* Charge staves and wands */
+			o_ptr->pval = k_ptr->charge_base;
+
+			if (k_ptr->charge_dd && k_ptr->charge_ds)
+				o_ptr->pval += damroll(k_ptr->charge_dd, k_ptr->charge_ds);
 
 			break;
 		}
@@ -2560,6 +2470,7 @@ static void a_m_aux_4(object_type *o_ptr, int level, int power)
 
 			/* Transfer the pval. */
 			o_ptr->pval = k_ptr->pval;
+
 			break;
 		}
 
@@ -2956,6 +2867,30 @@ static bool kind_is_good(int k_idx)
 			if (k_ptr->sval == SV_AMULET_TRICKERY) return (TRUE);
 			return (FALSE);
 		}
+		
+		/* Potions -- Potions of life, healing, *healing* are good,
+		 * restore mana for spell casters are good,
+		 * as are stat potions, when the stat is not maximised,
+		 * as is augmentation (acts as a potion of 'restoration' if all
+		 * stats are maximised).
+		 * 
+		 * XXX If we make too many useful items 'good' we may want to
+		 * consider limiting the total number of good drops to uniques
+		 * and truely nasty monsters.
+		 */
+		case TV_POTION:
+		{
+			if (k_ptr->sval == SV_POTION_HEALING) return (TRUE);
+			if (k_ptr->sval == SV_POTION_STAR_HEALING) return (TRUE);
+			if (k_ptr->sval == SV_POTION_LIFE) return (TRUE);
+			if ((k_ptr->sval == SV_POTION_RESTORE_MANA) && (p_ptr->msp > 0)) return (TRUE);
+			if ((k_ptr->sval >= SV_POTION_INC_STR) && (k_ptr->sval <= SV_POTION_INC_CHR))
+			{
+				if (p_ptr->stat_cur[k_ptr->sval - SV_POTION_INC_STR] < 18+100) return (TRUE);
+			}
+			if (k_ptr->sval == SV_POTION_AUGMENTATION) return (TRUE);
+			return (FALSE);
+		}
 	}
 
 	/* Assume not good */
@@ -2976,6 +2911,7 @@ static bool kind_is_good(int k_idx)
 bool make_object(object_type *j_ptr, bool good, bool great)
 {
 	int prob, base;
+	object_kind *k_ptr;
 
 
 	/* Chance of "special object" */
@@ -3023,18 +2959,19 @@ bool make_object(object_type *j_ptr, bool good, bool great)
 	/* Apply magic (allow artifacts) */
 	apply_magic(j_ptr, object_level, TRUE, good, great);
 
-	/* Hack -- generate multiple spikes/missiles */
-	switch (j_ptr->tval)
+
+	/* Generate multiple items */
+	/* Imported from Steamband and Sangband */
+	/* XXX Will probably not work so well for stacks of potions (yet) */
+	k_ptr = &k_info[j_ptr->k_idx];
+
+	if (k_ptr->gen_mult_prob >= 100 ||
+	    k_ptr->gen_mult_prob >= randint(100))
 	{
-		case TV_SPIKE:
-		case TV_SHOT:
-		case TV_ARROW:
-		case TV_BOLT:
-		{
-			j_ptr->number = damroll(6, 7);
-		}
+		j_ptr->number = damroll(k_ptr->gen_dice, k_ptr->gen_side);
 	}
 
+
 	/* Notice "okay" out-of-depth objects */
 	if (!cursed_p(j_ptr) && !broken_p(j_ptr) &&
 	    (k_info[j_ptr->k_idx].level > p_ptr->depth))
@@ -3201,7 +3138,7 @@ s16b floor_carry(int y, int x, object_type *j_ptr)
  */
 void drop_near(object_type *j_ptr, int chance, int y, int x)
 {
-	int i, k, d, s;
+	int i, k, n, d, s;
 
 	int bs, bn;
 	int by, bx;
@@ -3278,6 +3215,7 @@ void drop_near(object_type *j_ptr, int chance, int y, int x)
 
 			/* No objects */
 			k = 0;
+			n = 0;
 
 			/* Scan objects in that grid */
 			for (o_ptr = get_first_object(ty, tx); o_ptr; o_ptr = get_next_object(o_ptr))
@@ -3286,7 +3224,10 @@ void drop_near(object_type *j_ptr, int chance, int y, int x)
 				if (object_similar(o_ptr, j_ptr)) comb = TRUE;
 
 				/* Count objects */
-				k++;
+				if (!squelch_hide_item(o_ptr))
+					k++;
+				else
+					n++;
 			}
 
 			/* Add new object */
@@ -3295,8 +3236,8 @@ void drop_near(object_type *j_ptr, int chance, int y, int x)
 			/* Option -- disallow stacking */
 			if (adult_no_stacking && (k > 1)) continue;
 			
-			/* Paranoia */
-			if (k > MAX_FLOOR_STACK) continue;
+			/* Paranoia? */
+			if ((k + n) > MAX_FLOOR_STACK) continue;
 
 			/* Calculate score */
 			s = 1000 - (d + k * 5);
@@ -3862,10 +3803,22 @@ void floor_item_optimize(int item)
  */
 bool inven_carry_okay(const object_type *o_ptr)
 {
-	int j;
-
 	/* Empty slot? */
-	if (p_ptr->inven_cnt < INVEN_PACK) return (TRUE);
+	if (p_ptr->inven_cnt < INVEN_PACK) return TRUE;
+
+	/* Check if it can stack */
+	if (inven_stack_okay(o_ptr)) return TRUE;
+
+	/* Nope */
+	return FALSE;
+}
+
+/*
+ * Check to see if an item is stackable in the inventory
+ */
+bool inven_stack_okay(const object_type *o_ptr)
+{
+	int j;
 
 	/* Similar slot? */
 	for (j = 0; j < INVEN_PACK; j++)
@@ -3908,6 +3861,8 @@ s16b inven_carry(object_type *o_ptr)
 
 	object_type *j_ptr;
 
+	/* Apply an autoinscription */
+	apply_autoinscription(o_ptr);
 
 	/* Check for combining */
 	for (j = 0; j < INVEN_PACK; j++)
@@ -4219,6 +4174,7 @@ void inven_drop(int item, int amt)
 
 /*
  * Combine items in the pack
+ * Also "pick up" any gold in the inventory by accident
  *
  * Note special handling of the "overflow" slot
  */
@@ -4235,14 +4191,24 @@ void combine_pack(void)
 	/* Combine the pack (backwards) */
 	for (i = INVEN_PACK; i > 0; i--)
 	{
+		bool slide = FALSE;
+
 		/* Get the item */
 		o_ptr = &inventory[i];
 
 		/* Skip empty items */
 		if (!o_ptr->k_idx) continue;
 
+		/* Absorb gold */
+		if (o_ptr->tval == TV_GOLD)
+		{
+			/* Count the gold */
+			slide = TRUE;
+			p_ptr->au += o_ptr->pval;
+		}
+
 		/* Scan the items above that item */
-		for (j = 0; j < i; j++)
+		else for (j = 0; j < i; j++)
 		{
 			/* Get the item */
 			j_ptr = &inventory[j];
@@ -4254,31 +4220,35 @@ void combine_pack(void)
 			if (object_similar(j_ptr, o_ptr))
 			{
 				/* Take note */
-				flag = TRUE;
+				flag = slide = TRUE;
 
 				/* Add together the item counts */
 				object_absorb(j_ptr, o_ptr);
 
-				/* One object is gone */
-				p_ptr->inven_cnt--;
-
-				/* Slide everything down */
-				for (k = i; k < INVEN_PACK; k++)
-				{
-					/* Hack -- slide object */
-					COPY(&inventory[k], &inventory[k+1], object_type);
-				}
-
-				/* Hack -- wipe hole */
-				object_wipe(&inventory[k]);
-
-				/* Window stuff */
-				p_ptr->window |= (PW_INVEN);
-
-				/* Done */
 				break;
 			}
 		}
+
+
+		/* Compact the inventory */
+		if (slide)
+		{
+			/* One object is gone */
+			p_ptr->inven_cnt--;
+
+			/* Slide everything down */
+			for (k = i; k < INVEN_PACK; k++)
+			{
+				/* Hack -- slide object */
+				COPY(&inventory[k], &inventory[k+1], object_type);
+			}
+
+			/* Hack -- wipe hole */
+			object_wipe(&inventory[k]);
+
+			/* Window stuff */
+			p_ptr->window |= (PW_INVEN);
+		}
 	}
 
 	/* Message */
