@@ -12,633 +12,21 @@
 #include "angband.h"
 
 #include "init.h"
+#include "randname.h"
 
 /*
  * Random artifact generator (randart) by Greg Wooledge.
- *
- * The external "names.txt" file was sucked into this file for simplicity.
  */
 
-#ifdef GJW_RANDART
-
-static cptr names_list =
-"adanedhel\n"
-"adurant\n"
-"aeglos\n"
-"aegnor\n"
-"aelin\n"
-"aeluin\n"
-"aerandir\n"
-"aerin\n"
-"agarwaen\n"
-"aglareb\n"
-"aglarond\n"
-"aglon\n"
-"ainulindale\n"
-"ainur\n"
-"alcarinque\n"
-"aldaron\n"
-"aldudenie\n"
-"almaren\n"
-"alqualonde\n"
-"aman\n"
-"amandil\n"
-"amarie\n"
-"amarth\n"
-"amlach\n"
-"amon\n"
-"amras\n"
-"amrod\n"
-"anach\n"
-"anar\n"
-"anarion\n"
-"ancalagon\n"
-"ancalimon\n"
-"anarrima\n"
-"andor\n"
-"andram\n"
-"androth\n"
-"anduin\n"
-"andunie\n"
-"anfauglir\n"
-"anfauglith\n"
-"angainor\n"
-"angband\n"
-"anghabar\n"
-"anglachel\n"
-"angrenost\n"
-"angrim\n"
-"angrist\n"
-"angrod\n"
-"anguirel\n"
-"annael\n"
-"annatar\n"
-"annon\n"
-"annuminas\n"
-"apanonar\n"
-"aradan\n"
-"aragorn\n"
-"araman\n"
-"aranel\n"
-"aranruth\n"
-"aranwe\n"
-"aras\n"
-"aratan\n"
-"aratar\n"
-"arathorn\n"
-"arda\n"
-"ard-galen\n"
-"aredhel\n"
-"ar-feiniel\n"
-"argonath\n"
-"arien\n"
-"armenelos\n"
-"arminas\n"
-"arnor\n"
-"aros\n"
-"arossiach\n"
-"arthad\n"
-"arvernien\n"
-"arwen\n"
-"ascar\n"
-"astaldo\n"
-"atalante\n"
-"atanamir\n"
-"atanatari\n"
-"atani\n"
-"aule\n"
-"avallone\n"
-"avari\n"
-"avathar\n"
-"balan\n"
-"balar\n"
-"balrog\n"
-"barad\n"
-"baragund\n"
-"barahir\n"
-"baran\n"
-"baranduin\n"
-"bar\n"
-"bauglir\n"
-"beleg\n"
-"belegaer\n"
-"belegost\n"
-"belegund\n"
-"beleriand\n"
-"belfalas\n"
-"belthil\n"
-"belthronding\n"
-"beor\n"
-"beraid\n"
-"bereg\n"
-"beren\n"
-"boromir\n"
-"boron\n"
-"bragollach\n"
-"brandir\n"
-"bregolas\n"
-"bregor\n"
-"brethil\n"
-"brilthor\n"
-"brithiach\n"
-"brithombar\n"
-"brithon\n"
-"cabed\n"
-"calacirya\n"
-"calaquendi\n"
-"calenardhon\n"
-"calion\n"
-"camlost\n"
-"caragdur\n"
-"caranthir\n"
-"carcharoth\n"
-"cardolan\n"
-"carnil\n"
-"celeborn\n"
-"celebrant\n"
-"celebrimbor\n"
-"celebrindal\n"
-"celebros\n"
-"celegorm\n"
-"celon\n"
-"cirdan\n"
-"cirith\n"
-"cirth\n"
-"ciryatan\n"
-"ciryon\n"
-"coimas\n"
-"corollaire\n"
-"crissaegrim\n"
-"cuarthal\n"
-"cuivienen\n"
-"culurien\n"
-"curufin\n"
-"curufinwe\n"
-"curunir\n"
-"cuthalion\n"
-"daedeloth\n"
-"daeron\n"
-"dagnir\n"
-"dagor\n"
-"dagorlad\n"
-"dairuin\n"
-"danwedh\n"
-"delduwath\n"
-"denethor\n"
-"dimbar\n"
-"dimrost\n"
-"dinen\n"
-"dior\n"
-"dirnen\n"
-"dolmed\n"
-"doriath\n"
-"dorlas\n"
-"dorthonion\n"
-"draugluin\n"
-"drengist\n"
-"duath\n"
-"duinath\n"
-"duilwen\n"
-"dunedain\n"
-"dungortheb\n"
-"earendil\n"
-"earendur\n"
-"earnil\n"
-"earnur\n"
-"earrame\n"
-"earwen\n"
-"echor\n"
-"echoriath\n"
-"ecthelion\n"
-"edain\n"
-"edrahil\n"
-"eglador\n"
-"eglarest\n"
-"eglath\n"
-"eilinel\n"
-"eithel\n"
-"ekkaia\n"
-"elbereth\n"
-"eldalie\n"
-"eldalieva\n"
-"eldamar\n"
-"eldar\n"
-"eledhwen\n"
-"elemmire\n"
-"elende\n"
-"elendil\n"
-"elendur\n"
-"elenna\n"
-"elentari\n"
-"elenwe\n"
-"elerrina\n"
-"elleth\n"
-"elmoth\n"
-"elostirion\n"
-"elrond\n"
-"elros\n"
-"elu\n"
-"eluchil\n"
-"elured\n"
-"elurin\n"
-"elwe\n"
-"elwing\n"
-"emeldir\n"
-"endor\n"
-"engrin\n"
-"engwar\n"
-"eol\n"
-"eonwe\n"
-"ephel\n"
-"erchamion\n"
-"ereb\n"
-"ered\n"
-"erech\n"
-"eregion\n"
-"ereinion\n"
-"erellont\n"
-"eressea\n"
-"eriador\n"
-"eru\n"
-"esgalduin\n"
-"este\n"
-"estel\n"
-"estolad\n"
-"ethir\n"
-"ezellohar\n"
-"faelivrin\n"
-"falas\n"
-"falathar\n"
-"falathrim\n"
-"falmari\n"
-"faroth\n"
-"fauglith\n"
-"feanor\n"
-"feanturi\n"
-"felagund\n"
-"finarfin\n"
-"finduilas\n"
-"fingolfin\n"
-"fingon\n"
-"finwe\n"
-"firimar\n"
-"formenos\n"
-"fornost\n"
-"frodo\n"
-"fuin\n"
-"fuinur\n"
-"gabilgathol\n"
-"galad\n"
-"galadriel\n"
-"galathilion\n"
-"galdor\n"
-"galen\n"
-"galvorn\n"
-"gandalf\n"
-"gaurhoth\n"
-"gelion\n"
-"gelmir\n"
-"gelydh\n"
-"gil\n"
-"gildor\n"
-"giliath\n"
-"ginglith\n"
-"girith\n"
-"glaurung\n"
-"glingal\n"
-"glirhuin\n"
-"gloredhel\n"
-"glorfindel\n"
-"golodhrim\n"
-"gondolin\n"
-"gondor\n"
-"gonnhirrim\n"
-"gorgoroth\n"
-"gorlim\n"
-"gorthaur\n"
-"gorthol\n"
-"gothmog\n"
-"guilin\n"
-"guinar\n"
-"guldur\n"
-"gundor\n"
-"gurthang\n"
-"gwaith\n"
-"gwareth\n"
-"gwindor\n"
-"hadhodrond\n"
-"hador\n"
-"haladin\n"
-"haldad\n"
-"haldan\n"
-"haldar\n"
-"haldir\n"
-"haleth\n"
-"halmir\n"
-"handir\n"
-"harad\n"
-"hareth\n"
-"hathaldir\n"
-"hathol\n"
-"haudh\n"
-"helcar\n"
-"helcaraxe\n"
-"helevorn\n"
-"helluin\n"
-"herumor\n"
-"herunumen\n"
-"hildorien\n"
-"himlad\n"
-"himring\n"
-"hirilorn\n"
-"hisilome\n"
-"hithaeglir\n"
-"hithlum\n"
-"hollin\n"
-"huan\n"
-"hunthor\n"
-"huor\n"
-"hurin\n"
-"hyarmendacil\n"
-"hyarmentir\n"
-"iant\n"
-"iaur\n"
-"ibun\n"
-"idril\n"
-"illuin\n"
-"ilmare\n"
-"ilmen\n"
-"iluvatar\n"
-"imlach\n"
-"imladris\n"
-"indis\n"
-"ingwe\n"
-"irmo\n"
-"isil\n"
-"isildur\n"
-"istari\n"
-"ithil\n"
-"ivrin\n"
-"kelvar\n"
-"kementari\n"
-"ladros\n"
-"laiquendi\n"
-"lalaith\n"
-"lamath\n"
-"lammoth\n"
-"lanthir\n"
-"laurelin\n"
-"leithian\n"
-"legolin\n"
-"lembas\n"
-"lenwe\n"
-"linaewen\n"
-"lindon\n"
-"lindorie\n"
-"loeg\n"
-"lomelindi\n"
-"lomin\n"
-"lomion\n"
-"lorellin\n"
-"lorien\n"
-"lorindol\n"
-"losgar\n"
-"lothlann\n"
-"lothlorien\n"
-"luin\n"
-"luinil\n"
-"lumbar\n"
-"luthien\n"
-"mablung\n"
-"maedhros\n"
-"maeglin\n"
-"maglor\n"
-"magor\n"
-"mahanaxar\n"
-"mahtan\n"
-"maiar\n"
-"malduin\n"
-"malinalda\n"
-"mandos\n"
-"manwe\n"
-"mardil\n"
-"melian\n"
-"melkor\n"
-"menegroth\n"
-"meneldil\n"
-"menelmacar\n"
-"meneltarma\n"
-"minas\n"
-"minastir\n"
-"mindeb\n"
-"mindolluin\n"
-"mindon\n"
-"minyatur\n"
-"mirdain\n"
-"miriel\n"
-"mithlond\n"
-"mithrandir\n"
-"mithrim\n"
-"mordor\n"
-"morgoth\n"
-"morgul\n"
-"moria\n"
-"moriquendi\n"
-"mormegil\n"
-"morwen\n"
-"nahar\n"
-"naeramarth\n"
-"namo\n"
-"nandor\n"
-"nargothrond\n"
-"narog\n"
-"narsil\n"
-"narsilion\n"
-"narya\n"
-"nauglamir\n"
-"naugrim\n"
-"ndengin\n"
-"neithan\n"
-"neldoreth\n"
-"nenar\n"
-"nenning\n"
-"nenuial\n"
-"nenya\n"
-"nerdanel\n"
-"nessa\n"
-"nevrast\n"
-"nibin\n"
-"nienna\n"
-"nienor\n"
-"nimbrethil\n"
-"nimloth\n"
-"nimphelos\n"
-"nimrais\n"
-"nimras\n"
-"ningloron\n"
-"niniel\n"
-"ninniach\n"
-"ninquelote\n"
-"niphredil\n"
-"nirnaeth\n"
-"nivrim\n"
-"noegyth\n"
-"nogrod\n"
-"noldolante\n"
-"noldor\n"
-"numenor\n"
-"nurtale\n"
-"obel\n"
-"ohtar\n"
-"oiolosse\n"
-"oiomure\n"
-"olorin\n"
-"olvar\n"
-"olwe\n"
-"ondolinde\n"
-"orfalch\n"
-"ormal\n"
-"orocarni\n"
-"orodreth\n"
-"orodruin\n"
-"orome\n"
-"oromet\n"
-"orthanc\n"
-"osgiliath\n"
-"osse\n"
-"ossiriand\n"
-"palantir\n"
-"pelargir\n"
-"pelori\n"
-"periannath\n"
-"quendi\n"
-"quenta\n"
-"quenya\n"
-"radagast\n"
-"radhruin\n"
-"ragnor\n"
-"ramdal\n"
-"rana\n"
-"rathloriel\n"
-"rauros\n"
-"region\n"
-"rerir\n"
-"rhovanion\n"
-"rhudaur\n"
-"rhun\n"
-"rhunen\n"
-"rian\n"
-"ringil\n"
-"ringwil\n"
-"romenna\n"
-"rudh\n"
-"rumil\n"
-"saeros\n"
-"salmar\n"
-"saruman\n"
-"sauron\n"
-"serech\n"
-"seregon\n"
-"serinde\n"
-"shelob\n"
-"silmarien\n"
-"silmaril\n"
-"silpion\n"
-"sindar\n"
-"singollo\n"
-"sirion\n"
-"soronume\n"
-"sul\n"
-"sulimo\n"
-"talath\n"
-"taniquetil\n"
-"tar\n"
-"taras\n"
-"tarn\n"
-"tathren\n"
-"taur\n"
-"tauron\n"
-"teiglin\n"
-"telchar\n"
-"telemnar\n"
-"teleri\n"
-"telperion\n"
-"telumendil\n"
-"thalion\n"
-"thalos\n"
-"thangorodrim\n"
-"thargelion\n"
-"thingol\n"
-"thoronath\n"
-"thorondor\n"
-"thranduil\n"
-"thuringwethil\n"
-"tilion\n"
-"tintalle\n"
-"tinuviel\n"
-"tirion\n"
-"tirith\n"
-"tol\n"
-"tulkas\n"
-"tumhalad\n"
-"tumladen\n"
-"tuna\n"
-"tuor\n"
-"turambar\n"
-"turgon\n"
-"turin\n"
-"uial\n"
-"uilos\n"
-"uinen\n"
-"ulairi\n"
-"ulmo\n"
-"ulumuri\n"
-"umanyar\n"
-"umarth\n"
-"umbar\n"
-"ungoliant\n"
-"urthel\n"
-"uruloki\n"
-"utumno\n"
-"vaire\n"
-"valacirca\n"
-"valandil\n"
-"valaquenta\n"
-"valar\n"
-"valaraukar\n"
-"valaroma\n"
-"valier\n"
-"valimar\n"
-"valinor\n"
-"valinoreva\n"
-"valmar\n"
-"vana\n"
-"vanyar\n"
-"varda\n"
-"vasa\n"
-"vilya\n"
-"vingilot\n"
-"vinyamar\n"
-"voronwe\n"
-"wethrin\n"
-"wilwarin\n"
-"yavanna\n"
-;
-
 #define MAX_TRIES 200
-#define BUFLEN 1024
 
+/* Random name parameters */
+#define BUFLEN 1024
 #define MIN_NAME_LEN 5
 #define MAX_NAME_LEN 9
-#define S_WORD 26
-#define E_WORD S_WORD
 
 #define sign(x)	((x) > 0 ? 1 : ((x) < 0 ? -1 : 0))
 
-
-static long lprobs[S_WORD+1][S_WORD+1][S_WORD+1];
-static long ltotal[S_WORD+1][S_WORD+1];
-
 /*
  * Cache the results of lookup_kind(), which is expensive and would
  * otherwise be called much too often.
@@ -648,110 +36,8 @@ static s16b *kinds;
 /* Global just for convenience. */
 static int randart_verbose = 0;
 
-
 /*
- * Use W. Sheldon Simms' random name generator.  This function builds
- * probability tables which are used later on for letter selection.  It
- * relies on the ASCII character set.
- */
-static void build_prob(cptr learn)
-{
-	int c_prev, c_cur, c_next;
-
-	/* Build raw frequencies */
-	while (1)
-	{
-		c_prev = c_cur = S_WORD;
-
-		do
-		{
-			c_next = *learn++;
-		} while (!isalpha(c_next) && (c_next != '\0'));
-
-		if (c_next == '\0') break;
-
-		do
-		{
-			c_next = A2I(tolower(c_next));
-			lprobs[c_prev][c_cur][c_next]++;
-			ltotal[c_prev][c_cur]++;
-			c_prev = c_cur;
-			c_cur = c_next;
-			c_next = *learn++;
-		} while (isalpha(c_next));
-
-		lprobs[c_prev][c_cur][E_WORD]++;
-		ltotal[c_prev][c_cur]++;
-	}
-}
-
-
-/*
- * Use W. Sheldon Simms' random name generator.  Generate a random word using
- * the probability tables we built earlier.  Relies on the ASCII character
- * set.  Relies on European vowels (a, e, i, o, u).  The generated name should
- * be copied/used before calling this function again.
- */
-static char *make_word(void)
-{
-	static char word_buf[90];
-	int r, totalfreq;
-	int tries, lnum, vow;
-	int c_prev, c_cur, c_next;
-	char *cp;
-
-startover:
-	vow = 0;
-	lnum = 0;
-	tries = 0;
-	cp = word_buf;
-	c_prev = c_cur = S_WORD;
-
-	while (1)
-	{
-	    getletter:
-		c_next = 0;
-		r = rand_int(ltotal[c_prev][c_cur]);
-		totalfreq = lprobs[c_prev][c_cur][c_next];
-
-		while (totalfreq <= r)
-		{
-			c_next++;
-			totalfreq += lprobs[c_prev][c_cur][c_next];
-		}
-
-		if (c_next == E_WORD)
-		{
-			if ((lnum < MIN_NAME_LEN) || vow == 0)
-			{
-				tries++;
-				if (tries < 10) goto getletter;
-				goto startover;
-			}
-			*cp = '\0';
-			break;
-		}
-
-		if (lnum >= MAX_NAME_LEN) goto startover;
-
-		*cp = I2A(c_next);
-
-		if (is_a_vowel(*cp)) vow++;
-
-		cp++;
-		lnum++;
-		c_prev = c_cur;
-		c_cur = c_next;
-	}
-
-	word_buf[0] = toupper(word_buf[0]);
-
-	return (word_buf);
-}
-
-
-/*
- * Use W. Sheldon Simms' random name generator.
+ * Use W. Sheldon Simms' random name generator from randname.c
  */
 static errr init_names(void)
 {
@@ -764,21 +50,20 @@ static errr init_names(void)
 	/* Temporary space for names, while reading and randomizing them. */
 	cptr *names;
 
-
-	build_prob(names_list);
-
 	/* Allocate the "names" array */
 	/* ToDo: Make sure the memory is freed correctly in case of errors */
 	C_MAKE(names, z_info->a_max, cptr);
 
 	for (i = 0; i < z_info->a_max; i++)
 	{
-		char *word = make_word();
+		char word[MAX_NAME_LEN + 1];
+		randname_make(RANDNAME_TOLKIEN, MIN_NAME_LEN, MAX_NAME_LEN, word, sizeof word);
+		word[0] = toupper((unsigned char) word[0]);
 
 		if (rand_int(3) == 0)
-			sprintf(buf, "'%s'", word);
+			strnfmt(buf, sizeof(buf), "'%s'", word);
 		else
-			sprintf(buf, "of %s", word);
+			strnfmt(buf, sizeof(buf), "of %s", word);
 
 		names[i] = string_make(buf);
 	}
@@ -805,14 +90,14 @@ static errr init_names(void)
 
 	for (i = 1; i < z_info->a_max; i++)
 	{
-		strcpy(a_next, names[i-1]);
+		my_strcpy(a_next, names[i-1], name_size - 1);
 		if (a_info[i].tval > 0)		/* skip unused! */
 			a_info[i].name = a_next - a_base;
 		a_next += strlen(names[i-1]) + 1;
 	}
 
 	/* Free the old names */
-	C_KILL(a_name, a_head.name_size, char);
+	FREE(a_name);
 
 	for (i = 0; i < z_info->a_max; i++)
 	{
@@ -820,7 +105,7 @@ static errr init_names(void)
 	}
 
 	/* Free the "names" array */
-	C_FREE((void*)names, z_info->a_max, cptr);
+	FREE(names);
 
 	/* Store the names */
 	a_name = a_base;
@@ -931,6 +216,8 @@ static s32b artifact_power(int a_idx)
 			p += (a_ptr->dd * a_ptr->ds + 1) / 2;
 			if (a_ptr->flags1 & TR1_SLAY_EVIL) p = (p * 3) / 2;
 			if (a_ptr->flags1 & TR1_KILL_DRAGON) p = (p * 3) / 2;
+			if (a_ptr->flags1 & TR1_KILL_DEMON) p = (p * 3) / 2;
+			if (a_ptr->flags1 & TR1_KILL_UNDEAD) p = (p * 3) / 2;
 			if (a_ptr->flags1 & TR1_SLAY_ANIMAL) p = (p * 4) / 3;
 			if (a_ptr->flags1 & TR1_SLAY_UNDEAD) p = (p * 4) / 3;
 			if (a_ptr->flags1 & TR1_SLAY_DRAGON) p = (p * 4) / 3;
@@ -1408,6 +695,8 @@ static void remove_contradictory(artifact_type *a_ptr)
 
 	if (a_ptr->flags3 & TR3_LIGHT_CURSE) a_ptr->flags3 &= ~(TR3_BLESSED);
 	if (a_ptr->flags1 & TR1_KILL_DRAGON) a_ptr->flags1 &= ~(TR1_SLAY_DRAGON);
+	if (a_ptr->flags1 & TR1_KILL_DEMON) a_ptr->flags1 &= ~(TR1_SLAY_DEMON);
+	if (a_ptr->flags1 & TR1_KILL_UNDEAD) a_ptr->flags1 &= ~(TR1_SLAY_UNDEAD);
 	if (a_ptr->flags3 & TR3_DRAIN_EXP) a_ptr->flags3 &= ~(TR3_HOLD_LIFE);
 }
 
@@ -1415,6 +704,8 @@ static void remove_contradictory(artifact_type *a_ptr)
 /*
  * Randomly select an extra ability to be added to the artifact in question.
  * XXX - This function is way too large.
+ *
+ * ToDo: Add the KILL_UNDEAD and KILL_DEMON flags.
  */
 static void add_ability(artifact_type *a_ptr)
 {
@@ -1989,6 +1280,7 @@ static void scramble_artifact(int a_idx)
 
 	/* Restore some flags */
 	if (activates) a_ptr->flags3 |= TR3_ACTIVATE;
+	if (a_ptr->tval == TV_LITE) a_ptr->flags3 |= TR3_NO_FUEL;
 	if (a_idx < ART_MIN_NORMAL) a_ptr->flags3 |= TR3_INSTA_ART;
 
 	/*
@@ -2046,7 +1338,7 @@ static bool artifacts_acceptable(void)
 		if (randart_verbose)
 		{
 			char types[256];
-			sprintf(types, "%s%s%s%s%s%s%s%s%s%s",
+			strnfmt(types, sizeof(types), "%s%s%s%s%s%s%s%s%s%s",
 				swords > 0 ? " swords" : "",
 				polearms > 0 ? " polearms" : "",
 				blunts > 0 ? " blunts" : "",
@@ -2057,8 +1349,8 @@ static bool artifacts_acceptable(void)
 				hats > 0 ? " hats" : "",
 				gloves > 0 ? " gloves" : "",
 				boots > 0 ? " boots" : "");
-			msg_format("Restarting generation process: not enough%s",
-				types);
+
+			msg_format("Restarting generation process: not enough%s", types);
 		}
 
 		/* Not acceptable */
@@ -2091,7 +1383,7 @@ static errr scramble(void)
 	}
 
 	/* Free the "kinds" array */
-	C_FREE(kinds, z_info->a_max, s16b);
+	FREE(kinds);
 
 	/* Success */
 	return (0);
@@ -2139,10 +1431,3 @@ errr do_randart(u32b randart_seed, bool full)
 	return (err);
 }
 
-#else /* GJW_RANDART */
-
-#ifdef MACINTOSH
-static int i = 0;
-#endif /* MACINTOSH */
-
-#endif /* GJW_RANDART */
