@@ -478,7 +478,7 @@ void take_hit(int dam, cptr kb_str)
 	disturb(1, 0);
 
 	/* Mega-Hack -- Apply "invulnerability" */
-	if (p_ptr->invuln && (dam < 9000)) return;
+	if (p_ptr->timed[TMD_INVULN] && (dam < 9000)) return;
 
 	/* Hurt the player */
 	p_ptr->chp -= dam;
@@ -752,7 +752,8 @@ static int inven_damage(inven_func typ, int perc)
 	object_type *o_ptr;
 
 	char o_name[80];
-
+	
+	bool damage;
 
 	/* Count the casualties */
 	k = 0;
@@ -774,11 +775,82 @@ static int inven_damage(inven_func typ, int perc)
 			/* Scale the destruction chance up */
 			int chance = perc * 100;
 
-			/* Rods are tough */
-			if (o_ptr->tval == TV_ROD) chance = (chance / 4);
+			damage = FALSE;
+
+			/* Analyze the type to see if we just damage it */
+			switch (o_ptr->tval)
+			{
+				/* Weapons */
+				case TV_BOW:
+				case TV_SWORD:
+				case TV_HAFTED:
+				case TV_POLEARM:
+				case TV_DIGGING:
+				{
+					/* Chance to damage it */
+					if (rand_int(10000) < perc)
+					{
+						/* Damage the item */
+						o_ptr->to_h--;
+						o_ptr->to_d--;
+
+						/* Damaged! */
+						damage = TRUE;
+					}
+					else continue;
+
+					break;
+				}
+
+				/* Wearable items */
+				case TV_HELM:
+				case TV_CROWN:
+				case TV_SHIELD:
+				case TV_BOOTS:
+				case TV_GLOVES:
+				case TV_CLOAK:
+				case TV_SOFT_ARMOR:
+				case TV_HARD_ARMOR:
+				case TV_DRAG_ARMOR:
+				{
+					/* Chance to damage it */
+					if (rand_int(10000) < perc)
+					{
+						/* Damage the item */
+						o_ptr->to_a--;
+
+						/* Damaged! */
+						damage = TRUE;
+					}
+					else continue;
+
+					break;
+				}
+				
+				/* Rods are tough */
+				case TV_ROD:
+				{
+					chance = (chance / 4);
+					
+					break;
+				}
+			}
+
+			/* Damage instead of destroy */
+			if (damage)
+			{
+				/* Calculate bonuses */
+				p_ptr->update |= (PU_BONUS);
+
+				/* Window stuff */
+				p_ptr->window |= (PW_EQUIP | PW_PLAYER_0 | PW_PLAYER_1);
+
+				/* Casualty count */
+				amt = o_ptr->number;
+			}
 
 			/* Count the casualties */
-			for (amt = j = 0; j < o_ptr->number; ++j)
+			else for (amt = j = 0; j < o_ptr->number; ++j)
 			{
 				if (rand_int(10000) < chance) amt++;
 			}
@@ -790,12 +862,16 @@ static int inven_damage(inven_func typ, int perc)
 				object_desc(o_name, sizeof(o_name), o_ptr, FALSE, 3);
 
 				/* Message */
-				message_format(MSG_DESTROY, 0, "%sour %s (%c) %s destroyed!",
+				message_format(MSG_DESTROY, 0, "%sour %s (%c) %s %s!",
 				           ((o_ptr->number > 1) ?
 				            ((amt == o_ptr->number) ? "All of y" :
 				             (amt > 1 ? "Some of y" : "One of y")) : "Y"),
 				           o_name, index_to_label(i),
-				           ((amt > 1) ? "were" : "was"));
+				           ((amt > 1) ? "were" : "was"),
+					   (damage ? "damaged" : "destroyed"));
+
+				/* Damage already done? */
+				if (damage) continue;
 
 				/* Hack -- If rods, wands, or staves are destroyed, the total
 				 * maximum timeout or charges of the stack needs to be reduced,
@@ -903,7 +979,7 @@ void acid_dam(int dam, cptr kb_str)
 
 	/* Resist the damage */
 	if (p_ptr->resist_acid) dam = (dam + 2) / 3;
-	if (p_ptr->oppose_acid) dam = (dam + 2) / 3;
+	if (p_ptr->timed[TMD_OPP_ACID]) dam = (dam + 2) / 3;
 
 	/* If any armor gets hit, defend the player */
 	if (minus_ac()) dam = (dam + 1) / 2;
@@ -927,7 +1003,7 @@ void elec_dam(int dam, cptr kb_str)
 	if (p_ptr->immune_elec || (dam <= 0)) return;
 
 	/* Resist the damage */
-	if (p_ptr->oppose_elec) dam = (dam + 2) / 3;
+	if (p_ptr->timed[TMD_OPP_ELEC]) dam = (dam + 2) / 3;
 	if (p_ptr->resist_elec) dam = (dam + 2) / 3;
 
 	/* Take damage */
@@ -952,7 +1028,7 @@ void fire_dam(int dam, cptr kb_str)
 
 	/* Resist the damage */
 	if (p_ptr->resist_fire) dam = (dam + 2) / 3;
-	if (p_ptr->oppose_fire) dam = (dam + 2) / 3;
+	if (p_ptr->timed[TMD_OPP_FIRE]) dam = (dam + 2) / 3;
 
 	/* Take damage */
 	take_hit(dam, kb_str);
@@ -974,7 +1050,7 @@ void cold_dam(int dam, cptr kb_str)
 
 	/* Resist the damage */
 	if (p_ptr->resist_cold) dam = (dam + 2) / 3;
-	if (p_ptr->oppose_cold) dam = (dam + 2) / 3;
+	if (p_ptr->timed[TMD_OPP_COLD]) dam = (dam + 2) / 3;
 
 	/* Take damage */
 	take_hit(dam, kb_str);
@@ -1314,7 +1390,7 @@ static void apply_nexus(const monster_type *m_ptr)
 
 		case 6:
 		{
-			if (rand_int(100) < p_ptr->skill_sav)
+			if (rand_int(100) < p_ptr->skills[SKILL_SAV])
 			{
 				msg_print("You resist the effects!");
 				break;
@@ -1327,7 +1403,7 @@ static void apply_nexus(const monster_type *m_ptr)
 
 		case 7:
 		{
-			if (rand_int(100) < p_ptr->skill_sav)
+			if (rand_int(100) < p_ptr->skills[SKILL_SAV])
 			{
 				msg_print("You resist the effects!");
 				break;
@@ -1670,7 +1746,7 @@ static bool project_f(int who, int r, int y, int x, int dam, int typ)
 			/* Grid is in line of sight */
 			if (player_has_los_bold(y, x))
 			{
-				if (!p_ptr->blind)
+				if (!p_ptr->timed[TMD_BLIND])
 				{
 					/* Observe */
 					obvious = TRUE;
@@ -3256,7 +3332,7 @@ static bool project_p(int who, int r, int y, int x, int dam, int typ)
 	bool obvious = TRUE;
 
 	/* Player blind-ness */
-	bool blind = (p_ptr->blind ? TRUE : FALSE);
+	bool blind = (p_ptr->timed[TMD_BLIND] ? TRUE : FALSE);
 
 	/* Source monster */
 	monster_type *m_ptr;
@@ -3294,12 +3370,6 @@ static bool project_p(int who, int r, int y, int x, int dam, int typ)
 	/* Get the monster's real name */
 	monster_desc(killer, sizeof(killer), m_ptr, 0x88);
 
-	/* Event -- hit the player with beam/bolt/ball */
-	if (projection_hit_player(who, dam, typ))
-	{
-		/* HACK - event was handled, skip it */
-		typ = 0;
-	}
 
 	/* Analyze the damage */
 	switch (typ)
@@ -3341,11 +3411,11 @@ static bool project_p(int who, int r, int y, int x, int dam, int typ)
 		{
 			if (blind) msg_print("You are hit by poison!");
 			if (p_ptr->resist_pois) dam = (dam + 2) / 3;
-			if (p_ptr->oppose_pois) dam = (dam + 2) / 3;
+			if (p_ptr->timed[TMD_OPP_POIS]) dam = (dam + 2) / 3;
 			take_hit(dam, killer);
-			if (!(p_ptr->resist_pois || p_ptr->oppose_pois))
+			if (!(p_ptr->resist_pois || p_ptr->timed[TMD_OPP_POIS]))
 			{
-				(void)set_poisoned(p_ptr->poisoned + rand_int(dam) + 10);
+				(void)inc_timed(TMD_POISONED, rand_int(dam) + 10);
 			}
 			break;
 		}
@@ -3383,7 +3453,7 @@ static bool project_p(int who, int r, int y, int x, int dam, int typ)
 			if (!p_ptr->resist_sound)
 			{
 				int k = (randint((dam > 40) ? 35 : (dam * 3 / 4 + 5)));
-				(void)set_stun(p_ptr->stun + k);
+				(void)inc_timed(TMD_STUN, k);
 			}
 			break;
 		}
@@ -3428,11 +3498,11 @@ static bool project_p(int who, int r, int y, int x, int dam, int typ)
 			if (blind) msg_print("You are hit by something!");
 			if (!p_ptr->resist_sound)
 			{
-				(void)set_stun(p_ptr->stun + randint(40));
+				(void)inc_timed(TMD_STUN, randint(40));
 			}
 			if (!p_ptr->resist_confu)
 			{
-				(void)set_confused(p_ptr->confused + randint(5) + 5);
+				(void)inc_timed(TMD_CONFUSED, randint(5) + 5);
 			}
 			take_hit(dam, killer);
 			break;
@@ -3448,11 +3518,11 @@ static bool project_p(int who, int r, int y, int x, int dam, int typ)
 			}
 			if (!p_ptr->resist_confu && !p_ptr->resist_chaos)
 			{
-				(void)set_confused(p_ptr->confused + rand_int(20) + 10);
+				(void)inc_timed(TMD_CONFUSED, rand_int(20) + 10);
 			}
 			if (!p_ptr->resist_chaos)
 			{
-				(void)set_image(p_ptr->image + randint(10));
+				(void)inc_timed(TMD_IMAGE, randint(10));
 			}
 			if (!p_ptr->resist_nethr && !p_ptr->resist_chaos)
 			{
@@ -3490,7 +3560,7 @@ static bool project_p(int who, int r, int y, int x, int dam, int typ)
 			}
 			else
 			{
-				(void)set_cut(p_ptr->cut + dam);
+				(void)inc_timed(TMD_CUT, dam);
 			}
 			take_hit(dam, killer);
 			break;
@@ -3507,7 +3577,7 @@ static bool project_p(int who, int r, int y, int x, int dam, int typ)
 			else
 			{
 				int k = (randint((dam > 90) ? 35 : (dam / 3 + 5)));
-				(void)set_stun(p_ptr->stun + k);
+				(void)inc_timed(TMD_STUN, k);
 			}
 			take_hit(dam, killer);
 			break;
@@ -3523,7 +3593,7 @@ static bool project_p(int who, int r, int y, int x, int dam, int typ)
 			}
 			if (!p_ptr->resist_confu)
 			{
-				(void)set_confused(p_ptr->confused + randint(20) + 10);
+				(void)inc_timed(TMD_CONFUSED, randint(20) + 10);
 			}
 			take_hit(dam, killer);
 			break;
@@ -3567,7 +3637,7 @@ static bool project_p(int who, int r, int y, int x, int dam, int typ)
 			if (blind) msg_print("You are hit by something!");
 			if (!p_ptr->resist_sound)
 			{
-				(void)set_stun(p_ptr->stun + randint(20));
+				(void)inc_timed(TMD_STUN, randint(20));
 			}
 			take_hit(dam, killer);
 			break;
@@ -3577,7 +3647,7 @@ static bool project_p(int who, int r, int y, int x, int dam, int typ)
 		case GF_INERTIA:
 		{
 			if (blind) msg_print("You are hit by something strange!");
-			(void)set_slow(p_ptr->slow + rand_int(4) + 4);
+			(void)inc_timed(TMD_SLOW, rand_int(4) + 4);
 			take_hit(dam, killer);
 			break;
 		}
@@ -3592,7 +3662,7 @@ static bool project_p(int who, int r, int y, int x, int dam, int typ)
 			}
 			else if (!blind && !p_ptr->resist_blind)
 			{
-				(void)set_blind(p_ptr->blind + randint(5) + 2);
+				(void)inc_timed(TMD_BLIND, randint(5) + 2);
 			}
 			take_hit(dam, killer);
 			break;
@@ -3608,7 +3678,7 @@ static bool project_p(int who, int r, int y, int x, int dam, int typ)
 			}
 			else if (!blind && !p_ptr->resist_blind)
 			{
-				(void)set_blind(p_ptr->blind + randint(5) + 2);
+				(void)inc_timed(TMD_BLIND, randint(5) + 2);
 			}
 			take_hit(dam, killer);
 			break;
@@ -3675,11 +3745,11 @@ static bool project_p(int who, int r, int y, int x, int dam, int typ)
 			if (randint(127) > p_ptr->lev)
 				teleport_player(5);
 
-			(void)set_slow(p_ptr->slow + rand_int(4) + 4);
+			(void)inc_timed(TMD_SLOW, rand_int(4) + 4);
 			if (!p_ptr->resist_sound)
 			{
 				int k = (randint((dam > 90) ? 35 : (dam / 3 + 5)));
-				(void)set_stun(p_ptr->stun + k);
+				(void)inc_timed(TMD_STUN, k);
 			}
 			take_hit(dam, killer);
 			break;
@@ -3708,11 +3778,11 @@ static bool project_p(int who, int r, int y, int x, int dam, int typ)
 			cold_dam(dam, killer);
 			if (!p_ptr->resist_shard)
 			{
-				(void)set_cut(p_ptr->cut + damroll(5, 8));
+				(void)inc_timed(TMD_CUT, damroll(5, 8));
 			}
 			if (!p_ptr->resist_sound)
 			{
-				(void)set_stun(p_ptr->stun + randint(15));
+				(void)inc_timed(TMD_STUN, randint(15));
 			}
 			break;
 		}
@@ -3905,7 +3975,7 @@ bool project(int who, int rad, int y, int x, int dam, int typ, int flg)
 	bool drawn = FALSE;
 
 	/* Is the player blind? */
-	bool blind = (p_ptr->blind ? TRUE : FALSE);
+	bool blind = (p_ptr->timed[TMD_BLIND] ? TRUE : FALSE);
 
 	/* Number of grids in the "path" */
 	int path_n = 0;
@@ -4039,21 +4109,16 @@ bool project(int who, int rad, int y, int x, int dam, int typ, int flg)
 				/* Visual effects */
 				print_rel(c, a, y, x);
 				move_cursor_relative(y, x);
-				if (fresh_before)
-				{
-					Term_fresh();
-					if (p_ptr->window) window_stuff();
-				}
+
+				Term_fresh();
+				if (p_ptr->window) window_stuff();
 
 				Term_xtra(TERM_XTRA_DELAY, msec);
 
 				lite_spot(y, x);
 
-				if (fresh_before)
-				{
-					Term_fresh();
-					if (p_ptr->window) window_stuff();
-				}
+				Term_fresh();
+				if (p_ptr->window) window_stuff();
 
 				/* Display "beam" grids */
 				if (flg & (PROJECT_BEAM))
@@ -4172,7 +4237,7 @@ bool project(int who, int rad, int y, int x, int dam, int typ, int flg)
 			move_cursor_relative(y2, x2);
 
 			/* Flush each "radius" separately */
-			if (fresh_before) Term_fresh();
+			Term_fresh();
 
 			/* Flush */
 			if (p_ptr->window) window_stuff();
@@ -4205,7 +4270,7 @@ bool project(int who, int rad, int y, int x, int dam, int typ, int flg)
 			move_cursor_relative(y2, x2);
 
 			/* Flush the explosion */
-			if (fresh_before) Term_fresh();
+			Term_fresh();
 
 			/* Flush */
 			if (p_ptr->window) window_stuff();
