@@ -77,18 +77,27 @@ bool hp_player(int num)
  */
 void warding_glyph(void)
 {
+	object_type *o_ptr;
 	int py = p_ptr->py;
 	int px = p_ptr->px;
 
-	/* XXX XXX XXX */
-	if (!cave_clean_bold(py, px))
+	if (cave_feat[py][px] != FEAT_FLOOR)
 	{
-		msg_print("The object resists the spell.");
+		msg_print("There is no clear floor on which to cast the spell.");
 		return;
 	}
 
 	/* Create a glyph */
 	cave_set_feat(py, px, FEAT_GLYPH);
+
+	/* Shift any objects to further away */
+	for (o_ptr = get_first_object(py, px); o_ptr; o_ptr = get_next_object(o_ptr))
+	{
+		drop_near(o_ptr, 0, py, px);
+	}
+
+	/* Delete the "moved" objects from their original position */
+	delete_object(py, px);
 }
 
 
@@ -239,18 +248,14 @@ void identify_pack(void)
 		if (!o_ptr->k_idx) continue;
 
 		/* Aware and Known */
-		object_aware(o_ptr);
-		object_known(o_ptr);
+		if (object_known_p(o_ptr)) continue;
+
+		/* Identify it */
+		do_ident_item(i, o_ptr);
+
+		/* repeat with same slot */
+		i--;
 	}
-
-	/* Recalculate bonuses */
-	p_ptr->update |= (PU_BONUS);
-
-	/* Combine / Reorder the pack (later) */
-	p_ptr->notice |= (PN_COMBINE | PN_REORDER);
-
-	/* Window stuff */
-	p_ptr->window |= (PW_INVEN | PW_EQUIP | PW_PLAYER_0 | PW_PLAYER_1);
 }
 
 
@@ -278,11 +283,8 @@ static void uncurse_object(object_type *o_ptr)
 	/* Uncurse it */
 	o_ptr->ident &= ~(IDENT_CURSED);
 
-	/* Remove special inscription, if any */
-	if (o_ptr->discount >= INSCRIP_NULL) o_ptr->discount = 0;
-
-	/* Take note if allowed */
-	if (o_ptr->discount == 0) o_ptr->discount = INSCRIP_UNCURSED;
+	/* Mark as uncursed */
+	o_ptr->pseudo = INSCRIP_UNCURSED;
 
 	/* The object has been "sensed" */
 	o_ptr->ident |= (IDENT_SENSE);
@@ -391,19 +393,23 @@ bool restore_level(void)
  * Hack -- acquire self knowledge
  *
  * List various information about the player and/or his current equipment.
- *
  * See also "identify_fully()".
  *
+ * This tests the flags of the equipment being carried and the innate player
+ * flags, so any changes made in calc_bonuses need to be shadowed here.
+ *
  * Use the "roff()" routines, perhaps.  XXX XXX XXX
  *
  * Use the "show_file()" method, perhaps.  XXX XXX XXX
  *
  * This function cannot display more than 20 lines.  XXX XXX XXX
  */
-void self_knowledge(void)
+void self_knowledge(bool spoil)
 {
 	int i = 0, j, k;
 
+	u32b t1, t2, t3;
+
 	u32b f1 = 0L, f2 = 0L, f3 = 0L;
 
 	object_type *o_ptr;
@@ -414,15 +420,16 @@ void self_knowledge(void)
 	/* Get item flags from equipment */
 	for (k = INVEN_WIELD; k < INVEN_TOTAL; k++)
 	{
-		u32b t1, t2, t3;
-
 		o_ptr = &inventory[k];
 
 		/* Skip non-objects */
 		if (!o_ptr->k_idx) continue;
 
 		/* Extract the flags */
-		object_flags(o_ptr, &t1, &t2, &t3);
+		if (spoil)
+			object_flags(o_ptr, &t1, &t2, &t3);
+		else 
+			object_flags_known(o_ptr, &t1, &t2, &t3);
 
 		/* Extract flags */
 		f1 |= t1;
@@ -430,66 +437,74 @@ void self_knowledge(void)
 		f3 |= t3;
 	}
 
+	/* And flags from the player */
+	player_flags(&t1, &t2, &t3);
 
-	if (p_ptr->blind)
+	/* Extract flags */
+	f1 |= t1;
+	f2 |= t2;
+	f3 |= t3;
+
+
+	if (p_ptr->timed[TMD_BLIND])
 	{
 		info[i++] = "You cannot see.";
 	}
-	if (p_ptr->confused)
+	if (p_ptr->timed[TMD_CONFUSED])
 	{
 		info[i++] = "You are confused.";
 	}
-	if (p_ptr->afraid)
+	if (p_ptr->timed[TMD_AFRAID])
 	{
 		info[i++] = "You are terrified.";
 	}
-	if (p_ptr->cut)
+	if (p_ptr->timed[TMD_CUT])
 	{
 		info[i++] = "You are bleeding.";
 	}
-	if (p_ptr->stun)
+	if (p_ptr->timed[TMD_STUN])
 	{
 		info[i++] = "You are stunned.";
 	}
-	if (p_ptr->poisoned)
+	if (p_ptr->timed[TMD_POISONED])
 	{
 		info[i++] = "You are poisoned.";
 	}
-	if (p_ptr->image)
+	if (p_ptr->timed[TMD_IMAGE])
 	{
 		info[i++] = "You are hallucinating.";
 	}
 
-	if (p_ptr->aggravate)
+	if (f3 & TR3_AGGRAVATE)
 	{
 		info[i++] = "You aggravate monsters.";
 	}
-	if (p_ptr->teleport)
+	if (f3 & TR3_TELEPORT)
 	{
 		info[i++] = "Your position is very uncertain.";
 	}
 
-	if (p_ptr->blessed)
+	if (p_ptr->timed[TMD_BLESSED])
 	{
 		info[i++] = "You feel righteous.";
 	}
-	if (p_ptr->hero)
+	if (p_ptr->timed[TMD_HERO])
 	{
 		info[i++] = "You feel heroic.";
 	}
-	if (p_ptr->shero)
+	if (p_ptr->timed[TMD_SHERO])
 	{
 		info[i++] = "You are in a battle rage.";
 	}
-	if (p_ptr->protevil)
+	if (p_ptr->timed[TMD_PROTEVIL])
 	{
 		info[i++] = "You are protected from evil.";
 	}
-	if (p_ptr->shield)
+	if (p_ptr->timed[TMD_SHIELD])
 	{
 		info[i++] = "You are protected by a mystic shield.";
 	}
-	if (p_ptr->invuln)
+	if (p_ptr->timed[TMD_INVULN])
 	{
 		info[i++] = "You are temporarily invulnerable.";
 	}
@@ -509,172 +524,168 @@ void self_knowledge(void)
 	{
 		info[i++] = "You will soon be recalled.";
 	}
-	if (p_ptr->see_infra)
+	if (rp_ptr->infra || f1 & TR1_INFRA)
 	{
 		info[i++] = "Your eyes are sensitive to infrared light.";
 	}
 
-	if (p_ptr->slow_digest)
+	if (f3 & TR3_SLOW_DIGEST)
 	{
 		info[i++] = "Your appetite is small.";
 	}
-	if (p_ptr->ffall)
+	if (f3 & TR3_FEATHER)
 	{
 		info[i++] = "You land gently.";
 	}
-	if (p_ptr->lite)
-	{
-		info[i++] = "You are glowing with light.";
-	}
-	if (p_ptr->regenerate)
+	if (f3 & TR3_REGEN)
 	{
 		info[i++] = "You regenerate quickly.";
 	}
-	if (p_ptr->telepathy)
+	if (f3 & TR3_TELEPATHY)
 	{
 		info[i++] = "You have ESP.";
 	}
-	if (p_ptr->see_inv)
+	if (f3 & TR3_SEE_INVIS)
 	{
 		info[i++] = "You can see invisible creatures.";
 	}
-	if (p_ptr->free_act)
+	if (f3 & TR3_FREE_ACT)
 	{
 		info[i++] = "You have free action.";
 	}
-	if (p_ptr->hold_life)
+	if (f3 & TR3_HOLD_LIFE)
 	{
 		info[i++] = "You have a firm hold on your life force.";
 	}
 
-	if (p_ptr->immune_acid)
+	if (f2 & TR2_IM_ACID)
 	{
 		info[i++] = "You are completely immune to acid.";
 	}
-	else if ((p_ptr->resist_acid) && (p_ptr->oppose_acid))
+	else if ((f2 & TR2_RES_ACID) && (p_ptr->timed[TMD_OPP_ACID]))
 	{
 		info[i++] = "You resist acid exceptionally well.";
 	}
-	else if ((p_ptr->resist_acid) || (p_ptr->oppose_acid))
+	else if ((f2 & TR2_RES_ACID) || (p_ptr->timed[TMD_OPP_ACID]))
 	{
 		info[i++] = "You are resistant to acid.";
 	}
 
-	if (p_ptr->immune_elec)
+	if (f2 & TR2_IM_ELEC)
 	{
 		info[i++] = "You are completely immune to lightning.";
 	}
-	else if ((p_ptr->resist_elec) && (p_ptr->oppose_elec))
+	else if ((f2 & TR2_RES_ELEC) && (p_ptr->timed[TMD_OPP_ELEC]))
 	{
 		info[i++] = "You resist lightning exceptionally well.";
 	}
-	else if ((p_ptr->resist_elec) || (p_ptr->oppose_elec))
+	else if ((f2 & TR2_RES_ELEC) || (p_ptr->timed[TMD_OPP_ELEC]))
 	{
 		info[i++] = "You are resistant to lightning.";
 	}
 
-	if (p_ptr->immune_fire)
+	if (f2 & TR2_IM_FIRE)
 	{
 		info[i++] = "You are completely immune to fire.";
 	}
-	else if ((p_ptr->resist_fire) && (p_ptr->oppose_fire))
+	else if ((f2 & TR2_RES_FIRE) && (p_ptr->timed[TMD_OPP_FIRE]))
 	{
 		info[i++] = "You resist fire exceptionally well.";
 	}
-	else if ((p_ptr->resist_fire) || (p_ptr->oppose_fire))
+	else if ((f2 & TR2_RES_FIRE) || (p_ptr->timed[TMD_OPP_FIRE]))
 	{
 		info[i++] = "You are resistant to fire.";
 	}
 
-	if (p_ptr->immune_cold)
+	if (f2 & TR2_IM_COLD)
 	{
 		info[i++] = "You are completely immune to cold.";
 	}
-	else if ((p_ptr->resist_cold) && (p_ptr->oppose_cold))
+	else if ((f2 & TR2_RES_COLD) && (p_ptr->timed[TMD_OPP_COLD]))
 	{
 		info[i++] = "You resist cold exceptionally well.";
 	}
-	else if ((p_ptr->resist_cold) || (p_ptr->oppose_cold))
+	else if ((f2 & TR2_RES_COLD) || (p_ptr->timed[TMD_OPP_COLD]))
 	{
 		info[i++] = "You are resistant to cold.";
 	}
 
-	if ((p_ptr->resist_pois) && (p_ptr->oppose_pois))
+	if ((f2 & TR2_RES_POIS) && (p_ptr->timed[TMD_OPP_POIS]))
 	{
 		info[i++] = "You resist poison exceptionally well.";
 	}
-	else if ((p_ptr->resist_pois) || (p_ptr->oppose_pois))
+	else if ((f2 & TR2_RES_POIS) || (p_ptr->timed[TMD_OPP_POIS]))
 	{
 		info[i++] = "You are resistant to poison.";
 	}
 
-	if (p_ptr->resist_fear)
+	if (f2 & TR2_RES_FEAR)
 	{
 		info[i++] = "You are completely fearless.";
 	}
 
-	if (p_ptr->resist_lite)
+	if (f2 & TR2_RES_LITE)
 	{
 		info[i++] = "You are resistant to bright light.";
 	}
-	if (p_ptr->resist_dark)
+	if (f2 & TR2_RES_DARK)
 	{
 		info[i++] = "You are resistant to darkness.";
 	}
-	if (p_ptr->resist_blind)
+	if (f2 & TR2_RES_BLIND)
 	{
 		info[i++] = "Your eyes are resistant to blindness.";
 	}
-	if (p_ptr->resist_confu)
+	if (f2 & TR2_RES_CONFU)
 	{
 		info[i++] = "You are resistant to confusion.";
 	}
-	if (p_ptr->resist_sound)
+	if (f2 & TR2_RES_SOUND)
 	{
 		info[i++] = "You are resistant to sonic attacks.";
 	}
-	if (p_ptr->resist_shard)
+	if (f2 & TR2_RES_SHARD)
 	{
 		info[i++] = "You are resistant to blasts of shards.";
 	}
-	if (p_ptr->resist_nexus)
+	if (f2 & TR2_RES_NEXUS)
 	{
 		info[i++] = "You are resistant to nexus attacks.";
 	}
-	if (p_ptr->resist_nethr)
+	if (f2 & TR2_RES_NETHR)
 	{
 		info[i++] = "You are resistant to nether forces.";
 	}
-	if (p_ptr->resist_chaos)
+	if (f2 & TR2_RES_CHAOS)
 	{
 		info[i++] = "You are resistant to chaos.";
 	}
-	if (p_ptr->resist_disen)
+	if (f2 & TR2_RES_DISEN)
 	{
 		info[i++] = "You are resistant to disenchantment.";
 	}
 
-	if (p_ptr->sustain_str)
+	if (f2 & TR2_SUST_STR)
 	{
 		info[i++] = "Your strength is sustained.";
 	}
-	if (p_ptr->sustain_int)
+	if (f2 & TR2_SUST_INT)
 	{
 		info[i++] = "Your intelligence is sustained.";
 	}
-	if (p_ptr->sustain_wis)
+	if (f2 & TR2_SUST_WIS)
 	{
 		info[i++] = "Your wisdom is sustained.";
 	}
-	if (p_ptr->sustain_con)
-	{
-		info[i++] = "Your constitution is sustained.";
-	}
-	if (p_ptr->sustain_dex)
+	if (f2 & TR2_SUST_DEX)
 	{
 		info[i++] = "Your dexterity is sustained.";
 	}
-	if (p_ptr->sustain_chr)
+	if (f2 & TR2_SUST_CON)
+	{
+		info[i++] = "Your constitution is sustained.";
+	}
+	if (f2 & TR2_SUST_CHR)
 	{
 		info[i++] = "Your charisma is sustained.";
 	}
@@ -873,58 +884,6 @@ void self_knowledge(void)
 
 
 
-
-
-
-/*
- * Forget everything
- */
-bool lose_all_info(void)
-{
-	int i;
-
-	/* Forget info about objects */
-	for (i = 0; i < INVEN_TOTAL; i++)
-	{
-		object_type *o_ptr = &inventory[i];
-
-		/* Skip non-objects */
-		if (!o_ptr->k_idx) continue;
-
-		/* Allow "protection" by the MENTAL flag */
-		if (o_ptr->ident & (IDENT_MENTAL)) continue;
-
-		/* Remove special inscription, if any */
-		if (o_ptr->discount >= INSCRIP_NULL) o_ptr->discount = 0;
-
-		/* Hack -- Clear the "felt" flag */
-		o_ptr->ident &= ~(IDENT_SENSE);
-
-		/* Hack -- Clear the "known" flag */
-		o_ptr->ident &= ~(IDENT_KNOWN);
-
-		/* Hack -- Clear the "empty" flag */
-		o_ptr->ident &= ~(IDENT_EMPTY);
-	}
-
-	/* Recalculate bonuses */
-	p_ptr->update |= (PU_BONUS);
-
-	/* Combine / Reorder the pack (later) */
-	p_ptr->notice |= (PN_COMBINE | PN_REORDER);
-
-	/* Window stuff */
-	p_ptr->window |= (PW_INVEN | PW_EQUIP | PW_PLAYER_0 | PW_PLAYER_1);
-
-	/* Mega-Hack -- Forget the map */
-	wiz_dark();
-
-	/* It worked */
-	return (TRUE);
-}
-
-
-
 /*
  * Set word of recall as appropriate
  */
@@ -1258,7 +1217,8 @@ bool detect_objects_normal(void)
 			lite_spot(y, x);
 
 			/* Detect */
-			detect = TRUE;
+			if (!squelch_hide_item(o_ptr))
+				detect = TRUE;
 		}
 	}
 
@@ -1325,7 +1285,8 @@ bool detect_objects_magic(void)
 			lite_spot(y, x);
 
 			/* Detect */
-			detect = TRUE;
+			if (!squelch_hide_item(o_ptr))
+				detect = TRUE;
 		}
 	}
 
@@ -1888,8 +1849,6 @@ bool ident_spell(void)
 
 	object_type *o_ptr;
 
-	char o_name[80];
-
 	cptr q, s;
 
 	/* Only un-id'ed items */
@@ -1913,55 +1872,9 @@ bool ident_spell(void)
 	}
 
 
-	/* Identify it */
-	object_aware(o_ptr);
-	object_known(o_ptr);
+	/* Identify the object */
+	do_ident_item(item, o_ptr);
 
-	/* Recalculate bonuses */
-	p_ptr->update |= (PU_BONUS);
-
-	/* Combine / Reorder the pack (later) */
-	p_ptr->notice |= (PN_COMBINE | PN_REORDER);
-
-	/* Window stuff */
-	p_ptr->window |= (PW_INVEN | PW_EQUIP | PW_PLAYER_0 | PW_PLAYER_1);
-
-	/* Description */
-	object_desc(o_name, sizeof(o_name), o_ptr, TRUE, 3);
-
-	/* Possibly play a sound depending on object quality. */
-	if (o_ptr->pval < 0) 
-	{
-		/* This is a bad item. */
-		sound(MSG_IDENT_BAD);
-	} 
-	else if (o_ptr->name1 != 0)
-	{
-		/* We have a good artifact. */
-		sound(MSG_IDENT_ART);
-	}
-	else if (o_ptr->name2 != 0)
-	{
-		/* We have a good ego item. */
-		sound(MSG_IDENT_EGO);
-	}
-
-	/* Describe */
-	if (item >= INVEN_WIELD)
-	{
-		msg_format("%^s: %s (%c).",
-		           describe_use(item), o_name, index_to_label(item));
-	}
-	else if (item >= 0)
-	{
-		msg_format("In your pack: %s (%c).",
-		           o_name, index_to_label(item));
-	}
-	else
-	{
-		msg_format("On the ground: %s.",
-		           o_name);
-	}
 
 	/* Something happened */
 	return (TRUE);
@@ -1980,8 +1893,6 @@ bool identify_fully(void)
 
 	object_type *o_ptr;
 
-	char o_name[80];
-
 	cptr q, s;
 
 
@@ -2005,66 +1916,19 @@ bool identify_fully(void)
 		o_ptr = &o_list[0 - item];
 	}
 
-
-	/* Identify it */
-	object_aware(o_ptr);
-	object_known(o_ptr);
+	/* Identify the object */
+	do_ident_item(item, o_ptr);
 
 	/* Mark the item as fully known */
 	o_ptr->ident |= (IDENT_MENTAL);
 
-	/* Recalculate bonuses */
-	p_ptr->update |= (PU_BONUS);
-
-	/* Combine / Reorder the pack (later) */
-	p_ptr->notice |= (PN_COMBINE | PN_REORDER);
-
-	/* Window stuff */
-	p_ptr->window |= (PW_INVEN | PW_EQUIP | PW_PLAYER_0 | PW_PLAYER_1);
-
 	/* Handle stuff */
 	handle_stuff();
 
-	/* Description */
-	object_desc(o_name, sizeof(o_name), o_ptr, TRUE, 3);
-
-	/* Possibly play a sound depending on object quality. */
-	if (o_ptr->pval < 0) 
-	{
-		/* This is a bad item. */
-		sound(MSG_IDENT_BAD);
-	} 
-	else if (o_ptr->name1 != 0)
-	{
-		/* We have a good artifact. */
-		sound(MSG_IDENT_ART);
-	}
-	else if (o_ptr->name2 != 0)
-	{
-		/* We have a good ego item. */
-		sound(MSG_IDENT_EGO);
-	}
-
-	/* Describe */
-	if (item >= INVEN_WIELD)
-	{
-		msg_format("%^s: %s (%c).",
-		           describe_use(item), o_name, index_to_label(item));
-	}
-	else if (item >= 0)
-	{
-		msg_format("In your pack: %s (%c).",
-		           o_name, index_to_label(item));
-	}
-	else
-	{
-		msg_format("On the ground: %s.",
-		           o_name);
-	}
-
 	/* Describe it fully */
 	object_info_screen(o_ptr);
 
+
 	/* Success */
 	return (TRUE);
 }
@@ -2092,6 +1956,15 @@ static bool item_tester_hook_recharge(const object_type *o_ptr)
  * Recharge a wand or staff from the pack or on the floor.
  *
  * It is harder to recharge high level, and highly charged wands.
+ *
+ * XXX XXX XXX Beware of "sliding index errors".
+ *
+ * Should probably not "destroy" over-charged items, unless we
+ * "replace" them by, say, a broken stick or some such.  The only
+ * reason this is okay is because "scrolls of recharging" appear
+ * BEFORE all staves/wands in the inventory.  Note that the
+ * new "auto_sort_pack" option would correctly handle replacing
+ * the "broken" wand with any other item (i.e. a broken stick).
  */
 bool recharge(int num)
 {
@@ -2127,15 +2000,16 @@ bool recharge(int num)
 	lev = k_info[o_ptr->k_idx].level;
 
 	/* Recharge power */
-	i = (num + 100 - lev - (10 * o_ptr->pval)) / 15;
+	i = (num + 100 - lev - (10 * (o_ptr->pval / o_ptr->number))) / 15;
 
 	/* Back-fire */
 	if ((i <= 1) || (rand_int(i) == 0))
 	{
+		msg_print("The recharge backfires!");
 		msg_print("There is a bright flash of light.");
 
-		/* Drain the power */
-		o_ptr->pval = 0;
+		/* Reduce the charges of rods/wands/staves */
+		reduce_charges(o_ptr, 1);
 
 		/* *Identified* items keep the knowledge about the charges */
 		if (!(o_ptr->ident & IDENT_MENTAL))
@@ -2144,8 +2018,20 @@ bool recharge(int num)
 			o_ptr->ident &= ~(IDENT_KNOWN);
 		}
 
-		/* We know that the item is empty */
-		o_ptr->ident |= IDENT_EMPTY;
+		/* Reduce and describe inventory */
+		if (item >= 0)
+		{
+			inven_item_increase(item, -1);
+			inven_item_describe(item);
+			inven_item_optimize(item);
+		}
+		/* Reduce and describe floor item */
+		else
+		{
+			floor_item_increase(0 - item, -1);
+			floor_item_describe(0 - item);
+			floor_item_optimize(0 - item);
+		}
 	}
 
 	/* Recharge */
@@ -2354,6 +2240,7 @@ void aggravate_monsters(int who)
 bool banishment(void)
 {
 	int i;
+	unsigned dam = 0;
 
 	char typ;
 
@@ -2381,9 +2268,12 @@ bool banishment(void)
 		delete_monster_idx(i);
 
 		/* Take some damage */
-		take_hit(randint(4), "the strain of casting Banishment");
+		dam += randint(4);
 	}
 
+	/* Hurt the player */
+	take_hit(dam, "the strain of casting Banishment");
+
 	/* Update monster list window */
 	p_ptr->window |= PW_MONLIST;
 
@@ -2398,6 +2288,7 @@ bool banishment(void)
 bool mass_banishment(void)
 {
 	int i;
+	unsigned dam = 0;
 
 	bool result = FALSE;
 
@@ -2421,12 +2312,15 @@ bool mass_banishment(void)
 		delete_monster_idx(i);
 
 		/* Take some damage */
-		take_hit(randint(3), "the strain of casting Mass Banishment");
-
-		/* Note effect */
-		result = TRUE;
+		dam += randint(3);
 	}
 
+	/* Hurt the player */
+	take_hit(dam, "the strain of casting Mass Banishment");
+
+	/* Calculate result */
+	result = (dam > 0) ? TRUE : FALSE;
+
 	/* Update monster list window */
 	if (result) p_ptr->window |= PW_MONLIST;
 
@@ -2600,7 +2494,7 @@ void destroy_area(int y1, int x1, int r, bool full)
 		if (!p_ptr->resist_blind && !p_ptr->resist_lite)
 		{
 			/* Become blind */
-			(void)set_blind(p_ptr->blind + 10 + randint(10));
+			(void)inc_timed(TMD_BLIND, 10 + randint(10));
 		}
 	}
 
@@ -2771,14 +2665,14 @@ void earthquake(int cy, int cx, int r)
 				{
 					msg_print("You are bashed by rubble!");
 					damage = damroll(10, 4);
-					(void)set_stun(p_ptr->stun + randint(50));
+					(void)inc_timed(TMD_STUN, randint(50));
 					break;
 				}
 				case 3:
 				{
 					msg_print("You are crushed between the floor and ceiling!");
 					damage = damroll(10, 4);
-					(void)set_stun(p_ptr->stun + randint(50));
+					(void)inc_timed(TMD_STUN, randint(50));
 					break;
 				}
 			}
@@ -3226,7 +3120,7 @@ bool lite_area(int dam, int rad)
 	int flg = PROJECT_GRID | PROJECT_KILL;
 
 	/* Hack -- Message */
-	if (!p_ptr->blind)
+	if (!p_ptr->timed[TMD_BLIND])
 	{
 		msg_print("You are surrounded by a white light.");
 	}
@@ -3254,7 +3148,7 @@ bool unlite_area(int dam, int rad)
 	int flg = PROJECT_GRID | PROJECT_KILL;
 
 	/* Hack -- Message */
-	if (!p_ptr->blind)
+	if (!p_ptr->timed[TMD_BLIND])
 	{
 		msg_print("Darkness surrounds you.");
 	}
@@ -3922,3 +3816,71 @@ void ring_of_power(int dir)
 		}
 	}
 }
+
+
+/*
+ * Identify an item.
+ *
+ * `item` is used to print the slot occupied by an object in equip/inven.
+ * Any negative value assigned to "item" can be used for specifying an object
+ * on the floor.
+ */
+void do_ident_item(int item, object_type *o_ptr)
+{
+	char o_name[80];
+
+	/* Identify it */
+	object_aware(o_ptr);
+	object_known(o_ptr);
+
+	/* Apply an autoinscription, if necessary */
+	apply_autoinscription(o_ptr);
+
+	/* Set squelch flag */
+	p_ptr->notice |= PN_SQUELCH;
+
+	/* Recalculate bonuses */
+	p_ptr->update |= (PU_BONUS);
+
+	/* Combine / Reorder the pack (later) */
+	p_ptr->notice |= (PN_COMBINE | PN_REORDER);
+
+	/* Window stuff */
+	p_ptr->window |= (PW_INVEN | PW_EQUIP | PW_PLAYER_0 | PW_PLAYER_1);
+
+	/* Description */
+	object_desc(o_name, sizeof(o_name), o_ptr, TRUE, 3);
+
+	/* Possibly play a sound depending on object quality. */
+	if (o_ptr->pval < 0)
+	{
+		/* This is a bad item. */
+		sound(MSG_IDENT_BAD);
+	}
+	else if (o_ptr->name1 != 0)
+	{
+		/* We have a good artifact. */
+		sound(MSG_IDENT_ART);
+	}
+	else if (o_ptr->name2 != 0)
+	{
+		/* We have a good ego item. */
+		sound(MSG_IDENT_EGO);
+	}
+
+	/* Describe */
+	if (item >= INVEN_WIELD)
+	{
+		msg_format("%^s: %s (%c).",
+			  describe_use(item), o_name, index_to_label(item));
+	}
+	else if (item >= 0)
+	{
+		msg_format("In your pack: %s (%c).",
+			  o_name, index_to_label(item));
+	}
+	else
+	{
+		msg_format("On the ground: %s.", o_name);
+	}
+}
