@@ -79,6 +79,7 @@ typedef struct owner_type owner_type;
 typedef struct store_type store_type;
 typedef struct magic_type magic_type;
 typedef struct player_magic player_magic;
+typedef struct spell_type spell_type;
 typedef struct player_sex player_sex;
 typedef struct player_race player_race;
 typedef struct player_class player_class;
@@ -86,7 +87,7 @@ typedef struct hist_type hist_type;
 typedef struct player_other player_other;
 typedef struct player_type player_type;
 typedef struct start_item start_item;
-
+typedef struct autoinscription autoinscription;
 
 
 /**** Available structs ****/
@@ -112,6 +113,7 @@ struct maxima
 	u16b b_max;		/* Max size per element of "b_info[]" */
 	u16b c_max;		/* Max size for "c_info[]" */
 	u16b flavor_max; /* Max size for "flavor_info[]" */
+	u16b s_max;		/* Max size for "s_info[]" */
 
 	u16b o_max;		/* Max size for "o_list[]" */
 	u16b m_max;		/* Max size for "mon_list[]" */
@@ -187,13 +189,22 @@ struct object_kind
 	byte x_attr;		/* Desired object attribute */
 	char x_char;		/* Desired object character */
 
+	byte charge_base;	/* Charge base */
+	byte charge_dd, charge_ds;	/* Charge dice/sides */
+
+	byte gen_mult_prob;     /* Probability of generating more than one */
+	byte gen_dice;          /* Average number to generate - dice rolled */
+	byte gen_side;          /* Average number to generate - dice sides */
+
 
 	u16b flavor;		/* Special object flavor (or zero) */
+	u16b note;          /* Autoinscription field (later) */
 
+	bool aware;		/* The player is "aware" of the item's effects */
+	bool tried;		/* The player has "tried" one of the items */
 
-	bool aware;			/* The player is "aware" of the item's effects */
-
-	bool tried;			/* The player has "tried" one of the items */
+	bool squelch;		/* Squelch this item? */
+	bool everseen;		/* Used to despoilify squelch menus */
 };
 
 
@@ -271,6 +282,8 @@ struct ego_item_type
 	byte max_pval;		/* Maximum pval */
 
 	byte xtra;			/* Extra sustain/resist/power */
+
+	bool everseen;		/* Do not spoil squelch menus */
 };
 
 
@@ -328,7 +341,13 @@ struct monster_race
 
 	s32b mexp;				/* Exp value for kill */
 
-	s16b extra;				/* Unused (for now) */
+	s16b power;				/* Monster power */
+	
+#ifdef ALLOW_TEMPLATES_PROCESS
+
+	s16b highest_threat;	/* Monster highest threat */
+
+#endif /* ALLOW_TEMPLATES_PROCESS */
 
 	byte freq_innate;		/* Innate spell frequency */
 	byte freq_spell;		/* Other spell frequency */
@@ -353,6 +372,7 @@ struct monster_race
 
 	byte max_num;			/* Maximum population allowed per level */
 	byte cur_num;			/* Monster population on current level */
+
 };
 
 
@@ -452,7 +472,7 @@ struct object_type
 
 	s16b pval;			/* Item extra-parameter */
 
-	byte discount;		/* Discount (if any) */
+	byte pseudo;		/* Pseudo-ID marker */
 
 	byte number;		/* Number of items */
 
@@ -524,7 +544,7 @@ struct monster_type
 
 #ifdef DRS_SMART_OPTIONS
 
-	u32b smart;			/* Field for "smart_learn" */
+	u32b smart;			/* Field for "adult_ai_learn" */
 
 #endif /* DRS_SMART_OPTIONS */
 
@@ -606,14 +626,15 @@ struct store_type
 	s16b stock_size;		/* Stock -- Total Size of Array */
 	object_type *stock;		/* Stock -- Actual stock items */
 
-#ifndef USE_SCRIPT
 	s16b table_num;     /* Table -- Number of entries */
 	s16b table_size;    /* Table -- Total Size of Array */
 	s16b *table;        /* Table -- Legal item kinds */
-#endif /* USE_SCRIPT */
 };
 
 
+/*
+ * A structure to hold class-dependent information on spells.
+ */
 struct magic_type
 {
 	byte slevel;		/* Required level (to learn) */
@@ -634,6 +655,22 @@ struct player_magic
 };
 
 
+/*
+ * And here's the structure for the "fixed" spell information
+ */
+struct spell_type
+{
+	u32b name;			/* Name (offset) */
+	u32b text;			/* Text (offset) */
+
+	byte realm;			/* 0 = mage; 1 = priest */
+	byte tval;			/* Item type for book this spell is in */
+	byte sval;			/* Item sub-type for book (= book number) */
+	byte snum;			/* Position of spell within book */
+
+	byte spell_index;	/* Index into player_magic array */
+};
+
 
 /*
  * Player sex info
@@ -844,31 +881,7 @@ struct player_type
 	s16b stat_max[A_MAX];	/* Current "maximal" stat values */
 	s16b stat_cur[A_MAX];	/* Current "natural" stat values */
 
-	s16b fast;			/* Timed -- Fast */
-	s16b slow;			/* Timed -- Slow */
-	s16b blind;			/* Timed -- Blindness */
-	s16b paralyzed;		/* Timed -- Paralysis */
-	s16b confused;		/* Timed -- Confusion */
-	s16b afraid;		/* Timed -- Fear */
-	s16b image;			/* Timed -- Hallucination */
-	s16b poisoned;		/* Timed -- Poisoned */
-	s16b cut;			/* Timed -- Cut */
-	s16b stun;			/* Timed -- Stun */
-
-	s16b protevil;		/* Timed -- Protection */
-	s16b invuln;		/* Timed -- Invulnerable */
-	s16b hero;			/* Timed -- Heroism */
-	s16b shero;			/* Timed -- Super Heroism */
-	s16b shield;		/* Timed -- Shield Spell */
-	s16b blessed;		/* Timed -- Blessed */
-	s16b tim_invis;		/* Timed -- See Invisible */
-	s16b tim_infra;		/* Timed -- Infra Vision */
-
-	s16b oppose_acid;	/* Timed -- oppose acid */
-	s16b oppose_elec;	/* Timed -- oppose lightning */
-	s16b oppose_fire;	/* Timed -- oppose heat */
-	s16b oppose_cold;	/* Timed -- oppose cold */
-	s16b oppose_pois;	/* Timed -- oppose poison */
+	s16b timed[TMD_MAX];	/* Timed effects */
 
 	s16b word_recall;	/* Word of recall counter */
 
@@ -927,6 +940,7 @@ struct player_type
 
 	s16b resting;			/* Resting counter */
 	s16b running;			/* Running counter */
+	bool running_withpathfind;      /* Are we using the pathfinder ? */
 
 	s16b run_cur_dir;		/* Direction we are running */
 	s16b run_old_dir;		/* Direction we came from */
@@ -935,10 +949,13 @@ struct player_type
 	bool run_break_right;	/* Looking for a break (right) */
 	bool run_break_left;	/* Looking for a break (left) */
 
+	bool auto_pickup_okay;      /* Allow automatic pickup */
+
 	s16b command_cmd;		/* Gives identity of current command */
 	s16b command_arg;		/* Gives argument of current command */
 	s16b command_rep;		/* Gives repetition of current command */
 	s16b command_dir;		/* Gives direction of current command */
+	event_type command_cmd_ex; /* Gives additional information of current command */
 
 	s16b command_see;		/* See "cmd1.c" */
 	s16b command_wrk;		/* See "cmd1.c" */
@@ -1000,7 +1017,6 @@ struct player_type
 
 	bool slow_digest;	/* Slower digestion */
 	bool ffall;			/* Feather falling */
-	bool lite;			/* Permanent light */
 	bool regenerate;	/* Regeneration */
 	bool telepathy;		/* Telepathy */
 	bool see_inv;		/* See invisible */
@@ -1028,16 +1044,7 @@ struct player_type
 
 	s16b see_infra;		/* Infravision range */
 
-	s16b skill_dis;		/* Skill: Disarming */
-	s16b skill_dev;		/* Skill: Magic Devices */
-	s16b skill_sav;		/* Skill: Saving throw */
-	s16b skill_stl;		/* Skill: Stealth factor */
-	s16b skill_srh;		/* Skill: Searching ability */
-	s16b skill_fos;		/* Skill: Searching frequency */
-	s16b skill_thn;		/* Skill: To hit (normal) */
-	s16b skill_thb;		/* Skill: To hit (shooting) */
-	s16b skill_tht;		/* Skill: To hit (throwing) */
-	s16b skill_dig;		/* Skill: Digging */
+	s16b skills[SKILL_MAX];	/* Skills */
 
 	u32b noise;			/* Derived from stealth */
 
@@ -1109,3 +1116,11 @@ struct flavor_type
 	byte x_attr;    /* Desired flavor attribute */
 	char x_char;    /* Desired flavor character */
 };
+
+/* Information for object auto-inscribe */
+struct autoinscription
+{
+	s16b kind_idx;
+	s16b inscription_idx;
+};
+
