@@ -9,12 +9,9 @@
  */
 
 #include "angband.h"
-
-
-#ifndef USE_SCRIPT
-
 #include "script.h"
 
+
 static bool eat_food(object_type *o_ptr, bool *ident)
 {
 	/* Analyze the food */
@@ -22,9 +19,9 @@ static bool eat_food(object_type *o_ptr, bool *ident)
 	{
 		case SV_FOOD_POISON:
 		{
-			if (!(p_ptr->resist_pois || p_ptr->oppose_pois))
+			if (!(p_ptr->resist_pois || p_ptr->timed[TMD_OPP_POIS]))
 			{
-				if (set_poisoned(p_ptr->poisoned + rand_int(10) + 10))
+				if (inc_timed(TMD_POISONED, rand_int(10) + 10))
 				{
 					*ident = TRUE;
 				}
@@ -36,7 +33,7 @@ static bool eat_food(object_type *o_ptr, bool *ident)
 		{
 			if (!p_ptr->resist_blind)
 			{
-				if (set_blind(p_ptr->blind + rand_int(200) + 200))
+				if (inc_timed(TMD_BLIND, rand_int(200) + 200))
 				{
 					*ident = TRUE;
 				}
@@ -48,7 +45,7 @@ static bool eat_food(object_type *o_ptr, bool *ident)
 		{
 			if (!p_ptr->resist_fear)
 			{
-				if (set_afraid(p_ptr->afraid + rand_int(10) + 10))
+				if (inc_timed(TMD_AFRAID, rand_int(10) + 10))
 				{
 					*ident = TRUE;
 				}
@@ -60,7 +57,7 @@ static bool eat_food(object_type *o_ptr, bool *ident)
 		{
 			if (!p_ptr->resist_confu)
 			{
-				if (set_confused(p_ptr->confused + rand_int(10) + 10))
+				if (inc_timed(TMD_CONFUSED, rand_int(10) + 10))
 				{
 					*ident = TRUE;
 				}
@@ -72,7 +69,7 @@ static bool eat_food(object_type *o_ptr, bool *ident)
 		{
 			if (!p_ptr->resist_chaos)
 			{
-				if (set_image(p_ptr->image + rand_int(250) + 250))
+				if (inc_timed(TMD_IMAGE, rand_int(250) + 250))
 				{
 					*ident = TRUE;
 				}
@@ -84,7 +81,7 @@ static bool eat_food(object_type *o_ptr, bool *ident)
 		{
 			if (!p_ptr->free_act)
 			{
-				if (set_paralyzed(p_ptr->paralyzed + rand_int(10) + 10))
+				if (inc_timed(TMD_PARALYZED, rand_int(10) + 10))
 				{
 					*ident = TRUE;
 				}
@@ -142,25 +139,25 @@ static bool eat_food(object_type *o_ptr, bool *ident)
 
 		case SV_FOOD_CURE_POISON:
 		{
-			if (set_poisoned(0)) *ident = TRUE;
+			if (clear_timed(TMD_POISONED)) *ident = TRUE;
 			break;
 		}
 
 		case SV_FOOD_CURE_BLINDNESS:
 		{
-			if (set_blind(0)) *ident = TRUE;
+			if (clear_timed(TMD_BLIND)) *ident = TRUE;
 			break;
 		}
 
 		case SV_FOOD_CURE_PARANOIA:
 		{
-			if (set_afraid(0)) *ident = TRUE;
+			if (clear_timed(TMD_AFRAID)) *ident = TRUE;
 			break;
 		}
 
 		case SV_FOOD_CURE_CONFUSION:
 		{
-			if (set_confused(0)) *ident = TRUE;
+			if (clear_timed(TMD_CONFUSED)) *ident = TRUE;
 			break;
 		}
 
@@ -207,7 +204,7 @@ static bool eat_food(object_type *o_ptr, bool *ident)
 		case SV_FOOD_WAYBREAD:
 		{
 			msg_print("That tastes good.");
-			(void)set_poisoned(0);
+			(void)clear_timed(TMD_POISONED);
 			(void)hp_player(damroll(4, 8));
 			*ident = TRUE;
 			break;
@@ -245,7 +242,7 @@ static bool quaff_potion(object_type *o_ptr, bool *ident)
 
 		case SV_POTION_SLOWNESS:
 		{
-			if (set_slow(p_ptr->slow + randint(25) + 15)) *ident = TRUE;
+			if (inc_timed(TMD_SLOW, randint(25) + 15)) *ident = TRUE;
 			break;
 		}
 
@@ -253,17 +250,17 @@ static bool quaff_potion(object_type *o_ptr, bool *ident)
 		{
 			msg_print("The potion makes you vomit!");
 			(void)set_food(PY_FOOD_STARVE - 1);
-			(void)set_poisoned(0);
-			(void)set_paralyzed(p_ptr->paralyzed + 4);
+			(void)clear_timed(TMD_POISONED);
+			(void)inc_timed(TMD_PARALYZED, 4);
 			*ident = TRUE;
 			break;
 		}
 
 		case SV_POTION_POISON:
 		{
-			if (!(p_ptr->resist_pois || p_ptr->oppose_pois))
+			if (!(p_ptr->resist_pois || p_ptr->timed[TMD_OPP_POIS]))
 			{
-				if (set_poisoned(p_ptr->poisoned + rand_int(15) + 10))
+				if (inc_timed(TMD_POISONED, rand_int(15) + 10))
 				{
 					*ident = TRUE;
 				}
@@ -275,7 +272,7 @@ static bool quaff_potion(object_type *o_ptr, bool *ident)
 		{
 			if (!p_ptr->resist_blind)
 			{
-				if (set_blind(p_ptr->blind + rand_int(100) + 100))
+				if (inc_timed(TMD_BLIND, rand_int(100) + 100))
 				{
 					*ident = TRUE;
 				}
@@ -287,7 +284,7 @@ static bool quaff_potion(object_type *o_ptr, bool *ident)
 		{
 			if (!p_ptr->resist_confu)
 			{
-				if (set_confused(p_ptr->confused + rand_int(20) + 15))
+				if (inc_timed(TMD_CONFUSED, rand_int(20) + 15))
 				{
 					*ident = TRUE;
 				}
@@ -299,7 +296,7 @@ static bool quaff_potion(object_type *o_ptr, bool *ident)
 		{
 			if (!p_ptr->free_act)
 			{
-				if (set_paralyzed(p_ptr->paralyzed + rand_int(4) + 4))
+				if (inc_timed(TMD_PARALYZED, rand_int(4) + 4))
 				{
 					*ident = TRUE;
 				}
@@ -372,8 +369,8 @@ static bool quaff_potion(object_type *o_ptr, bool *ident)
 		{
 			msg_print("Massive explosions rupture your body!");
 			take_hit(damroll(50, 20), "a potion of Detonation");
-			(void)set_stun(p_ptr->stun + 75);
-			(void)set_cut(p_ptr->cut + 5000);
+			(void)inc_timed(TMD_STUN, 75);
+			(void)inc_timed(TMD_CUT, 5000);
 			*ident = TRUE;
 			break;
 		}
@@ -388,7 +385,7 @@ static bool quaff_potion(object_type *o_ptr, bool *ident)
 
 		case SV_POTION_INFRAVISION:
 		{
-			if (set_tim_infra(p_ptr->tim_infra + 100 + randint(100)))
+			if (inc_timed(TMD_SINFRA, 100 + randint(100)))
 			{
 				*ident = TRUE;
 			}
@@ -397,7 +394,7 @@ static bool quaff_potion(object_type *o_ptr, bool *ident)
 
 		case SV_POTION_DETECT_INVIS:
 		{
-			if (set_tim_invis(p_ptr->tim_invis + 12 + randint(12)))
+			if (inc_timed(TMD_SINVIS, 12 + randint(12)))
 			{
 				*ident = TRUE;
 			}
@@ -406,38 +403,38 @@ static bool quaff_potion(object_type *o_ptr, bool *ident)
 
 		case SV_POTION_SLOW_POISON:
 		{
-			if (set_poisoned(p_ptr->poisoned / 2)) *ident = TRUE;
+			if (set_timed(TMD_POISONED, p_ptr->timed[TMD_POISONED] / 2)) *ident = TRUE;
 			break;
 		}
 
 		case SV_POTION_CURE_POISON:
 		{
-			if (set_poisoned(0)) *ident = TRUE;
+			if (clear_timed(TMD_POISONED)) *ident = TRUE;
 			break;
 		}
 
 		case SV_POTION_BOLDNESS:
 		{
-			if (set_afraid(0)) *ident = TRUE;
+			if (clear_timed(TMD_AFRAID)) *ident = TRUE;
 			break;
 		}
 
 		case SV_POTION_SPEED:
 		{
-			if (!p_ptr->fast)
+			if (!p_ptr->timed[TMD_FAST])
 			{
-				if (set_fast(randint(25) + 15)) *ident = TRUE;
+				if (set_timed(TMD_FAST, randint(25) + 15)) *ident = TRUE;
 			}
 			else
 			{
-				(void)set_fast(p_ptr->fast + 5);
+				(void)inc_timed(TMD_FAST, 5);
 			}
 			break;
 		}
 
 		case SV_POTION_RESIST_HEAT:
 		{
-			if (set_oppose_fire(p_ptr->oppose_fire + randint(10) + 10))
+			if (inc_timed(TMD_OPP_FIRE, randint(10) + 10))
 			{
 				*ident = TRUE;
 			}
@@ -446,7 +443,7 @@ static bool quaff_potion(object_type *o_ptr, bool *ident)
 
 		case SV_POTION_RESIST_COLD:
 		{
-			if (set_oppose_cold(p_ptr->oppose_cold + randint(10) + 10))
+			if (inc_timed(TMD_OPP_COLD, randint(10) + 10))
 			{
 				*ident = TRUE;
 			}
@@ -456,66 +453,69 @@ static bool quaff_potion(object_type *o_ptr, bool *ident)
 		case SV_POTION_HEROISM:
 		{
 			if (hp_player(10)) *ident = TRUE;
-			if (set_afraid(0)) *ident = TRUE;
-			if (set_hero(p_ptr->hero + randint(25) + 25)) *ident = TRUE;
+			if (clear_timed(TMD_AFRAID)) *ident = TRUE;
+			if (inc_timed(TMD_HERO, randint(25) + 25)) *ident = TRUE;
 			break;
 		}
 
 		case SV_POTION_BERSERK_STRENGTH:
 		{
 			if (hp_player(30)) *ident = TRUE;
-			if (set_afraid(0)) *ident = TRUE;
-			if (set_shero(p_ptr->shero + randint(25) + 25)) *ident = TRUE;
+			if (clear_timed(TMD_AFRAID)) *ident = TRUE;
+			if (inc_timed(TMD_SHERO, randint(25) + 25)) *ident = TRUE;
 			break;
 		}
 
 		case SV_POTION_CURE_LIGHT:
 		{
 			if (hp_player(damroll(2, 8))) *ident = TRUE;
-			if (set_blind(0)) *ident = TRUE;
-			if (set_cut(p_ptr->cut - 10)) *ident = TRUE;
+			if (clear_timed(TMD_BLIND)) *ident = TRUE;
+			if (dec_timed(TMD_CUT, 10)) *ident = TRUE;
 			break;
 		}
 
 		case SV_POTION_CURE_SERIOUS:
 		{
 			if (hp_player(damroll(4, 8))) *ident = TRUE;
-			if (set_blind(0)) *ident = TRUE;
-			if (set_confused(0)) *ident = TRUE;
-			if (set_cut((p_ptr->cut / 2) - 50)) *ident = TRUE;
+			if (clear_timed(TMD_BLIND)) *ident = TRUE;
+			if (clear_timed(TMD_CONFUSED)) *ident = TRUE;
+			if (set_timed(TMD_CUT, (p_ptr->timed[TMD_CUT] / 2) - 50)) *ident = TRUE;
 			break;
 		}
 
 		case SV_POTION_CURE_CRITICAL:
 		{
 			if (hp_player(damroll(6, 8))) *ident = TRUE;
-			if (set_blind(0)) *ident = TRUE;
-			if (set_confused(0)) *ident = TRUE;
-			if (set_poisoned(0)) *ident = TRUE;
-			if (set_stun(0)) *ident = TRUE;
-			if (set_cut(0)) *ident = TRUE;
+			if (clear_timed(TMD_BLIND)) *ident = TRUE;
+			if (clear_timed(TMD_CONFUSED)) *ident = TRUE;
+			if (clear_timed(TMD_POISONED)) *ident = TRUE;
+			if (clear_timed(TMD_STUN)) *ident = TRUE;
+			if (clear_timed(TMD_CUT)) *ident = TRUE;
+			if (clear_timed(TMD_AMNESIA)) *ident = TRUE;
 			break;
 		}
 
 		case SV_POTION_HEALING:
 		{
 			if (hp_player(300)) *ident = TRUE;
-			if (set_blind(0)) *ident = TRUE;
-			if (set_confused(0)) *ident = TRUE;
-			if (set_poisoned(0)) *ident = TRUE;
-			if (set_stun(0)) *ident = TRUE;
-			if (set_cut(0)) *ident = TRUE;
+			if (clear_timed(TMD_BLIND)) *ident = TRUE;
+			if (clear_timed(TMD_CONFUSED)) *ident = TRUE;
+			if (clear_timed(TMD_POISONED)) *ident = TRUE;
+			if (clear_timed(TMD_STUN)) *ident = TRUE;
+			if (clear_timed(TMD_CUT)) *ident = TRUE;
+			if (clear_timed(TMD_AMNESIA)) *ident = TRUE;
 			break;
 		}
 
 		case SV_POTION_STAR_HEALING:
 		{
 			if (hp_player(1200)) *ident = TRUE;
-			if (set_blind(0)) *ident = TRUE;
-			if (set_confused(0)) *ident = TRUE;
-			if (set_poisoned(0)) *ident = TRUE;
-			if (set_stun(0)) *ident = TRUE;
-			if (set_cut(0)) *ident = TRUE;
+			if (clear_timed(TMD_BLIND)) *ident = TRUE;
+			if (clear_timed(TMD_CONFUSED)) *ident = TRUE;
+			if (clear_timed(TMD_POISONED)) *ident = TRUE;
+			if (clear_timed(TMD_STUN)) *ident = TRUE;
+			if (clear_timed(TMD_CUT)) *ident = TRUE;
+			if (clear_timed(TMD_AMNESIA)) *ident = TRUE;
 			break;
 		}
 
@@ -523,12 +523,13 @@ static bool quaff_potion(object_type *o_ptr, bool *ident)
 		{
 			msg_print("You feel life flow through your body!");
 			restore_level();
-			(void)set_poisoned(0);
-			(void)set_blind(0);
-			(void)set_confused(0);
-			(void)set_image(0);
-			(void)set_stun(0);
-			(void)set_cut(0);
+			(void)clear_timed(TMD_POISONED);
+			(void)clear_timed(TMD_BLIND);
+			(void)clear_timed(TMD_CONFUSED);
+			(void)clear_timed(TMD_IMAGE);
+			(void)clear_timed(TMD_STUN);
+			(void)clear_timed(TMD_CUT);
+			(void)clear_timed(TMD_AMNESIA);
 			(void)do_res_stat(A_STR);
 			(void)do_res_stat(A_CON);
 			(void)do_res_stat(A_DEX);
@@ -670,7 +671,7 @@ static bool quaff_potion(object_type *o_ptr, bool *ident)
 			(void)detect_objects_gold();
 			(void)detect_objects_normal();
 			identify_pack();
-			self_knowledge();
+			self_knowledge(TRUE);
 			*ident = TRUE;
 			break;
 		}
@@ -679,7 +680,7 @@ static bool quaff_potion(object_type *o_ptr, bool *ident)
 		{
 			msg_print("You begin to know yourself a little better...");
 			message_flush();
-			self_knowledge();
+			self_knowledge(TRUE);
 			*ident = TRUE;
 			break;
 		}
@@ -719,7 +720,7 @@ static bool read_scroll(object_type *o_ptr, bool *ident)
 		{
 			if (!p_ptr->resist_blind)
 			{
-				(void)set_blind(p_ptr->blind + 3 + randint(5));
+				(void)inc_timed(TMD_BLIND, 3 + randint(5));
 			}
 			if (unlite_area(10, 3)) *ident = TRUE;
 			break;
@@ -931,19 +932,19 @@ static bool read_scroll(object_type *o_ptr, bool *ident)
 
 		case SV_SCROLL_BLESSING:
 		{
-			if (set_blessed(p_ptr->blessed + randint(12) + 6)) *ident = TRUE;
+			if (inc_timed(TMD_BLESSED, randint(12) + 6)) *ident = TRUE;
 			break;
 		}
 
 		case SV_SCROLL_HOLY_CHANT:
 		{
-			if (set_blessed(p_ptr->blessed + randint(24) + 12)) *ident = TRUE;
+			if (inc_timed(TMD_BLESSED, randint(24) + 12)) *ident = TRUE;
 			break;
 		}
 
 		case SV_SCROLL_HOLY_PRAYER:
 		{
-			if (set_blessed(p_ptr->blessed + randint(48) + 24)) *ident = TRUE;
+			if (inc_timed(TMD_BLESSED, randint(48) + 24)) *ident = TRUE;
 			break;
 		}
 
@@ -961,7 +962,7 @@ static bool read_scroll(object_type *o_ptr, bool *ident)
 		case SV_SCROLL_PROTECTION_FROM_EVIL:
 		{
 			k = 3 * p_ptr->lev;
-			if (set_protevil(p_ptr->protevil + randint(25) + k)) *ident = TRUE;
+			if (inc_timed(TMD_PROTEVIL, randint(25) + k)) *ident = TRUE;
 			break;
 		}
 
@@ -1040,7 +1041,7 @@ static bool use_staff(object_type *o_ptr, bool *ident)
 		{
 			if (!p_ptr->resist_blind)
 			{
-				if (set_blind(p_ptr->blind + 3 + randint(5))) *ident = TRUE;
+				if (inc_timed(TMD_BLIND, 3 + randint(5))) *ident = TRUE;
 			}
 			if (unlite_area(10, 3)) *ident = TRUE;
 			break;
@@ -1048,7 +1049,7 @@ static bool use_staff(object_type *o_ptr, bool *ident)
 
 		case SV_STAFF_SLOWNESS:
 		{
-			if (set_slow(p_ptr->slow + randint(30) + 15)) *ident = TRUE;
+			if (inc_timed(TMD_SLOW, randint(30) + 15)) *ident = TRUE;
 			break;
 		}
 
@@ -1089,7 +1090,7 @@ static bool use_staff(object_type *o_ptr, bool *ident)
 		{
 			if (remove_curse())
 			{
-				if (!p_ptr->blind)
+				if (!p_ptr->timed[TMD_BLIND])
 				{
 					msg_print("The staff glows blue for a moment...");
 				}
@@ -1100,7 +1101,7 @@ static bool use_staff(object_type *o_ptr, bool *ident)
 
 		case SV_STAFF_STARLITE:
 		{
-			if (!p_ptr->blind)
+			if (!p_ptr->timed[TMD_BLIND])
 			{
 				msg_print("The end of the staff glows brightly...");
 			}
@@ -1168,19 +1169,19 @@ static bool use_staff(object_type *o_ptr, bool *ident)
 
 		case SV_STAFF_CURING:
 		{
-			if (set_blind(0)) *ident = TRUE;
-			if (set_poisoned(0)) *ident = TRUE;
-			if (set_confused(0)) *ident = TRUE;
-			if (set_stun(0)) *ident = TRUE;
-			if (set_cut(0)) *ident = TRUE;
+			if (clear_timed(TMD_BLIND)) *ident = TRUE;
+			if (clear_timed(TMD_POISONED)) *ident = TRUE;
+			if (clear_timed(TMD_CONFUSED)) *ident = TRUE;
+			if (clear_timed(TMD_STUN)) *ident = TRUE;
+			if (clear_timed(TMD_CUT)) *ident = TRUE;
 			break;
 		}
 
 		case SV_STAFF_HEALING:
 		{
 			if (hp_player(300)) *ident = TRUE;
-			if (set_stun(0)) *ident = TRUE;
-			if (set_cut(0)) *ident = TRUE;
+			if (clear_timed(TMD_STUN)) *ident = TRUE;
+			if (clear_timed(TMD_CUT)) *ident = TRUE;
 			break;
 		}
 
@@ -1213,13 +1214,13 @@ static bool use_staff(object_type *o_ptr, bool *ident)
 
 		case SV_STAFF_SPEED:
 		{
-			if (!p_ptr->fast)
+			if (!p_ptr->timed[TMD_FAST])
 			{
-				if (set_fast(randint(30) + 15)) *ident = TRUE;
+				if (set_timed(TMD_FAST, randint(30) + 15)) *ident = TRUE;
 			}
 			else
 			{
-				(void)set_fast(p_ptr->fast + 5);
+				(void)inc_timed(TMD_FAST, 5);
 			}
 			break;
 		}
@@ -1247,12 +1248,12 @@ static bool use_staff(object_type *o_ptr, bool *ident)
 		{
 			if (dispel_evil(120)) *ident = TRUE;
 			k = 3 * p_ptr->lev;
-			if (set_protevil(p_ptr->protevil + randint(25) + k)) *ident = TRUE;
-			if (set_poisoned(0)) *ident = TRUE;
-			if (set_afraid(0)) *ident = TRUE;
+			if (inc_timed(TMD_PROTEVIL, randint(25) + k)) *ident = TRUE;
+			if (clear_timed(TMD_POISONED)) *ident = TRUE;
+			if (clear_timed(TMD_AFRAID)) *ident = TRUE;
 			if (hp_player(50)) *ident = TRUE;
-			if (set_stun(0)) *ident = TRUE;
-			if (set_cut(0)) *ident = TRUE;
+			if (clear_timed(TMD_STUN)) *ident = TRUE;
+			if (clear_timed(TMD_CUT)) *ident = TRUE;
 			break;
 		}
 
@@ -1300,10 +1301,10 @@ static bool aim_wand(object_type *o_ptr, bool *ident)
 	lev = k_info[o_ptr->k_idx].level;
 
 	/* Base chance of success */
-	chance = p_ptr->skill_dev;
+	chance = p_ptr->skills[SKILL_DEV];
 
 	/* Confusion hurts skill */
-	if (p_ptr->confused) chance = chance / 2;
+	if (p_ptr->timed[TMD_CONFUSED]) chance = chance / 2;
 
 	/* High level objects are harder */
 	chance = chance - ((lev > 50) ? 50 : lev);
@@ -1599,10 +1600,10 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 	lev = k_info[o_ptr->k_idx].level;
 
 	/* Base chance of success */
-	chance = p_ptr->skill_dev;
+	chance = p_ptr->skills[SKILL_DEV];
 
 	/* Confusion hurts skill */
-	if (p_ptr->confused) chance = chance / 2;
+	if (p_ptr->timed[TMD_CONFUSED]) chance = chance / 2;
 
 	/* High level objects are harder */
 	chance = chance - ((lev > 50) ? 50 : lev);
@@ -1696,19 +1697,19 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 
 		case SV_ROD_CURING:
 		{
-			if (set_blind(0)) *ident = TRUE;
-			if (set_poisoned(0)) *ident = TRUE;
-			if (set_confused(0)) *ident = TRUE;
-			if (set_stun(0)) *ident = TRUE;
-			if (set_cut(0)) *ident = TRUE;
+			if (clear_timed(TMD_BLIND)) *ident = TRUE;
+			if (clear_timed(TMD_POISONED)) *ident = TRUE;
+			if (clear_timed(TMD_CONFUSED)) *ident = TRUE;
+			if (clear_timed(TMD_STUN)) *ident = TRUE;
+			if (clear_timed(TMD_CUT)) *ident = TRUE;
 			break;
 		}
 
 		case SV_ROD_HEALING:
 		{
 			if (hp_player(500)) *ident = TRUE;
-			if (set_stun(0)) *ident = TRUE;
-			if (set_cut(0)) *ident = TRUE;
+			if (clear_timed(TMD_STUN)) *ident = TRUE;
+			if (clear_timed(TMD_CUT)) *ident = TRUE;
 			break;
 		}
 
@@ -1726,13 +1727,13 @@ static bool zap_rod(object_type *o_ptr, bool *ident)
 
 		case SV_ROD_SPEED:
 		{
-			if (!p_ptr->fast)
+			if (!p_ptr->timed[TMD_FAST])
 			{
-				if (set_fast(randint(30) + 15)) *ident = TRUE;
+				if (set_timed(TMD_FAST, randint(30) + 15)) *ident = TRUE;
 			}
 			else
 			{
-				(void)set_fast(p_ptr->fast + 5);
+				(void)inc_timed(TMD_FAST, 5);
 			}
 			break;
 		}
@@ -1909,7 +1910,7 @@ static bool activate_object(object_type *o_ptr, bool *ident)
 			{
 				msg_format("The %s lets out a shrill wail...", o_name);
 				k = 3 * p_ptr->lev;
-				(void)set_protevil(p_ptr->protevil + randint(25) + k);
+				(void)inc_timed(TMD_PROTEVIL, randint(25) + k);
 				break;
 			}
 
@@ -1923,13 +1924,13 @@ static bool activate_object(object_type *o_ptr, bool *ident)
 			case ACT_HASTE2:
 			{
 				msg_format("The %s glows brightly...", o_name);
-				if (!p_ptr->fast)
+				if (!p_ptr->timed[TMD_FAST])
 				{
-					(void)set_fast(randint(75) + 75);
+					(void)set_timed(TMD_FAST, randint(75) + 75);
 				}
 				else
 				{
-					(void)set_fast(p_ptr->fast + 5);
+					(void)inc_timed(TMD_FAST, 5);
 				}
 				break;
 			}
@@ -1978,14 +1979,14 @@ static bool activate_object(object_type *o_ptr, bool *ident)
 			{
 				msg_format("Your %s glows many colours...", o_name);
 				(void)hp_player(30);
-				(void)set_afraid(0);
-				(void)set_shero(p_ptr->shero + randint(50) + 50);
-				(void)set_blessed(p_ptr->blessed + randint(50) + 50);
-				(void)set_oppose_acid(p_ptr->oppose_acid + randint(50) + 50);
-				(void)set_oppose_elec(p_ptr->oppose_elec + randint(50) + 50);
-				(void)set_oppose_fire(p_ptr->oppose_fire + randint(50) + 50);
-				(void)set_oppose_cold(p_ptr->oppose_cold + randint(50) + 50);
-				(void)set_oppose_pois(p_ptr->oppose_pois + randint(50) + 50);
+				(void)clear_timed(TMD_AFRAID);
+				(void)inc_timed(TMD_SHERO, randint(50) + 50);
+				(void)inc_timed(TMD_BLESSED, randint(50) + 50);
+				(void)inc_timed(TMD_OPP_ACID, randint(50) + 50);
+				(void)inc_timed(TMD_OPP_ELEC, randint(50) + 50);
+				(void)inc_timed(TMD_OPP_FIRE, randint(50) + 50);
+				(void)inc_timed(TMD_OPP_COLD, randint(50) + 50);
+				(void)inc_timed(TMD_OPP_POIS, randint(50) + 50);
 				break;
 			}
 
@@ -1994,7 +1995,7 @@ static bool activate_object(object_type *o_ptr, bool *ident)
 				msg_format("Your %s glows a bright white...", o_name);
 				msg_print("You feel much better...");
 				(void)hp_player(1000);
-				(void)set_cut(0);
+				(void)clear_timed(TMD_CUT);
 				break;
 			}
 
@@ -2032,18 +2033,18 @@ static bool activate_object(object_type *o_ptr, bool *ident)
 				msg_format("Your %s glows deep blue...", o_name);
 				msg_print("You feel a warm tingling inside...");
 				(void)hp_player(500);
-				(void)set_cut(0);
+				(void)clear_timed(TMD_CUT);
 				break;
 			}
 
 			case ACT_RESIST:
 			{
 				msg_format("Your %s glows many colours...", o_name);
-				(void)set_oppose_acid(p_ptr->oppose_acid + randint(20) + 20);
-				(void)set_oppose_elec(p_ptr->oppose_elec + randint(20) + 20);
-				(void)set_oppose_fire(p_ptr->oppose_fire + randint(20) + 20);
-				(void)set_oppose_cold(p_ptr->oppose_cold + randint(20) + 20);
-				(void)set_oppose_pois(p_ptr->oppose_pois + randint(20) + 20);
+				(void)inc_timed(TMD_OPP_ACID, randint(20) + 20);
+				(void)inc_timed(TMD_OPP_ELEC, randint(20) + 20);
+				(void)inc_timed(TMD_OPP_FIRE, randint(20) + 20);
+				(void)inc_timed(TMD_OPP_COLD, randint(20) + 20);
+				(void)inc_timed(TMD_OPP_POIS, randint(20) + 20);
 				break;
 			}
 
@@ -2126,13 +2127,13 @@ static bool activate_object(object_type *o_ptr, bool *ident)
 			case ACT_HASTE1:
 			{
 				msg_format("Your %s glows bright green...", o_name);
-				if (!p_ptr->fast)
+				if (!p_ptr->timed[TMD_FAST])
 				{
-					(void)set_fast(randint(20) + 20);
+					(void)set_timed(TMD_FAST, randint(20) + 20);
 				}
 				else
 				{
-					(void)set_fast(p_ptr->fast + 5);
+					(void)inc_timed(TMD_FAST, 5);
 				}
 				break;
 			}
@@ -2140,8 +2141,8 @@ static bool activate_object(object_type *o_ptr, bool *ident)
 			case ACT_REM_FEAR_POIS:
 			{
 				msg_format("Your %s glows deep blue...", o_name);
-				(void)set_afraid(0);
-				(void)set_poisoned(0);
+				(void)clear_timed(TMD_AFRAID);
+				(void)clear_timed(TMD_POISONED);
 				break;
 			}
 
@@ -2212,7 +2213,7 @@ static bool activate_object(object_type *o_ptr, bool *ident)
 			{
 				msg_format("Your %s radiates deep purple...", o_name);
 				hp_player(damroll(4, 8));
-				(void)set_cut((p_ptr->cut / 2) - 50);
+				(void)set_timed(TMD_CUT, (p_ptr->timed[TMD_CUT] / 2) - 50);
 				break;
 			}
 
@@ -2286,7 +2287,7 @@ static bool activate_object(object_type *o_ptr, bool *ident)
 			case ACT_BERSERKER:
 			{
 				msg_format("Your %s glows in anger...", o_name);
-				set_shero(p_ptr->shero + randint(50) + 50);
+				inc_timed(TMD_SHERO, randint(50) + 50);
 				break;
 			}
 		}
@@ -2477,7 +2478,7 @@ static bool activate_object(object_type *o_ptr, bool *ident)
 			case SV_RING_ACID:
 			{
 				fire_ball(GF_ACID, dir, 70, 2);
-				set_oppose_acid(p_ptr->oppose_acid + randint(20) + 20);
+				inc_timed(TMD_OPP_ACID, randint(20) + 20);
 				o_ptr->timeout = rand_int(50) + 50;
 				break;
 			}
@@ -2485,7 +2486,7 @@ static bool activate_object(object_type *o_ptr, bool *ident)
 			case SV_RING_FLAMES:
 			{
 				fire_ball(GF_FIRE, dir, 80, 2);
-				set_oppose_fire(p_ptr->oppose_fire + randint(20) + 20);
+				inc_timed(TMD_OPP_FIRE, randint(20) + 20);
 				o_ptr->timeout = rand_int(50) + 50;
 				break;
 			}
@@ -2493,7 +2494,7 @@ static bool activate_object(object_type *o_ptr, bool *ident)
 			case SV_RING_ICE:
 			{
 				fire_ball(GF_COLD, dir, 75, 2);
-				set_oppose_cold(p_ptr->oppose_cold + randint(20) + 20);
+				inc_timed(TMD_OPP_COLD, randint(20) + 20);
 				o_ptr->timeout = rand_int(50) + 50;
 				break;
 			}
@@ -2501,7 +2502,7 @@ static bool activate_object(object_type *o_ptr, bool *ident)
 			case SV_RING_LIGHTNING:
 			{
 				fire_ball(GF_ELEC, dir, 85, 2);
-				set_oppose_elec(p_ptr->oppose_elec + randint(20) + 20);
+				inc_timed(TMD_OPP_ELEC, randint(20) + 20);
 				o_ptr->timeout = rand_int(50) + 50;
 				break;
 			}
@@ -2772,11 +2773,3 @@ void describe_item_activation(const object_type *o_ptr)
 		}
 	}
 }
-
-#else
-
-#ifdef MACINTOSH
-static int i = 0;
-#endif
-
-#endif /* USE_SCRIPT */
