@@ -12,15 +12,13 @@
 #include "angband.h"
 #include "script.h"
 
-#ifndef USE_SCRIPT
-
 
 /*
- * Maximum number of spells per realm
+ * The defines below must match the spell numbers in spell.txt
+ * if they don't, "interesting" things will probably happen.
+ *
+ * It would be nice if we could get rid of this dependency.
  */
-#define BOOKS_PER_REALM 10
-
-
 #define SPELL_MAGIC_MISSILE             0
 #define SPELL_DETECT_MONSTERS           1
 #define SPELL_PHASE_DOOR                2
@@ -163,456 +161,43 @@
 #define PRAYER_ALTER_REALITY           57
 
 
-/*
- * Spells in each book (mage spells then priest spells)
- */
-static const s16b spell_list[2][BOOKS_PER_REALM][SPELLS_PER_BOOK] =
-{
-	/* Mage spells */
-	{
-		/* Magic for Beginners */
-		{
-			SPELL_MAGIC_MISSILE,
-			SPELL_DETECT_MONSTERS,
-			SPELL_PHASE_DOOR,
-			SPELL_LIGHT_AREA,
-			SPELL_TREASURE_DETECTION,
-			SPELL_CURE_LIGHT_WOUNDS,
-			SPELL_OBJECT_DETECTION,
-			SPELL_FIND_TRAPS_DOORS,
-			SPELL_STINKING_CLOUD,
-		},
-
-		/* Conjurings and Tricks */
-		{
-			SPELL_CONFUSE_MONSTER,
-			SPELL_LIGHTNING_BOLT,
-			SPELL_TRAP_DOOR_DESTRUCTION,
-			SPELL_CURE_POISON,
-			SPELL_SLEEP_MONSTER,
-			SPELL_TELEPORT_SELF,
-			SPELL_SPEAR_OF_LIGHT,
-			SPELL_FROST_BOLT,
-			SPELL_WONDER,
-		},
-
-		/* Incantations and Illusions */
-		{
-			SPELL_SATISFY_HUNGER,
-			SPELL_RECHARGE_ITEM_I,
-			SPELL_TURN_STONE_TO_MUD,
-			SPELL_FIRE_BOLT,
-			SPELL_POLYMORPH_OTHER,
-			SPELL_IDENTIFY,
-			SPELL_DETECT_INVISIBLE,
-			SPELL_ACID_BOLT,
-			SPELL_SLOW_MONSTER,
-		},
-
-		/* Sorcery and Evocations */
-		{
-			SPELL_FROST_BALL,
-			SPELL_TELEPORT_OTHER,
-			SPELL_HASTE_SELF,
-			SPELL_MASS_SLEEP,
-			SPELL_FIRE_BALL,
-			SPELL_DETECT_ENCHANTMENT,
-			-1,
-			-1,
-			-1,
-		},
-
-		/* Resistances of Scarabtarices */
-		{
-			SPELL_RESIST_COLD,
-			SPELL_RESIST_FIRE,
-			SPELL_RESIST_POISON,
-			SPELL_RESISTANCE,
-			SPELL_SHIELD,
-			-1,
-			-1,
-			-1,
-			-1,
-		},
-
-		/* Raal's Tome of Destruction */
-		{
-			SPELL_SHOCK_WAVE,
-			SPELL_EXPLOSION,
-			SPELL_CLOUD_KILL,
-			SPELL_ACID_BALL,
-			SPELL_ICE_STORM,
-			SPELL_METEOR_SWARM,
-			SPELL_RIFT,
-			-1,
-			-1,
-		},
-
-		/* Mordenkainen's Escapes */
-		{
-			SPELL_DOOR_CREATION,
-			SPELL_STAIR_CREATION,
-			SPELL_TELEPORT_LEVEL,
-			SPELL_WORD_OF_RECALL,
-			SPELL_RUNE_OF_PROTECTION,
-			-1,
-			-1,
-			-1,
-			-1,
-		},
-
-		/* Tenser's transformations */
-		{
-			SPELL_HEROISM,
-			SPELL_BERSERKER,
-			SPELL_ENCHANT_ARMOR,
-			SPELL_ENCHANT_WEAPON,
-			SPELL_RECHARGE_ITEM_II,
-			SPELL_ELEMENTAL_BRAND,
-			-1,
-			-1,
-			-1,
-		},
-
-		/* Kelek's Grimoire of Power */
-		{
-			SPELL_EARTHQUAKE,
-			SPELL_BEDLAM,
-			SPELL_REND_SOUL,
-			SPELL_BANISHMENT,
-			SPELL_WORD_OF_DESTRUCTION,
-			SPELL_MASS_BANISHMENT,
-			SPELL_CHAOS_STRIKE,
-			SPELL_MANA_STORM,
-			-1,
-		},
-	},
-
-	/* Priest spells */
-	{
-		/* Beginners Handbook */
-		{
-			PRAYER_DETECT_EVIL,
-			PRAYER_CURE_LIGHT_WOUNDS,
-			PRAYER_BLESS,
-			PRAYER_REMOVE_FEAR,
-			PRAYER_CALL_LIGHT,
-			PRAYER_FIND_TRAPS,
-			PRAYER_DETECT_DOORS_STAIRS,
-			PRAYER_SLOW_POISON,
-			-1,
-		},
-
-		/* Words of Wisdom */
-		{
-			PRAYER_SCARE_MONSTER,
-			PRAYER_PORTAL,
-			PRAYER_CURE_SERIOUS_WOUNDS,
-			PRAYER_CHANT,
-			PRAYER_SANCTUARY,
-			PRAYER_SATISFY_HUNGER,
-			PRAYER_REMOVE_CURSE,
-			PRAYER_RESIST_HEAT_COLD,
-			-1,
-		},
-
-		/* Chants and Blessings */
-		{
-			PRAYER_NEUTRALIZE_POISON,
-			PRAYER_ORB_OF_DRAINING,
-			PRAYER_CURE_CRITICAL_WOUNDS,
-			PRAYER_SENSE_INVISIBLE,
-			PRAYER_PROTECTION_FROM_EVIL,
-			PRAYER_EARTHQUAKE,
-			PRAYER_SENSE_SURROUNDINGS,
-			PRAYER_CURE_MORTAL_WOUNDS,
-			PRAYER_TURN_UNDEAD,
-		},
-
-		/* Exorcism and Dispelling */
-		{
-			PRAYER_PRAYER,
-			PRAYER_DISPEL_UNDEAD,
-			PRAYER_HEAL,
-			PRAYER_DISPEL_EVIL,
-			PRAYER_GLYPH_OF_WARDING,
-			PRAYER_HOLY_WORD,
-			-1,
-			-1,
-			-1,
-		},
-
-		/* Ethereal openings */
-		{
-			PRAYER_BLINK,
-			PRAYER_TELEPORT_SELF,
-			PRAYER_TELEPORT_OTHER,
-			PRAYER_TELEPORT_LEVEL,
-			PRAYER_WORD_OF_RECALL,
-			PRAYER_ALTER_REALITY,
-			-1,
-			-1,
-			-1,
-		},
-
-		/* Godly Insights */
-		{
-			PRAYER_DETECT_MONSTERS,
-			PRAYER_DETECTION,
-			PRAYER_PERCEPTION,
-			PRAYER_PROBING,
-			PRAYER_CLAIRVOYANCE,
-			-1,
-			-1,
-			-1,
-			-1,
-		},
-
-		/* Purifications and Healing */
-		{
-			PRAYER_CURE_SERIOUS_WOUNDS2,
-			PRAYER_CURE_MORTAL_WOUNDS2,
-			PRAYER_HEALING,
-			PRAYER_RESTORATION,
-			PRAYER_REMEMBRANCE,
-			-1,
-			-1,
-			-1,
-			-1,
-		},
-
-		/* Holy Infusions */
-		{
-			PRAYER_UNBARRING_WAYS,
-			PRAYER_RECHARGING,
-			PRAYER_DISPEL_CURSE,
-			PRAYER_ENCHANT_WEAPON,
-			PRAYER_ENCHANT_ARMOUR,
-			PRAYER_ELEMENTAL_BRAND,
-			-1,
-			-1,
-			-1,
-		},
-
-		/* Wrath of God */
-		{
-			PRAYER_DISPEL_UNDEAD2,
-			PRAYER_DISPEL_EVIL2,
-			PRAYER_BANISH_EVIL,
-			PRAYER_WORD_OF_DESTRUCTION,
-			PRAYER_ANNIHILATION,
-			-1,
-			-1,
-			-1,
-			-1,
-		}
-	}
-};
-
-
-/*
- * Names of the spells (mage spells then priest spells)
- */
-static cptr spell_names[2][PY_MAX_SPELLS] =
-{
-	/*** Mage Spells ***/
-
-	{
-	  "Magic Missile",
-	  "Detect Monsters",
-	  "Phase Door",
-	  "Light Area",
-	  "Find Hidden Traps/Doors",
-	  "Cure Light Wounds",
-	  "Detect Treasure",
-	  "Detect Objects",
-	  "Identify",
-	  "Detect Invisible",
-	  "Detect Enchantment",
-	  "Stinking Cloud",
-	  "Lightning Bolt",
-	  "Confuse Monster",
-	  "Sleep Monster",
-	  "Wonder",
-	  "Frost Bolt",
-	  "Acid Bolt",
-	  "Fire Bolt",
-	  "Trap/Door Destruction",
-	  "Spear of Light",
-	  "Turn Stone to Mud",
-	  "Door Creation",
-	  "Earthquake",
-	  "Stair Creation",
-	  "Cure Poison",
-	  "Satisfy Hunger",
-	  "Heroism",
-	  "Berserker",
-	  "Haste Self",
-	  "Teleport Self",
-	  "Slow Monster",
-	  "Teleport Other",
-	  "Teleport Level",
-	  "Word of Recall",
-	  "Polymorph Other",
-	  "Shock Wave",
-	  "Explosion",
-	  "Cloudkill",
-	  "Mass Sleep",
-	  "Bedlam",
-	  "Rend Soul",
-	  "Word of Destruction",
-	  "Chaos Strike",
-	  "Resist Cold",
-	  "Resist Fire",
-	  "Resist Poison",
-	  "Resistance",
-	  "Shield",
-	  "Rune of Protection",
-	  "Lesser Recharging",
-	  "Enchant Armor",
-	  "Enchant Weapon",
-	  "Greater Recharging",
-	  "Elemental Brand",
-	  "Frost Ball",
-	  "Acid Ball",
-	  "Fire Ball",
-	  "Ice Storm",
-	  "Banishment",
-	  "Meteor Swarm",
-	  "Mass Banishment",
-	  "Rift",
-	  "Mana Storm"
-	},
-
-
-	/*** Priest Spells ***/
-
-	{
-		/* Beginners Handbook (sval 0) */
-		"Detect Evil",
-		"Cure Light Wounds",
-		"Bless",
-		"Remove Fear",
-		"Call Light",
-		"Find Traps",
-		"Detect Doors/Stairs",
-		"Slow Poison",
-
-		/* Words of Wisdom (sval 1) */
-		"Scare Monster",
-		"Portal",
-		"Cure Serious Wounds",
-		"Chant",
-		"Sanctuary",
-		"Satisfy Hunger",
-		"Remove Curse",
-		"Resist Heat and Cold",
-
-		/* Chants and Blessings (sval 2) */
-		"Neutralize Poison",
-		"Orb of Draining",
-		"Cure Critical Wounds",
-		"Sense Invisible",
-		"Protection from Evil",
-		"Earthquake",
-		"Sense Surroundings",
-		"Cure Mortal Wounds",
-		"Turn Undead",
-
-		/* Exorcism and Dispelling (sval 3) */
-		"Prayer",
-		"Dispel Undead",
-		"Heal",
-		"Dispel Evil",
-		"Glyph of Warding",
-		"Holy Word",
-
-		/* Godly Insights... (sval 5) */
-		"Detect Monsters",
-		"Detection",
-		"Perception",
-		"Probing",
-		"Clairvoyance",
-
-		/* Purifications and Healing (sval 6) */
-		"Cure Serious Wounds",
-		"Cure Mortal Wounds",
-		"Healing",
-		"Restoration",
-		"Remembrance",
-
-		/* Wrath of God (sval 8) */
-		"Dispel Undead",
-		"Dispel Evil",
-		"Banish Evil",
-		"Word of Destruction",
-		"Annihilation",
-
-		/* Holy Infusions (sval 7) */
-		"Unbarring Ways",
-		"Recharging",
-		"Dispel Curse",
-		"Enchant Weapon",
-		"Enchant Armour",
-		"Elemental Brand",
-
-		/* Ethereal openings (sval 4) */
-		"Blink",
-		"Teleport Self",
-		"Teleport Other",
-		"Teleport Level",
-		"Word of Recall",
-		"Alter Reality",
-
-		"(blank)",
-		"(blank)",
-		"(blank)",
-		"(blank)",
-		"(blank)",
-		"(blank)"
-	}
-};
-
 
 int get_spell_index(const object_type *o_ptr, int index)
 {
-	int spell;
-	int num = 0;
-
-	/* Get the item's sval */
+	int realm, spell;
 	int sval = o_ptr->sval;
 
-	int spell_type;
-
 	/* Check bounds */
-	if ((index < 0) || (index >= SPELLS_PER_BOOK)) return (-1);
-	if ((sval < 0) || (sval >= BOOKS_PER_REALM)) return (-1);
+	if ((index < 0) || (index >= SPELLS_PER_BOOK)) return -1;
+	if ((sval < 0) || (sval >= BOOKS_PER_REALM)) return -1;
 
 	/* Mage or priest spells? */
 	if (cp_ptr->spell_book == TV_MAGIC_BOOK)
-		spell_type = 0;
+		realm = 0;
 	else
-		spell_type = 1;
+		realm = 1;
 
-	return spell_list[spell_type][sval][index];
+	/* Get the spell */
+	spell = spell_list[realm][sval][index];
+	if (spell == -1) return -1;
+
+	return s_info[spell].spell_index;
 }
 
 
 cptr get_spell_name(int tval, int spell)
 {
 	if (tval == TV_MAGIC_BOOK)
-		return spell_names[0][spell];
+		return s_name + s_info[spell].name;
 	else
-		return spell_names[1][spell];
+		return s_name + s_info[spell + PY_MAX_SPELLS].name;
 }
 
 
-cptr get_spell_info(int tval, int spell)
+void get_spell_info(int tval, int spell, char *p, size_t len)
 {
-	static char p[80];
-
-	/* Default */
-	strcpy(p, "");
+	/* Blank 'p' first */
+	p[0] = '\0';
 
 	/* Mage spells */
 	if (tval == TV_MAGIC_BOOK)
@@ -623,94 +208,97 @@ cptr get_spell_info(int tval, int spell)
 		switch (spell)
 		{
 		case SPELL_MAGIC_MISSILE:
-			sprintf(p, " dam %dd4", 3 + ((plev - 1) / 5));
+			strnfmt(p, len, " dam %dd4", 3 + ((plev - 1) / 5));
 			break;
 		case SPELL_PHASE_DOOR:
-			sprintf(p, " range 10");
+			strnfmt(p, len, " range 10");
 			break;
+		case SPELL_LIGHT_AREA:
+			strnfmt(p, len, " dam 2d%d", (plev / 2));
+			break; 
 		case SPELL_CURE_LIGHT_WOUNDS:
-			sprintf(p, " heal 2d8");
+			strnfmt(p, len, " heal 2d8");
 			break;
 		case SPELL_STINKING_CLOUD:
-			sprintf(p, " dam %d", 10 + (plev / 2));
+			strnfmt(p, len, " dam %d", 10 + (plev / 2));
 			break;
 		case SPELL_LIGHTNING_BOLT:
-			sprintf(p, " dam %dd6", (3 + ((plev - 5) / 6)));
+			strnfmt(p, len, " dam %dd6", (3 + ((plev - 5) / 6)));
 			break;
 		case SPELL_FROST_BOLT:
-			sprintf(p, " dam %dd8", (5 + ((plev - 5) / 4)));
+			strnfmt(p, len, " dam %dd8", (5 + ((plev - 5) / 4)));
 			break;
 		case SPELL_ACID_BOLT:
-			sprintf(p, " dam %dd8", (8 + ((plev - 5) / 4)));
+			strnfmt(p, len, " dam %dd8", (8 + ((plev - 5) / 4)));
 			break;
 		case SPELL_FIRE_BOLT:
-			sprintf(p, " dam %dd8", (6 + ((plev - 5) / 4)));
+			strnfmt(p, len, " dam %dd8", (6 + ((plev - 5) / 4)));
 			break;
 		case SPELL_SPEAR_OF_LIGHT:
-			sprintf(p, " dam 6d8");
+			strnfmt(p, len, " dam 6d8");
 			break;
 		case SPELL_HEROISM:
-			sprintf(p, " dur 25+d25");
+			strnfmt(p, len, " dur 25+d25");
 			break;
 		case SPELL_BERSERKER:
-			sprintf(p, " dur 25+d25");
+			strnfmt(p, len, " dur 25+d25");
 			break;
 		case SPELL_HASTE_SELF:
-			sprintf(p, " dur %d+d20", plev);
+			strnfmt(p, len, " dur %d+d20", plev);
 			break;
 		case SPELL_TELEPORT_SELF:
-			sprintf(p, " range %d", plev * 5);
+			strnfmt(p, len, " range %d", plev * 5);
 			break;
 		case SPELL_SHOCK_WAVE:
-			sprintf(p, " dam %d", 10 + plev);
+			strnfmt(p, len, " dam %d", 10 + plev);
 			break;
 		case SPELL_EXPLOSION:
-			sprintf(p, " dam %d", 20 + plev * 2);
+			strnfmt(p, len, " dam %d", 20 + plev * 2);
 			break;
 		case SPELL_CLOUD_KILL:
-			sprintf(p, " dam %d", 40 + (plev / 2));
+			strnfmt(p, len, " dam %d", 40 + (plev / 2));
 			break;
 		case SPELL_REND_SOUL:
-			sprintf(p, " dam 11d%d", plev);
+			strnfmt(p, len, " dam 11d%d", plev);
 			break;
 		case SPELL_CHAOS_STRIKE:
-			sprintf(p, " dam 13d%d", plev);
+			strnfmt(p, len, " dam 13d%d", plev);
 			break;
 		case SPELL_RESIST_COLD:
-			sprintf(p, " dur 20+d20");
+			strnfmt(p, len, " dur 20+d20");
 			break;
 		case SPELL_RESIST_FIRE:
-			sprintf(p, " dur 20+d20");
+			strnfmt(p, len, " dur 20+d20");
 			break;
 		case SPELL_RESIST_POISON:
-			sprintf(p, " dur 20+d20");
+			strnfmt(p, len, " dur 20+d20");
 			break;
 		case SPELL_RESISTANCE:
-			sprintf(p, " dur 20+d20");
+			strnfmt(p, len, " dur 20+d20");
 			break;
 		case SPELL_SHIELD:
-			sprintf(p, " dur 30+d20");
+			strnfmt(p, len, " dur 30+d20");
 			break;
 		case SPELL_FROST_BALL:
-			sprintf(p, " dam %d", 30 + plev);
+			strnfmt(p, len, " dam %d", 30 + plev);
 			break;
 		case SPELL_ACID_BALL:
-			sprintf(p, " dam %d", 40 + plev);
+			strnfmt(p, len, " dam %d", 40 + plev);
 			break;
 		case SPELL_FIRE_BALL:
-			sprintf(p, " dam %d", 55 + plev);
+			strnfmt(p, len, " dam %d", 55 + plev);
 			break;
 		case SPELL_ICE_STORM:
-			sprintf(p, " dam %d", 50 + (plev * 2));
+			strnfmt(p, len, " dam %d", 50 + (plev * 2));
 			break;
 		case SPELL_METEOR_SWARM:
-			sprintf(p, " dam %dx%d", 30 + plev / 2, 2 + plev / 20);
+			strnfmt(p, len, " dam %dx%d", 30 + plev / 2, 2 + plev / 20);
 			break;
 		case SPELL_RIFT:
-			sprintf(p, " dam 40+%dd7", plev);
+			strnfmt(p, len, " dam 40+%dd7", plev);
 			break;
 		case SPELL_MANA_STORM:
-			sprintf(p, " dam %d", 300 + plev * 2);
+			strnfmt(p, len, " dam %d", 300 + plev * 2);
 			break;
 		}
 	}
@@ -724,82 +312,85 @@ cptr get_spell_info(int tval, int spell)
 		switch (spell)
 		{
 			case PRAYER_CURE_LIGHT_WOUNDS:
-				strcpy(p, " heal 2d10");
+				my_strcpy(p, " heal 2d10", len);
 				break;
 			case PRAYER_BLESS:
-				strcpy(p, " dur 12+d12");
+				my_strcpy(p, " dur 12+d12", len);
 				break;
+			case PRAYER_CALL_LIGHT:
+				strnfmt(p, len, " dam 2d%d", (plev / 2));
+				break; 
 			case PRAYER_PORTAL:
-				sprintf(p, " range %d", 3 * plev);
+				strnfmt(p, len, " range %d", 3 * plev);
 				break;
 			case PRAYER_CURE_SERIOUS_WOUNDS:
-				strcpy(p, " heal 4d10");
+				my_strcpy(p, " heal 4d10", len);
 				break;
 			case PRAYER_CHANT:
-				strcpy(p, " dur 24+d24");
+				my_strcpy(p, " dur 24+d24", len);
 				break;
 			case PRAYER_RESIST_HEAT_COLD:
-				strcpy(p, " dur 10+d10");
+				my_strcpy(p, " dur 10+d10", len);
 				break;
 			case PRAYER_ORB_OF_DRAINING:
-				sprintf(p, " %d+3d6", plev +
+				strnfmt(p, len, " %d+3d6", plev +
 				        (plev / ((cp_ptr->flags & CF_BLESS_WEAPON) ? 2 : 4)));
 				break;
 			case PRAYER_CURE_CRITICAL_WOUNDS:
-				strcpy(p, " heal 6d10");
+				my_strcpy(p, " heal 6d10", len);
 				break;
 			case PRAYER_SENSE_INVISIBLE:
-				strcpy(p, " dur 24+d24");
+				my_strcpy(p, " dur 24+d24", len);
 				break;
 			case PRAYER_PROTECTION_FROM_EVIL:
-				sprintf(p, " dur %d+d25", 3 * plev);
+				strnfmt(p, len, " dur %d+d25", 3 * plev);
 				break;
 			case PRAYER_CURE_MORTAL_WOUNDS:
-				strcpy(p, " heal 8d10");
+				my_strcpy(p, " heal 8d10", len);
 				break;
 			case PRAYER_PRAYER:
-				strcpy(p, " dur 48+d48");
+				my_strcpy(p, " dur 48+d48", len);
 				break;
 			case PRAYER_DISPEL_UNDEAD:
-				sprintf(p, " dam d%d", 3 * plev);
+				strnfmt(p, len, " dam d%d", 3 * plev);
 				break;
 			case PRAYER_HEAL:
-				strcpy(p, " heal 300");
+				my_strcpy(p, " heal 300", len);
 				break;
 			case PRAYER_DISPEL_EVIL:
-				sprintf(p, " dam d%d", 3 * plev);
+				strnfmt(p, len, " dam d%d", 3 * plev);
 				break;
 			case PRAYER_HOLY_WORD:
-				strcpy(p, " heal 1000");
+				my_strcpy(p, " heal 1000", len);
 				break;
 			case PRAYER_CURE_SERIOUS_WOUNDS2:
-				strcpy(p, " heal 4d10");
+				my_strcpy(p, " heal 4d10", len);
 				break;
 			case PRAYER_CURE_MORTAL_WOUNDS2:
-				strcpy(p, " heal 8d10");
+				my_strcpy(p, " heal 8d10", len);
 				break;
 			case PRAYER_HEALING:
-				strcpy(p, " heal 2000");
+				my_strcpy(p, " heal 2000", len);
 				break;
 			case PRAYER_DISPEL_UNDEAD2:
-				sprintf(p, " dam d%d", 4 * plev);
+				strnfmt(p, len, " dam d%d", 4 * plev);
 				break;
 			case PRAYER_DISPEL_EVIL2:
-				sprintf(p, " dam d%d", 4 * plev);
+				strnfmt(p, len, " dam d%d", 4 * plev);
 				break;
 			case PRAYER_ANNIHILATION:
-				strcpy(p, " dam 200");
+				my_strcpy(p, " dam 200", len);
 				break;
 			case PRAYER_BLINK:
-				strcpy(p, " range 10");
+				my_strcpy(p, " range 10", len);
 				break;
 			case PRAYER_TELEPORT_SELF:
-				sprintf(p, " range %d", 8 * plev);
+				strnfmt(p, len, " range %d", 8 * plev);
 				break;
 		}
 	}
 
-	return (p);
+	return;
 }
 
 
@@ -921,7 +512,7 @@ static bool cast_mage_spell(int spell)
 		{
 
 			(void)hp_player(damroll(2, 8));
-			(void)set_cut(p_ptr->cut - 15);
+			(void)dec_timed(TMD_CUT, 15);
 			break;
 		}
 
@@ -976,7 +567,7 @@ static bool cast_mage_spell(int spell)
 
 		case SPELL_CURE_POISON:
 		{
-			(void)set_poisoned(0);
+			(void)clear_timed(TMD_POISONED);
 			break;
 		}
 
@@ -1210,13 +801,13 @@ static bool cast_mage_spell(int spell)
 
 		case SPELL_RESIST_FIRE:
 		{
-			(void)set_oppose_fire(p_ptr->oppose_fire + randint(20) + 20);
+			(void)inc_timed(TMD_OPP_FIRE, randint(20) + 20);
 			break;
 		}
 
 		case SPELL_RESIST_COLD:
 		{
-			(void)set_oppose_cold(p_ptr->oppose_cold + randint(20) + 20);
+			(void)inc_timed(TMD_OPP_COLD, randint(20) + 20);
 			break;
 		}
 
@@ -1227,52 +818,52 @@ static bool cast_mage_spell(int spell)
 
 		case SPELL_RESIST_POISON:
 		{
-			(void)set_oppose_pois(p_ptr->oppose_pois + randint(20) + 20);
+			(void)inc_timed(TMD_OPP_POIS, randint(20) + 20);
 			break;
 		}
 
 		case SPELL_RESISTANCE:
 		{
 			int time = randint(20) + 20;
-			(void)set_oppose_acid(p_ptr->oppose_acid + time);
-			(void)set_oppose_elec(p_ptr->oppose_elec + time);
-			(void)set_oppose_fire(p_ptr->oppose_fire + time);
-			(void)set_oppose_cold(p_ptr->oppose_cold + time);
-			(void)set_oppose_pois(p_ptr->oppose_pois + time);
+			(void)inc_timed(TMD_OPP_ACID, time);
+			(void)inc_timed(TMD_OPP_ELEC, time);
+			(void)inc_timed(TMD_OPP_FIRE, time);
+			(void)inc_timed(TMD_OPP_COLD, time);
+			(void)inc_timed(TMD_OPP_POIS, time);
 			break;
 		}
 
 		case SPELL_HEROISM:
 		{
 			(void)hp_player(10);
-			(void)set_hero(p_ptr->hero + randint(25) + 25);
-			(void)set_afraid(0);
+			(void)inc_timed(TMD_HERO, randint(25) + 25);
+			(void)clear_timed(TMD_AFRAID);
 			break;
 		}
 
 		case SPELL_SHIELD:
 		{
-			(void)set_shield(p_ptr->shield + randint(20) + 30);
+			(void)inc_timed(TMD_SHIELD, randint(20) + 30);
 			break;
 		}
 
 		case SPELL_BERSERKER:
 		{
 			(void)hp_player(30);
-			(void)set_shero(p_ptr->shero + randint(25) + 25);
-			(void)set_afraid(0);
+			(void)inc_timed(TMD_SHERO, randint(25) + 25);
+			(void)clear_timed(TMD_AFRAID);
 			break;
 		}
 
 		case SPELL_HASTE_SELF:
 		{
-			if (!p_ptr->fast)
+			if (!p_ptr->timed[TMD_FAST])
 			{
-				(void)set_fast(randint(20) + plev);
+				(void)set_timed(TMD_FAST, randint(20) + plev);
 			}
 			else
 			{
-				(void)set_fast(p_ptr->fast + randint(5));
+				(void)inc_timed(TMD_FAST, randint(5));
 			}
 			break;
 		}
@@ -1341,19 +932,19 @@ static bool cast_priest_spell(int spell)
 		case PRAYER_CURE_LIGHT_WOUNDS:
 		{
 			(void)hp_player(damroll(2, 10));
-			(void)set_cut(p_ptr->cut - 10);
+			(void)dec_timed(TMD_CUT, 10);
 			break;
 		}
 
 		case PRAYER_BLESS:
 		{
-			(void)set_blessed(p_ptr->blessed + randint(12) + 12);
+			(void)inc_timed(TMD_BLESSED, randint(12) + 12);
 			break;
 		}
 
 		case PRAYER_REMOVE_FEAR:
 		{
-			(void)set_afraid(0);
+			(void)clear_timed(TMD_AFRAID);
 			break;
 		}
 
@@ -1378,7 +969,7 @@ static bool cast_priest_spell(int spell)
 
 		case PRAYER_SLOW_POISON:
 		{
-			(void)set_poisoned(p_ptr->poisoned / 2);
+			(void)set_timed(TMD_POISONED, p_ptr->timed[TMD_POISONED] / 2);
 			break;
 		}
 
@@ -1398,13 +989,13 @@ static bool cast_priest_spell(int spell)
 		case PRAYER_CURE_SERIOUS_WOUNDS:
 		{
 			(void)hp_player(damroll(4, 10));
-			(void)set_cut((p_ptr->cut / 2) - 20);
+			(void)set_timed(TMD_CUT, (p_ptr->timed[TMD_CUT] / 2) - 20);
 			break;
 		}
 
 		case PRAYER_CHANT:
 		{
-			(void)set_blessed(p_ptr->blessed + randint(24) + 24);
+			(void)inc_timed(TMD_BLESSED, randint(24) + 24);
 			break;
 		}
 
@@ -1428,14 +1019,14 @@ static bool cast_priest_spell(int spell)
 
 		case PRAYER_RESIST_HEAT_COLD:
 		{
-			(void)set_oppose_fire(p_ptr->oppose_fire + randint(10) + 10);
-			(void)set_oppose_cold(p_ptr->oppose_cold + randint(10) + 10);
+			(void)inc_timed(TMD_OPP_FIRE, randint(10) + 10);
+			(void)inc_timed(TMD_OPP_COLD, randint(10) + 10);
 			break;
 		}
 
 		case PRAYER_NEUTRALIZE_POISON:
 		{
-			(void)set_poisoned(0);
+			(void)clear_timed(TMD_POISONED);
 			break;
 		}
 
@@ -1452,19 +1043,20 @@ static bool cast_priest_spell(int spell)
 		case PRAYER_CURE_CRITICAL_WOUNDS:
 		{
 			(void)hp_player(damroll(6, 10));
-			(void)set_cut(0);
+			(void)clear_timed(TMD_CUT);
+			(void)clear_timed(TMD_AMNESIA);
 			break;
 		}
 
 		case PRAYER_SENSE_INVISIBLE:
 		{
-			(void)set_tim_invis(p_ptr->tim_invis + randint(24) + 24);
+			(void)inc_timed(TMD_SINVIS, randint(24) + 24);
 			break;
 		}
 
 		case PRAYER_PROTECTION_FROM_EVIL:
 		{
-			(void)set_protevil(p_ptr->protevil + randint(25) + 3 * p_ptr->lev);
+			(void)inc_timed(TMD_PROTEVIL, randint(25) + 3 * p_ptr->lev);
 			break;
 		}
 
@@ -1483,8 +1075,8 @@ static bool cast_priest_spell(int spell)
 		case PRAYER_CURE_MORTAL_WOUNDS:
 		{
 			(void)hp_player(damroll(8, 10));
-			(void)set_stun(0);
-			(void)set_cut(0);
+			(void)clear_timed(TMD_STUN);
+			(void)clear_timed(TMD_CUT);
 			break;
 		}
 
@@ -1496,7 +1088,7 @@ static bool cast_priest_spell(int spell)
 
 		case PRAYER_PRAYER:
 		{
-			(void)set_blessed(p_ptr->blessed + randint(48) + 48);
+			(void)inc_timed(TMD_BLESSED, randint(48) + 48);
 			break;
 		}
 
@@ -1509,8 +1101,8 @@ static bool cast_priest_spell(int spell)
 		case PRAYER_HEAL:
 		{
 			(void)hp_player(300);
-			(void)set_stun(0);
-			(void)set_cut(0);
+			(void)clear_timed(TMD_STUN);
+			(void)clear_timed(TMD_CUT);
 			break;
 		}
 
@@ -1530,10 +1122,10 @@ static bool cast_priest_spell(int spell)
 		{
 			(void)dispel_evil(randint(plev * 4));
 			(void)hp_player(1000);
-			(void)set_afraid(0);
-			(void)set_poisoned(0);
-			(void)set_stun(0);
-			(void)set_cut(0);
+			(void)clear_timed(TMD_AFRAID);
+			(void)clear_timed(TMD_POISONED);
+			(void)clear_timed(TMD_STUN);
+			(void)clear_timed(TMD_CUT);
 			break;
 		}
 
@@ -1569,23 +1161,23 @@ static bool cast_priest_spell(int spell)
 		case PRAYER_CURE_SERIOUS_WOUNDS2:
 		{
 			(void)hp_player(damroll(4, 10));
-			(void)set_cut(0);
+			(void)clear_timed(TMD_CUT);
 			break;
 		}
 
 		case PRAYER_CURE_MORTAL_WOUNDS2:
 		{
 			(void)hp_player(damroll(8, 10));
-			(void)set_stun(0);
-			(void)set_cut(0);
+			(void)clear_timed(TMD_STUN);
+			(void)clear_timed(TMD_CUT);
 			break;
 		}
 
 		case PRAYER_HEALING:
 		{
 			(void)hp_player(2000);
-			(void)set_stun(0);
-			(void)set_cut(0);
+			(void)clear_timed(TMD_STUN);
+			(void)clear_timed(TMD_CUT);
 			break;
 		}
 
@@ -1731,5 +1323,3 @@ bool cast_spell(int tval, int index)
 		return cast_priest_spell(index);
 	}
 }
-
-#endif /* USE_SCRIPT */
