@@ -27,6 +27,9 @@
  *	- Array[h*w] -- Attribute array
  *	- Array[h*w] -- Character array
  *
+ *	- next screen saved
+ *	- hook to be called on screen size change
+ *
  * Note that the attr/char pair at (x,y) is a[y][x]/c[y][x]
  * and that the row of attr/chars at (0,y) is a[y]/c[y]
  */
@@ -49,9 +52,53 @@ struct term_win
 
 	byte *vta;
 	char *vtc;
+
+	term_win *next;
 };
 
 
+/*
+ * Event record for general input
+ * Note that there are currently no event subtypes.
+ */
+
+typedef enum 
+{
+	EVT_NONE		= 0x0,
+	EVT_ESCAPE		= 0x0001,		/* Synonym for KBRD + key = ESCAPE */
+	EVT_KBRD		= 0x0002,		/* keypress */
+	EVT_MOUSE		= 0x0004,		/* mousepress */
+	EVT_BACK		= 0x0008,		/* Up one level in heirarchical menus. */
+	EVT_MOVE		= 0x0010,		/* menu movement */
+	EVT_SELECT		= 0x0020,		/* Menu selection */
+	EVT_BUTTON		= 0x0040,		/* button press */
+	EVT_CMD			= 0x0080,		/* Command key execute */
+	EVT_OK			= 0x0100,		/* Callback successful */
+									/* For example, a command key action. */
+	EVT_REFRESH		= 0x0200,		/* Display refresh */
+	EVT_RESIZE		= 0x0400,		/* Display resize */
+
+	EVT_AGAIN		= 0x4000000,	/* Retry notification */
+	EVT_STOP		= 0x8000000		/* Loop stopped (never handled) */
+
+} event_class;
+
+#ifdef RISCOS
+#define event_type ang_event_type
+#endif
+
+typedef struct event_type event_type;
+
+struct event_type
+{
+	event_class type;
+	byte mousex, mousey;
+	char key; 
+	short index;
+};
+
+#define EVENT_EMPTY		{ EVT_NONE, 0, 0, 0, 0 }
+
 
 /*
  * An actual "term" structure
@@ -181,7 +228,7 @@ struct term
 	byte attr_blank;
 	char char_blank;
 
-	char *key_queue;
+	event_type *key_queue;
 
 	u16b key_head;
 	u16b key_tail;
@@ -229,8 +276,6 @@ struct term
 
 
 
-
-
 /**** Available Constants ****/
 
 
@@ -244,7 +289,6 @@ struct term
  * The "TERM_XTRA_EVENT" action uses "v" to "wait" for an event
  * The "TERM_XTRA_SHAPE" action uses "v" to "show" the cursor
  * The "TERM_XTRA_FROSH" action uses "v" for the index of the row
- * The "TERM_XTRA_SOUND" action uses "v" for the index of a sound
  * The "TERM_XTRA_ALIVE" action uses "v" to "activate" (or "close")
  * The "TERM_XTRA_LEVEL" action uses "v" to "resume" (or "suspend")
  * The "TERM_XTRA_DELAY" action uses "v" as a "millisecond" value
@@ -258,7 +302,6 @@ struct term
 #define TERM_XTRA_FROSH 5	/* Flush one row (optional) */
 #define TERM_XTRA_FRESH 6	/* Flush all rows (optional) */
 #define TERM_XTRA_NOISE 7	/* Make a noise (optional) */
-#define TERM_XTRA_SOUND 8	/* Make a sound (optional) */
 #define TERM_XTRA_BORED 9	/* Handle stuff when bored (optional) */
 #define TERM_XTRA_REACT 10	/* React to global changes (optional) */
 #define TERM_XTRA_ALIVE 11	/* Change the "hard" level (optional) */
@@ -266,6 +309,37 @@ struct term
 #define TERM_XTRA_DELAY 13	/* Delay some milliseconds (optional) */
 
 
+/*
+ * Angband "attributes" (with symbols, and base (R,G,B) codes)
+ *
+ * The "(R,G,B)" codes are given in "fourths" of the "maximal" value,
+ * and should "gamma corrected" on most (non-Macintosh) machines.
+ */
+#define TERM_DARK       0   /* 'd' */   /* 0,0,0 */
+#define TERM_WHITE      1   /* 'w' */   /* 4,4,4 */
+#define TERM_SLATE      2   /* 's' */   /* 2,2,2 */
+#define TERM_ORANGE     3   /* 'o' */   /* 4,2,0 */
+#define TERM_RED        4   /* 'r' */   /* 3,0,0 */
+#define TERM_GREEN      5   /* 'g' */   /* 0,2,1 */
+#define TERM_BLUE       6   /* 'b' */   /* 0,0,4 */
+#define TERM_UMBER      7   /* 'u' */   /* 2,1,0 */
+#define TERM_L_DARK     8   /* 'D' */   /* 1,1,1 */
+#define TERM_L_WHITE    9   /* 'W' */   /* 3,3,3 */
+#define TERM_VIOLET     10  /* 'v' */   /* 4,0,4 */
+#define TERM_YELLOW     11  /* 'y' */   /* 4,4,0 */
+#define TERM_L_RED      12  /* 'R' */   /* 4,0,0 */
+#define TERM_L_GREEN    13  /* 'G' */   /* 0,4,0 */
+#define TERM_L_BLUE     14  /* 'B' */   /* 0,4,4 */
+#define TERM_L_UMBER    15  /* 'U' */   /* 3,2,1 */
+
+/*
+ * Maximum number of colours, and number of "basic" Angband colours
+ */ 
+#define MAX_COLORS		256
+#define BASIC_COLORS	16
+
+
+
 /**** Available Variables ****/
 
 extern term *Term;
@@ -298,15 +372,15 @@ extern errr Term_locate(int *x, int *y);
 extern errr Term_what(int x, int y, byte *a, char *c);
 
 extern errr Term_flush(void);
+extern errr Term_mousepress(int x, int y, char button);
 extern errr Term_keypress(int k);
 extern errr Term_key_push(int k);
-extern errr Term_inkey(char *ch, bool wait, bool take);
+extern errr Term_event_push(const event_type *ke);
+extern errr Term_inkey(event_type *ch, bool wait, bool take);
 
 extern errr Term_save(void);
 extern errr Term_load(void);
 
-extern errr Term_exchange(void);
-
 extern errr Term_resize(int w, int h);
 
 extern errr Term_activate(term *t);
@@ -318,3 +392,4 @@ extern errr term_init(term *t, int w, int h, int k);
 #endif
 
 
+
