@@ -1,20 +1,25 @@
-/* File: z-virt.c */
-
 /*
- * Copyright (c) 1997 Ben Harrison
+ * File: z-virt.c
+ * Purpose: Memory management routines
  *
- * This software may be copied and distributed for educational, research,
- * and not for profit purposes provided that this copyright and statement
- * are included in all such copies.
+ * Copyright (c) 1997 Ben Harrison.
+ *
+ * This work is free software; you can redistribute it and/or modify it
+ * under the terms of either:
+ *
+ * a) the GNU General Public License as published by the Free Software
+ *    Foundation, version 2, or
+ *
+ * b) the "Angband licence":
+ *    This software may be copied and distributed for educational, research,
+ *    and not for profit purposes provided that this copyright and statement
+ *    are included in all such copies.  Other copyrights may also apply.
  */
-
-/* Purpose: Memory management routines -BEN- */
-
 #include "z-virt.h"
-
 #include "z-util.h"
 
 
+
 /*
  * Optional auxiliary "rpanic" function
  */
@@ -24,15 +29,15 @@ void* (*rpanic_aux)(size_t) = NULL;
  * The system is out of memory, so panic.  If "rpanic_aux" is set,
  * it can be used to free up some memory and do a new "ralloc()",
  * or if not, it can be used to save things, clean up, and exit.
- * By default, this function simply crashes the computer.
+ * By default, this function simply quits the computer.
  */
 void* rpanic(size_t len)
 {
 	/* Hopefully, we have a real "panic" function */
 	if (rpanic_aux) return ((*rpanic_aux)(len));
 
-	/* Attempt to crash before icky things happen */
-	core("Out of Memory!");
+	/* Attempt to quit before icky things happen */
+	quit("Out of Memory!");
 
 	/* Paranoia */
 	return (NULL);
